(***************************************************************************)
(*                                                                         *)
(* -------------- SOURCE CODE OF THE PROGRAM PACKAGE CODICA -------------- *)
(*                                                                         *)
(*                          ALL RIGHTS RESERVED.                           *)
(*   THIS PACKAGE IS SUBJECTED TO COPYRIGHT. NO PART OF IT MAY BE USED,    *)
(*        COPIED OR REPRODUCED, IN ANY FORM, PHYSICAL OR DIGITAL,          *)
(*                           FOR COMMERCIAL USE.                           *)
(*                                                                         *)
(* ----------------------------------------------------------------------- *)
(*                                                                         *)
(* :$Id: Codica.m                                                          *)
(*       1st release: december 2002, Mathematica 3.0                       *)
(*       2nd release: february 2004, Mathematica 5.0                       *)
(*       3rd release: april 2005, Mathematica 5.0                          *)
(*                                                                         *)
(* :Codica: v 1.0 10.04.2000                                               *)
(*          v 1.1 13.09.2000                                               *)
(*          v 2.0 01.11.2000                                               *)
(*          v 2.1 13.11.2000                                               *)
(*          v 2.2 03.12.2001                                               *)
(*          v 2.3 21.10.2002                                               *)
(*          v 3.0 24.02.2004                                               *)
(*          v 4.0 01.02.2005                                               *)
(*          v 5.0 01.04.2005                                               *)
(*                                                                         *)
(* :Copyright: Copyright 2000-2005 by Ramon Egli. All rights reserved.     *)
(*             Actual adress: Dr. Ramon Egli                               *)
(*                            Institute for Rock Magnetism                 *)
(*                            291 Shepherd Laboratories                    *)
(*                            100 Union Street SE                          *)
(*                            Minneapolis, MN 55455-0128                   *)
(*                            U.S.A                                        *)
(*             E-mail: eglix007@umn.edu                                    *)
(*                                                                         *)
(* :Mathematica Version: 5.0 or later for Microsoft Windows                *)
(*                                                                         *)
(* :Title: Codica                                                          *)
(*                                                                         *)
(* :Programs: Codica (COercivity DIstribution CAlculator)                  *)
(*                                                                         *)
(* :Summary: This package performs robust noise filtering on sigmoidal     *)
(*           curves and  calculates the first derivative on different      *)
(*           scales.                                                       *)
(*                                                                         *)
(* Notice: To see the correct structure of the source code, wiew it always *)
(*         with the character format "Courier New". No tabulators are      *)
(*         used. Maximum width: 80 characters.                             *)
(*         Use 10 pt characters to print on A4 paper.                      *)
(*                                                                         *)
(***************************************************************************)


StylePrint[
"CODICA v.5.0 for Mathematica 5.0 and later versions.
\nDistributed with the package MAG-MIX release 1(B03), 04/04/2005.
\nCopyright 2005 by Ramon Egli. All rights reserved.",
           FontWeight->"Bold"];

BeginPackage["Utilities`Codica`"]

Codica::usage=
"CODICA v.5.0 is an interactive fitting program especially conceived
to fit and filter properly datasets which can be represented by curves with
two horizontal asymptots, and to estimate their first derivative. It reads
data from an ASCII file named \"filename\" and writes output ASCII files
for the fitted curve (extension: .cum), and the first derivative on a linear
(extension: .slin), logarithmic (extension: .slog) and power (extension: .spow)
scale.
\nThrough appropriate scale transformations of both the independent and the
dependent variable the curve represented by the input file is linearized. The
difference between the best-fitting line and the linearized curve is called
the residual curve. In the residual curve, the noise/signal ratio is maximized.
Odd points representing bad measurements can be easly recognized and removed on
the residual curve. The signal component of the residual curve is
approximatively sinusoidal, having therefore a narrow distribution of
frequencies in the Fourier space. The noise component is removed with a
least-squares collocation method, whose correlation length is chosen by the
user.
\nThe inverse transformation of the filtered residual curve is used to
reconvert the data into the original scale. An analytical first derivative of the
filtered data is used to calculate distribution curves on different scales of
the independent variable.
\nCODICA is particularly robust and efficient in removing white and non-white
noise. It provides an error estimation of the results, which is based on the
difference between the filtered data and the original data.
\n\nFormat of the input file (2 colums without comment lines, separated by
at least one space or one tabulator character):
\n    First column : independent variable,
\n    Second column: dependent variable.
\n\nOutput format for file \".cum\":
\n    First column : independent variable,
\n    Second column: fitted value,
\n    Third column : absolute error of the fitted value.
\n\nOutput format for files \".slin\", \".slog\", \".spow\":
\n    First column : independent variable,
\n    Second column: distribution value,
\n    Third column : relative error of the distribution value.
\n\nUse CODICA v.5.0 on Mathematica 5.0 or later. See manual for more details."


Begin["Utilities`Codica`Private`"]


InputCheck[string_,type_,limitations_]:=
Module[{bool,count,delta,DigitalQ,elements,grammar,in,
        in0,l,out,path,PathQ,pos},

Off[Syntax::com];
Off[Read::readn];
DigitalQ[x_]:=MatchQ[ReadList[StringToStream[x],Number],
                     {_?(NumberQ[#]&)}];
PathQ[path_]:=
If[StringLength[path]>3,
   If[Not[Intersection[Join[CharacterRange["a","z"],
                            CharacterRange["A","Z"]],
                       {First[Characters[path]]}]==={}],
      If[StringTake[path,{2,3}]===":/",
         If[Not[Last[Characters[path]]==="/"||
                Last[Characters[path]]==="\\"],
            True,
            False],
         False],
      False],
   False];
grammar=type[[1]];
elements=limitations[[1]];
bool=False;count=0;
While[bool===False,
count++;
If[count==1,
   in0=InputString[string]];
If[count<=Length[type],
   grammar=type[[count]];
   elements=limitations[[count]],
   count=1;
   in0=InputString[StringJoin["NONVALID INPUT.\n",string]];
   grammar=type[[1]];
   elements=limitations[[1]]];
If[in0==="abort",Abort[]];
in=in0;
Which[grammar==="Expression",
      Which[elements[[1]]===String,
            If[elements[[2]]==={},
               bool=True;out=in,
               If[MemberQ[elements[[2]],in],
                  bool=True;
                  out=in]],
            elements[[1]]===Integer,
            If[DigitalQ[in],
               in=ToExpression[in];
               If[Head[in]===Integer,
                  If[elements[[2,1]]<=in<=elements[[2,2]],
                     bool=True;
                     out=in]]],
            elements[[1]]===Real,
            If[DigitalQ[in],
               in=N[ToExpression[in]];
               If[Head[in]===Real,
                  If[elements[[2,1]]<=in<=elements[[2,2]],
                     bool=True;
                     out=in]]],
            elements[[1]]==="Real0",
            If[DigitalQ[in],
               in=N[ToExpression[in]];
               If[Head[in]===Real&&Not[in==0],
                  If[elements[[2,1]]<=in<=elements[[2,2]],
                     bool=True;
                     out=in]]],
            True,
            bool=False],
      grammar==="Path",
      If[PathQ[in]===True,
         bool=True;
         out=in,
         bool=False],
      grammar==="ShortList",
      in=StringJoin["{",in,"}"];
      l=StringLength[in];
      pos=StringPosition[in,","];
      If[StringPosition[in,"{"]=={{1,1}}&&
         StringPosition[in,"}"]=={{l,l}}&&
         StringPosition[in," "]=={}&&
         Length[pos]==Length[elements]-1,
         pos=Join[{1},
                  Table[pos[[k,1]],{k,1,Length[pos]}],
                  {l}];
         delta=Min[Table[pos[[k]]-pos[[k-1]],{k,2,Length[pos]}]];
         If[delta>1,
            in=Table[StringTake[in,{pos[[k-1]]+1,pos[[k]]-1}],
                     {k,2,Length[pos]}];
            bool={};
            For[i=1,i<=Length[in],i++,
                Which[elements[[i,1]]===String,
                      bool=Append[bool,
                                  If[elements[[i,2]]==={},
                                     True,
                                     MemberQ[elements[[i,2]],in[[i]]]]],
                      elements[[i,1]]===Integer,
                      If[DigitalQ[in[[i]]],
                         If[Head[ToExpression[in[[i]]]]===Integer,
                            bool=Append[bool,
                                        elements[[i,2,1]]<=
                                        ToExpression[in[[i]]]<=
                                        elements[[i,2,2]]],
                            bool=Append[bool,False]],
                         bool=Append[bool,False]],
                      elements[[i,1]]===Real,
                      If[DigitalQ[in[[i]]],
                         If[Head[N[ToExpression[in[[i]]]]]===Real,
                            bool=Append[bool,
                                        elements[[i,2,1]]<=
                                        ToExpression[in[[i]]]<=
                                        elements[[i,2,2]]],
                            bool=Append[bool,False]],
                         bool=Append[bool,False]],
                      elements[[i,1]]==="Real0",
                      If[DigitalQ[in[[i]]],
                         If[Head[N[ToExpression[in[[i]]]]]===Real,
                            bool=Append[bool,
                                        (elements[[i,2,1]]<=
                                         ToExpression[in[[i]]]<=
                                         elements[[i,2,2]])&&
                                         Not[ToExpression[in[[i]]]==0]],
                            bool=Append[bool,False]],
                         bool=Append[bool,False]],
                      True,
                      bool=Append[bool,False]]];
            If[FreeQ[bool,False],
               bool=True;out={};
               For[i=1,i<=Length[in],i++,
                   If[elements[[i,1]]===String,
                      out=Append[out,in[[i]]],
                      out=Append[out,ToExpression[in[[i]]]]]],
               bool=False]]],
      grammar==="LongList",
      in=StringJoin["{",in,"}"];
      Which[SyntaxQ[in]&&
            ToExpression[in]==={}&&elements[[1,1]]==0,
            bool=True;
            out={},
            
            elements[[2]]===String,
            l=StringLength[in];
            pos=StringPosition[in,","];
            pos=Join[{1},
                     Table[pos[[k,1]],{k,1,Length[pos]}],
                     {StringLength[in]}];
            If[elements[[1,1]]-1<=Length[pos]<=elements[[1,2]]-1,
               in=Table[StringTake[in,{pos[[k-1]]+1,pos[[k]]-1}],
                        {k,2,Length[pos]}];
               bool=Table[If[elements[[3]]==={},
                             True,
                             MemberQ[elements[[3]],in[[k]]]],
                          {k,1,Length[in]}];
               bool=Not[MemberQ[bool,False]];
               If[bool,out=in],
               bool=False],
            
            True,
            l=StringLength[in];
            pos=StringPosition[in,","];
            pos=Join[{1},
                     Table[pos[[k,1]],{k,1,Length[pos]}],
                     {StringLength[in]}];
            in=Table[StringTake[in,{pos[[k-1]]+1,pos[[k]]-1}],
                     {k,2,Length[pos]}];
            If[elements[[1,1]]<=Length[in]<=elements[[1,2]]&&
               MatchQ[in,{__?(DigitalQ[#]&)}],
               in=Table[ToExpression[in[[k]]],{k,1,Length[in]}];
               Which[elements[[2]]===Integer,
                     bool=MatchQ[in,
                                 {__?(IntegerQ[#]&&
                                      elements[[3,1]]<=#<=elements[[3,2]]&)}],
                     
                     elements[[2]]==="Ordinal",
                     bool=MatchQ[in,
                                 {__?(IntegerQ[#]&&
                                      elements[[3,1]]<=#<=elements[[3,2]]&&
                                      Not[#==0]&)}],
                     
                     elements[[2]]===Real,
                     bool=MatchQ[in,
                                 {__?((Head[#]===Real||Head[#]===Integer)&&
                                      elements[[3,1]]<=#<=elements[[3,2]]&)}],
                     
                     elements[[2]]==="Real0",
                     bool=MatchQ[in,
                                 {__?((Head[#]===Real||Head[#]===Integer)&&
                                      elements[[3,1]]<=#<=elements[[3,2]]&&
                                      Not[#==0]&)}],
                     
                     elements[[2]]==="RealD",
                     bool=MatchQ[in,
                                 {__?((Head[#]===Real||Head[#]===Integer)&&
                                      elements[[3,1]]<=#<=elements[[3,2]]&&
                                      Not[#==0]&)}]&&
                          Length[Intersection[in]]==Length[in],
                     True,
                     bool=True];
               If[bool,out=in],
               
               bool=False]
      ],
      
      
      True,
      bool=False]];
out
      ];


Prn[x_,n_]:=
StringJoin[ToString[SetPrecision[NumberForm[N[x],NumberFormat->(#1&)],n]],
           If[ToString[NumberForm[N[x],NumberFormat->(#3&)]]==="",
              "",
              "e"],
           ToString[NumberForm[N[x],NumberFormat->(#3&)]]];


PutNumbers[lst_,area_,step_]:=
Module[{a,b,l=Length[lst],numb,rng},

rng={Table[lst[[k,1]],{k,1,l}],
     Table[lst[[k,2]],{k,1,l}]};
rng={0.01*(area[[1,2]]-area[[1,1]]),
     0.01*(area[[2,2]]-area[[2,1]])};
numb=Table[{lst[[k]],
            Select[lst,(Min[lst[[Max[0,k-2],1]],lst[[k,1]]-rng[[1]]]<=#[[1]]<=
                        Max[lst[[Min[l,k+2],1]],lst[[k,1]]+rng[[1]]])&]},
           {k,step,l,step}];
Clear[a];Clear[b];
numb=Table[{numb[[k,1]],
            FindFit[numb[[k,2]],a+b*x,{a,b},x]},
           {k,1,Length[numb]}];
numb=Table[{numb[[k,1]],
            (Utilities`Codica`Private`b/.numb[[k,2,2]])*
            rng[[1]]*GoldenRatio/rng[[2]]},
           {k,1,Length[numb]}];
numb=Table[{numb[[k,2]],
            {numb[[k,1,1]]+rng[[1]]*numb[[k,2]]/Sqrt[1+numb[[k,2]]^2],
             numb[[k,1,2]]-rng[[2]]/Sqrt[1+numb[[k,2]]^2]}},
           {k,1,Length[numb]}];
numb=Table[Which[numb[[k,1]]<-1,
                 Text[StyleForm[ToString[step*k],
                                FontFamily->"Geneva",
                                FontColor->RGBColor[0,0,1]],
                      numb[[k,2]],
                      {1,0}],
                 numb[[k,1]]<1,
                 Text[StyleForm[ToString[step*k],
                                FontFamily->"Geneva",
                                FontColor->RGBColor[0,0,1]],
                      numb[[k,2]],
                      {0,1}],
                 True,
                 Text[StyleForm[ToString[step*k],
                                FontFamily->"Geneva",
                                FontColor->RGBColor[0,0,1]],
                      numb[[k,2]],
                      {-1,0}]],
           {k,1,Length[numb]}];
numb];


HypFit[list_,margin_,wts_,aq_,skew_]:=
Module[{amp,cen,off,para,res,scan,sat,sg,slp,wt},

Clear[a1];Clear[a2];Clear[a3];Clear[a4];Clear[a5];Clear[a6];
Clear[a7];Clear[a8];
If[aq,sg=1,sg=-1];
amp=margin[[1]]-margin[[2]];
sat=-Sign[amp]*0.02;
off=Min[margin[[1]],margin[[2]]];
cen=margin[[3]];
slp=2*Sign[amp]/(margin[[5]]-margin[[4]]);
amp=Abs[amp];
wt=wts;
Clear[a1];Clear[a2];Clear[a3];

para=FindFit[list,amp*(0.5000000001-0.5*Tanh[a1*(x-cen)])+off,
             {{a1,slp}},x,
             NormFunction->(Norm[#*wt]&),
             AccuracyGoal->Infinity];
para=Utilities`Codica`Private`a1/.para[[1]];

If[aq,
   para=FindFit[list,
                a3^2*(0.5000000001-0.5*Tanh[a1*(x-cen)])+off+
                a4^2*(x-cen+Log[Cosh[Abs[a1]*(x-cen)]]/
                      Abs[a1]+Log[2]/Abs[a1]),
                {{a1,para},
                 {a3,Sqrt[amp]},
                 {a4,Sqrt[Abs[sat*amp]]}},x,
                NormFunction->(Norm[#*wt]&),
                AccuracyGoal->Infinity];
   para={Utilities`Codica`Private`a1/.para[[1]],
         cen,
         Utilities`Codica`Private`a3/.para[[2]],
         Utilities`Codica`Private`a4/.para[[3]]};
   para=FindFit[list,
                a3^2*(0.5000000001-0.5*Tanh[a1*(x-a2)])+off+
                a4^2*(x-a2+Log[Cosh[Abs[a1]*(x-a2)]]/
                      Abs[a1]+Log[2]/Abs[a1]),
                {{a1,para[[1]]},
                 {a2,cen},
                 {a3,para[[3]]},
                 {a4,para[[4]]}},x,
                NormFunction->(Norm[#*wt]&),
                AccuracyGoal->Infinity];
   para={Utilities`Codica`Private`a1/.para[[1]],
         Utilities`Codica`Private`a2/.para[[2]],
         Utilities`Codica`Private`a3/.para[[3]],
         Utilities`Codica`Private`a4/.para[[4]]};
   para={para[[1]],para[[2]],para[[3]]^2,
         off,para[[4]]^2},
   
   para=FindFit[list,
                a3^2*(0.5000000001-0.5*Tanh[a1*(x-cen)])+margin[[1]]-a3^2-
                a4^2*(x-cen+Log[Cosh[Abs[a1]*(x-cen)]]/
                      Abs[a1]+Log[2]/Abs[a1]),
                {{a1,para},
                 {a3,Sqrt[amp-Abs[sat*amp]]},
                 {a4,Sqrt[Abs[sat*amp]]}},x,
                NormFunction->(Norm[#*wt]&),
                AccuracyGoal->Infinity];
   para={Utilities`Codica`Private`a1/.para[[1]],
         cen,
         Utilities`Codica`Private`a3/.para[[2]],
         Utilities`Codica`Private`a4/.para[[3]]};
   para=FindFit[list,
                a3^2*(0.5000000001-0.5*Tanh[a1*(x-a2)])+margin[[1]]-a3^2-
                a4^2*(x-a2+Log[Cosh[Abs[a1]*(x-a2)]]/Abs[a1]+Log[2]/Abs[a1]),
                {{a1,para[[1]]},
                 {a2,cen},
                 {a3,para[[3]]},
                 {a4,para[[4]]}},x,
                NormFunction->(Norm[#*wt]&),
                AccuracyGoal->Infinity];
   para={Utilities`Codica`Private`a1/.para[[1]],
         Utilities`Codica`Private`a2/.para[[2]],
         Utilities`Codica`Private`a3/.para[[3]],
         Utilities`Codica`Private`a4/.para[[4]]};
         
   para={para[[1]],para[[2]],para[[3]]^2,
         margin[[1]]-para[[3]]^2,-para[[4]]^2}];
   
   
If[skew,
   Print["Searching for an asymmetric best-fit hyperbolic model.
Please wait..."];
   res={};
   For[i=-1.2,i<=1,i=i+0.2,
       scan=FindFit[list,
                    a3*(0.5000000001-0.5*Tanh[ArcSinh[a1*(x-a2)]])^Exp[i]+a4+
                    a5^2*sg*
                    (x-a2+Log[Cosh[Abs[a1]*(x-a2)]]/Abs[a1]+Log[2]/Abs[a1]),
                    {{a1,para[[1]]},{a2,para[[2]]},
                     {a3,para[[3]]},{a4,para[[4]]},
                     {a5,Sqrt[Abs[para[[5]]]]}},x,
                    NormFunction->(Norm[#*wt]^2&),
                    AccuracyGoal->Infinity];
       scan={Utilities`Codica`Private`a1/.scan[[1]],
             Utilities`Codica`Private`a2/.scan[[2]],
             Utilities`Codica`Private`a3/.scan[[3]],
             Utilities`Codica`Private`a4/.scan[[4]],
             Utilities`Codica`Private`a5/.scan[[5]]};
       scan={Exp[i],
             scan,
             Sum[(scan[[3]]*(0.5000000001-
                  0.5*Tanh[ArcSinh[scan[[1]]*(list[[k,1]]-scan[[2]])]])^Exp[i]+
                  scan[[4]]+scan[[5]]^2*sg*
                  (list[[k,1]]-scan[[2]]+
                   Log[Cosh[Abs[scan[[1]]]*(list[[k,1]]-scan[[2]])]]/
                   Abs[scan[[1]]]+Log[2]/Abs[scan[[1]]])-
                  list[[k,2]])^2*wts[[k]],
                 {k,1,Length[list]}]};
       res=Append[res,scan]];
   res=First[Sort[res,#1[[3]]<#2[[3]]&]];
   
   para=FindFit[list,
                a3*(0.5000000001-0.5*Tanh[ArcSinh[a1*(x-a2)]])^a5+a4+
                a6^2*sg*
                (x-a2+Log[Cosh[Abs[a1]*(x-a2)]]/Abs[a1]+Log[2]/Abs[a1]),
                {{a1,res[[2,1]]},{a2,res[[2,2]]},
                 {a3,res[[2,3]]},{a4,res[[2,4]]},
                 {a5,res[[1]]},{a6,res[[2,5]]}},x,
                NormFunction->(Norm[#*wt]^2&),
                AccuracyGoal->Infinity];
   para={Utilities`Codica`Private`a1/.para[[1]],
         Utilities`Codica`Private`a2/.para[[2]],
         Utilities`Codica`Private`a3/.para[[3]],
         Utilities`Codica`Private`a4/.para[[4]],
         Utilities`Codica`Private`a5/.para[[5]],
         Utilities`Codica`Private`a6/.para[[6]]};
   para=FindFit[list,
                a3*(0.5000000001-0.5*Tanh[a5*ArcSinh[a1*(x-a2)/a5]])^a6+a4+
                a7^2*sg*
                (x-a2+Log[Cosh[Abs[a1]*(x-a2)]]/Abs[a1]+Log[2]/Abs[a1]),
                {{a1,para[[1]]},{a2,para[[2]]},{a3,para[[3]]},
                 {a4,para[[4]]},{a5,2},
                 {a6,para[[5]]},{a7,para[[6]]}},x,
                NormFunction->(Norm[#*wt]^2&),
                AccuracyGoal->Infinity];
   para={Utilities`Codica`Private`a1/.para[[1]],
         Utilities`Codica`Private`a2/.para[[2]],
         Utilities`Codica`Private`a3/.para[[3]],
         Utilities`Codica`Private`a4/.para[[4]],
         Utilities`Codica`Private`a5/.para[[5]],
         Utilities`Codica`Private`a6/.para[[6]],
         Utilities`Codica`Private`a7/.para[[7]]};
   para=FindFit[list,
                a3*(0.5000000001-0.5*Tanh[a5*ArcSinh[a1*(x-a2)/a5]])^a6+a4+
                a7^2*sg*
                (x-a2+Log[Cosh[Abs[a8]*(x-a2)]]/Abs[a8]+Log[2]/Abs[a8]),
                {{a1,para[[1]]},{a2,para[[2]]},{a3,para[[3]]},
                 {a4,para[[4]]},{a5,para[[5]]},
                 {a6,para[[6]]},{a7,para[[7]]},{a8,para[[1]]}},x,
                NormFunction->(Norm[#*wt]^2&),
                AccuracyGoal->Infinity];
   para={Utilities`Codica`Private`a1/.para[[1]],
         Utilities`Codica`Private`a2/.para[[2]],
         Utilities`Codica`Private`a3/.para[[3]],
         Utilities`Codica`Private`a4/.para[[4]],
         Utilities`Codica`Private`a5/.para[[5]],
         Utilities`Codica`Private`a6/.para[[6]],
         (Utilities`Codica`Private`a7/.para[[7]])^2*sg,
         Utilities`Codica`Private`a8/.para[[8]]}];
para];


HypLin[list_,prs_,rng_,wt_]:=
Module[{l=Length[list],cnt,out,outn,rngn},

Clear[a1];Clear[a2];Clear[a3];Clear[a4];Clear[a5];Clear[a6];
cnt=1;
Print["   Iteration #1..."];
out=
NMinimize[{Sum[wt[[k]]*Abs[prs[[5]]*
               Sinh[ArcTanh[1.0000000002-
               2*((list[[k,2]]-a2)/(a1-a2))^(1/prs[[6]])]/prs[[5]]]/
               prs[[1]]+prs[[2]]-list[[k,1]]]^2,
               {k,1,l}],
           a1>rng[[1,1]],
           a2<rng[[2,1]]},
          {{a1,rng[[1,1]],rng[[1,2]]},
           {a2,rng[[2,2]],rng[[2,1]]}},
          AccuracyGoal->Infinity][[2]];
out={Utilities`Codica`Private`a1/.out[[1]],
     Utilities`Codica`Private`a2/.out[[2]]};

rngn=rng;
While[out[[1]]>rngn[[1,2]]||out[[2]]<rngn[[2,2]],
rngn={{rng[[1,1]],Max[rngn[[1,2]],2*out[[1]]-rngn[[1,2]]]},
      {rng[[2,1]],Min[rngn[[2,2]],2*out[[2]]-rngn[[2,2]]]}};
cnt++;
Print[StringJoin["   Iteration #",ToString[cnt],"..."]];
out=
NMinimize[{Sum[wt[[k]]*Abs[prs[[5]]*
               Sinh[ArcTanh[1.0000000002-
               2*((list[[k,2]]-a2)/(a1-a2))^(1/prs[[6]])]/prs[[5]]]/
               prs[[1]]+prs[[2]]-list[[k,1]]]^2,
               {k,1,l}],
           a1>rng[[1,1]],
           a2<rng[[2,1]]},
          {{a1,rngn[[1,1]],rngn[[1,2]]},
           {a2,rngn[[2,2]],rngn[[2,1]]}},
          AccuracyGoal->Infinity][[2]];
out={Utilities`Codica`Private`a1/.out[[1]],
     Utilities`Codica`Private`a2/.out[[2]]}
     ]; (* Close While *)

cnt++;
Print[StringJoin["   Iteration #",ToString[cnt],"..."]];
rngn={{0.7*rng[[1,1]]+0.3*out[[1]],
       Max[rngn[[1,2]],1.5*out[[1]]-0.5*rngn[[1,2]]]},
      {0.7*rng[[1,1]]+0.3*out[[1]],
       Min[rngn[[2,2]],1.5*out[[2]]-0.5*rngn[[2,2]]]}};
outn=
TimeConstrained[
NMinimize[{Sum[wt[[k]]*Abs[prs[[5]]*
               Sinh[ArcTanh[1.0000000002-
               2*((list[[k,2]]-a2)/(a1-a2))^(1/prs[[6]])]/prs[[5]]]/a3+a4-
               list[[k,1]]]^2,
               {k,1,l}],
           a1>rng[[1,1]],
           a2<rng[[2,1]]},
          {{a1,rngn[[1,1]],rngn[[1,2]]},
           {a2,rngn[[2,2]],rngn[[2,1]]},
           Join[{a3},Sort[{0.5*prs[[1]],1.5*prs[[1]]}]],
           Join[{a4},Sort[{0.5*prs[[2]],1.5*prs[[2]]}]]},
          AccuracyGoal->Infinity],
                20];
If[outn===$Aborted,
   StylePrint["WARNING! Data are noisy. Precision goal has been reduced.",
              FontColor->RGBColor[1,0,0]];
   out={out[[1]],
        out[[2]],
        prs[[1]],
        prs[[2]],
        prs[[6]],
        prs[[5]]},
   out={Utilities`Codica`Private`a1/.outn[[2,1]],
        Utilities`Codica`Private`a2/.outn[[2,2]],
        Utilities`Codica`Private`a3/.outn[[2,3]],
        Utilities`Codica`Private`a4/.outn[[2,4]],
        prs[[6]],
        prs[[5]]}];

If[Not[outn===$Aborted],
cnt++;
Print[StringJoin["   Iteration #",ToString[cnt],"..."]];
rngn={{0.7*rng[[1,1]]+0.3*out[[1]],
       Max[rngn[[1,2]],1.5*out[[1]]-0.5*rngn[[1,2]]]},
      {0.7*rng[[1,1]]+0.3*out[[1]],
       Min[rngn[[2,2]],1.5*out[[2]]-0.5*rngn[[2,2]]]}};
outn=
TimeConstrained[
NMinimize[{Sum[wt[[k]]*Abs[a6*
               Sinh[ArcTanh[1.0000000002-
               2*((list[[k,2]]-a2)/(a1-a2))^(1/a5)]/a6]/a3+a4-
               list[[k,1]]]^2,
               {k,1,l}],
           a1>rng[[1,1]],
           a2<rng[[2,1]]},
          {{a1,rngn[[1,1]],rngn[[1,2]]},
           {a2,rngn[[2,2]],rngn[[2,1]]},
           Join[{a3},Sort[{0.8*out[[3]],1.2*out[[3]]}]],
           Join[{a4},Sort[{0.8*out[[4]],1.2*out[[4]]}]],
           Join[{a5},Sort[{0.5*out[[5]],1.5*out[[5]]}]],
           Join[{a6},Sort[{0.5*out[[6]],1.5*out[[6]]}]]},
          AccuracyGoal->Infinity],
                20];

If[outn===$Aborted,
   StylePrint["WARNING! Data are noisy. Precision goal has been reduced.",
              FontColor->RGBColor[1,0,0]],
   out={Utilities`Codica`Private`a1/.outn[[2,1]],
        Utilities`Codica`Private`a2/.outn[[2,2]],
        Utilities`Codica`Private`a3/.outn[[2,3]],
        Utilities`Codica`Private`a4/.outn[[2,4]],
        Utilities`Codica`Private`a5/.outn[[2,5]],
        Utilities`Codica`Private`a6/.outn[[2,6]]}];
  ]; (* Close If not *)

out={out[[3]],out[[4]],out[[1]]-out[[2]],out[[2]],out[[6]],out[[5]]};
out];


FindExtremes[list_,errors_]:=
Module[{delta,extr,max,min,n},

min=Table[If[(list[[k-2,2]]>list[[k-1,2]]>list[[k,2]])&&
             (list[[k+2,2]]>list[[k+1,2]]>list[[k,2]]),
             k,0],
          {k,3,Length[list]-2}];
min=Select[min,#>0&];
max=Table[If[(list[[k-2,2]]<list[[k-1,2]]<list[[k,2]])&&
             (list[[k+2,2]]<list[[k+1,2]]<list[[k,2]]),
             k,0],
          {k,3,Length[list]-2}];
max=Select[max,#>0&];
extr=Sort[Join[min,max]];
If[Length[extr]<2,
   n=Length[extr],
   delta=Table[Abs[list[[extr[[k+1]],2]]-list[[extr[[k]],2]]]-
               2*Max[errors[[extr[[k+1]]]],errors[[extr[[k]]]]],
               {k,1,Length[extr]-1}];
   delta=Select[delta,#<0&];
   n=Length[extr]-Length[delta]];

Max[0,n]+1
      ];


AutoCorrelation[data_,comment_]:=
Module[{n=Length[data],cd,cl,cll,cr,ex,fou,ft,ftt,lst,
        m,npk,out,pks,qq,qqs,res,rn,sds},

Clear[a1];Clear[a2];Clear[a3];Clear[a4];
rn=2*(Last[data][[1]]-First[data][[1]]);
fou=Table[{k,
           Abs[Sum[Exp[2*Pi*I*k*data[[j,1]]]*data[[j,2]]*
                   (data[[j+1,1]]-data[[j-1,1]]),
                   {j,2,n-1}]]},
          {k,0,n/rn,1/rn}];
cll=1/Sort[fou,#1[[2]]>#2[[2]]&][[1,1]];

If[comment,
   Print["Estimate the autocorrelation function of the residuals.
Please wait..."]];
ex=Table[Exp[k],{k,-1.6,1.4,0.1}];
pks={};qqs={};sds={};
For[i=1,i<=Length[ex],i++,
    lst=Table[{(data[[k,1]]+data[[2,1]]-data[[1,1]])^ex[[i]],
               data[[k,2]]},
              {k,1,n}];
    rn=Last[lst][[1]]-First[lst][[1]];
    cl=cll/(Last[data][[1]]-First[data][[1]])*rn;
    fou=Table[{k,
               Abs[Sum[Exp[2*Pi*I*k*lst[[j,1]]]*lst[[j,2]]*
                       (lst[[j+1,1]]-lst[[j-1,1]]),
                       {j,2,n-1}]]/rn},
               {k,0,2/cl,1/cl/50}];
    m=Sum[fou[[j,2]]^2*Log[10,fou[[j,1]]],{j,2,Length[fou]}]/
      Sum[fou[[j,2]]^2,{j,2,Length[fou]}];
    sds=Append[sds,
               Sqrt[Sum[fou[[j,2]]^2*(Log[10,fou[[j,1]]]-m)^2,
                        {j,2,Length[fou]}]/
                    Sum[fou[[j,2]]^2,
                        {j,2,Length[fou]}]]];
    qq=Sqrt[Sum[fou[[k,2]]^4,{k,1,Length[fou]}]]/
       Sum[fou[[k,2]]^2,{k,1,Length[fou]}];
    qqs=Append[qqs,qq];
    pk=Sort[fou,#1[[2]]>#2[[2]]&];
    pks=Append[pks,pk[[1,2]]]];
res={ex[[Last[Ordering[pks]]]],
     ex[[Last[Ordering[qqs]]]],
     ex[[First[Ordering[sds]]]]};

If[Max[res]/Min[res]<1.4,

ex=Log[ex[[Last[Ordering[qqs/sds]]]]];
ex=Table[Exp[k],{k,ex-0.1,ex+0.1,0.02}];
pks={};qqs={};sds={};
For[i=1,i<=Length[ex],i++,
    lst=Table[{(data[[k,1]]+data[[2,1]]-data[[1,1]])^ex[[i]],
               data[[k,2]]},
              {k,1,n}];
    rn=Last[lst][[1]]-First[lst][[1]];
    cl=cll/(Last[data][[1]]-First[data][[1]])*rn;
    fou=Table[{k,
               Abs[Sum[Exp[2*Pi*I*k*lst[[j,1]]]*lst[[j,2]]*
                       (lst[[j+1,1]]-lst[[j-1,1]]),
                       {j,2,n-1}]]/rn},
               {k,0,2/cl,1/cl/50}];
    m=Sum[fou[[j,2]]^2*Log[10,fou[[j,1]]],{j,2,Length[fou]}]/
      Sum[fou[[j,2]]^2,{j,2,Length[fou]}];
    sds=Append[sds,
               Sqrt[Sum[fou[[j,2]]^2*(Log[10,fou[[j,1]]]-m)^2,
                        {j,2,Length[fou]}]/
                    Sum[fou[[j,2]]^2,
                        {j,2,Length[fou]}]]];
    qq=Sqrt[Sum[fou[[k,2]]^4,{k,1,Length[fou]}]]/
       Sum[fou[[k,2]]^2,{k,1,Length[fou]}];
    qqs=Append[qqs,qq];
    pk=Sort[fou,#1[[2]]>#2[[2]]&];
    pks=Append[pks,pk[[1,2]]]];
res={ex[[Last[Ordering[pks]]]],
     ex[[Last[Ordering[qqs]]]],
     ex[[First[Ordering[sds]]]]};
If[Max[res]/Min[res]<1.4,
   ex=ex[[Last[Ordering[qqs/sds]]]],
   ex=1],

ex=1
   ]; (* Close If Max *)

lst=Table[{(data[[k,1]]+data[[2,1]]-data[[1,1]])^ex,data[[k,2]]},
          {k,1,n}];
rn=2*(Last[lst][[1]]-First[lst][[1]]);
fou=Table[{k,
           Abs[Sum[Exp[2*Pi*I*k*lst[[j,1]]]*lst[[j,2]]*
                   (lst[[j+1,1]]-lst[[j-1,1]]),
                   {j,2,n-1}]]},
          {k,0,n/rn,0.5/rn}];
cl=0.25/Sort[fou,#1[[2]]>#2[[2]]&][[1,1]];
cr=Table[{k,
          Sum[fou[[j,2]]^2*Cos[2*Pi*fou[[j,1]]*k],
              {j,1,Length[fou]}]},
         {k,0,rn/4,rn/200}];
 
ftt=FindFit[Drop[cr,1],
            a1^2*Exp[-x^2/a2^2]*Cos[2*Pi*a3*x],
            {{a1,Sqrt[Abs[cr[[2,2]]]]},
             {a2,5*cl},
             {a3,0.25/cl}},
            x];
ftt={Abs[Utilities`Codica`Private`a1/.ftt[[1]]],
     Abs[Utilities`Codica`Private`a2/.ftt[[2]]],
     Utilities`Codica`Private`a3/.ftt[[3]]};

ft=FindFit[cr,
           a1^2*Exp[-x^2/a2^2]*Cos[2*Pi*ftt[[3]]*x]+
           a3^2*Exp[-x^2/a4^2],
           {{a1,ftt[[1]]},
            {a2,ftt[[2]]},
            {a3,Max[cr[[1,2]]-ftt[[1]]^2,0.1*ftt[[1]]^2]},
            {a4,0.2*ftt[[2]]}},
           x];
ft={Abs[Utilities`Codica`Private`a1/.ft[[1]]],
    Abs[Utilities`Codica`Private`a2/.ft[[2]]],
    ftt[[3]],
    Abs[Utilities`Codica`Private`a3/.ft[[3]]],
    Abs[Utilities`Codica`Private`a4/.ft[[4]]]};

ft=FindFit[cr,
           a1^2*Exp[-x^2/a2^2]*Cos[2*Pi*a3*x]+
           a4^2*Exp[-x^2/a5^2],
           {{a1,ft[[1]]},
            {a2,ft[[2]]},
            {a3,ft[[3]]},
            {a4,ft[[4]]},
            {a5,ft[[5]]}},
           x];
ft={(Utilities`Codica`Private`a1/.ft[[1]])^2,
    Abs[Utilities`Codica`Private`a2/.ft[[2]]],
    Utilities`Codica`Private`a3/.ft[[3]],
    (Utilities`Codica`Private`a4/.ft[[4]])^2,
    Abs[Utilities`Codica`Private`a5/.ft[[5]]]};

cl=Max[8*(ft[[1]]/cr[[1,2]])^2*rn/n,
       Min[ft[[2]],0.25/ft[[3]]]];
cd=Min[ft[[2]],0.25/ft[[3]]];

Which[ft[[5]]>=ft[[2]]||
      0.25/ft[[3]]<Min[ft[[2]],ft[[5]]],
      If[comment,
         StylePrint["WARNING! CODICA could not estimate the autocorrelation
function of the residuals. Residuals are probably random.
\nA random Gaussian model will be used instead.",
                    FontColor->RGBColor[1,0,0]]];
      out={ex,{0,0,0,ft[[1]]+ft[[4]]}},
      ft[[4]]/(ft[[1]]+ft[[4]])>0.8,
      If[comment,
         StylePrint["WARNING! Residuals are almost random.",
                    FontColor->RGBColor[1,0,0]]];
      out={ex,{ft[[2]],ft[[3]],ft[[1]],ft[[4]]}},
      True,
      out={ex,{ft[[2]],ft[[3]],ft[[1]],ft[[4]]}};
      If[comment,
         StylePrint[StringJoin["Estimated signal-to-noise ratio   :",
                               Prn[ft[[4]]/(ft[[1]]+ft[[4]]),3]],
                    FontColor->RGBColor[0,0,1]]]];
out
      ];


AutoCollocation[data_,cpar_,cd_,rng_,dx_]:=
Module[{n=Length[data],an,cnn,cso,css,f,lst,m,rs,so,ss,v},

lst=Table[data[[k,2]],{k,1,n}];
v=Sum[data[[k,2]]^2,{k,1,n}]/n;
an=Max[0.0001,cpar[[4]]/(cpar[[3]]+cpar[[4]])];
If[cpar[[1]]>cd&&
   an<0.8,
   f[x_]:=Exp[-Min[100,x^2/cpar[[1]]^2]]*Cos[2*Pi*cpar[[2]]*x],
   f[x_]:=Exp[-Min[100,x^2/cd^2]]];

css=Table[Table[f[data[[k,1]]-data[[j,1]]],
                {j,1,n}],
          {k,1,n}];
cso=Table[Table[f[k-data[[j,1]]],
                {j,1,n}],
          {k,rng[[1]],rng[[2]]+0.1*dx,dx}];
cnn=Table[Table[If[k==j,1,0],
                {j,1,n}],
          {k,1,n}];

ss=Dot[Dot[css,Inverse[css+an*cnn]],lst];
rs=Norm[ss-lst]^2/n/v;
ss=Dot[Dot[css,Inverse[css+rs*cnn]],lst];
rs=Norm[ss-lst]^2/n/v;

ss=Dot[Dot[css,Inverse[css+rs*cnn]],lst];
so=Dot[Dot[cso,Inverse[css+rs*cnn]],lst];
m=Table[k,{k,rng[[1]],rng[[2]]+0.1*dx,dx}];
so=Table[{m[[k]],so[[k]]},{k,1,Length[so]}];

ss=Table[{data[[k,1]],ss[[k]]},{k,1,n}];
{ss,so}
      ];


Collocation[data_,fit_,cl_,rng_,dx_,xlst_]:=
Module[{n=Length[data],an,cnn,cnnf,cnnr,coo,cso,css,delta,err,f,
        ft,g,lst,m,nn,no,rs,so,sof,sor,ss,ssf,ssr,v},

v=Sum[data[[k,2]]^2,{k,1,n}]/n;
ft=Table[fit[[k,2]],{k,1,n}];
rs=Table[data[[k,2]]-fit[[k,2]],{k,1,n}];
an=Norm[rs]^2/n/v;
lst=Table[data[[k,2]],{k,1,n}];

f[x_]:=Exp[-Min[100,x^2/cl^2]];
g[x_]:=Exp[-Min[100,x^2/(2*cl)^2]];
m=Table[k,{k,rng[[1]],rng[[2]]+0.1*dx,dx}];

css=Table[Table[f[data[[k,1]]-data[[j,1]]],
                {j,1,n}],
          {k,1,n}];
cso=Table[Table[f[k-data[[j,1]]],
                {j,1,n}],
          {k,rng[[1]],rng[[2]]+0.1*dx,dx}];
cnn=Table[Table[If[k==j,1,0],
                {j,1,n}],
          {k,1,n}];
cnnf=an*cnn;
cnnr=0.1*cnn;
ssf=Dot[Dot[css,Inverse[css+cnnf]],ft];
ssr=Dot[Dot[css,Inverse[css+cnnr]],rs];

Print["Estimate the measurement errors..."];
delta=Abs[ssr+ssf-lst];

err=Table[Sum[delta[[j]]*g[data[[j]]-data[[k]]],{j,1,n}]/
          Sum[g[data[[j]]-data[[k]]],{j,1,n}],
          {k,1,n}];
err=Pi*err/2;

no=Table[Sum[delta[[j]]*g[data[[j]]-m[[k]]],{j,1,n}]/
         Sum[g[data[[j]]-m[[k]]],{j,1,n}],
         {k,1,Length[m]}];
no=Pi*no/2;

nn=Table[err[[k]]^2+Max[0,delta[[k]]-err[[k]]]^2,
         {k,1,n}];

cnnr=Table[Table[If[k==j,nn[[k]],0],
                 {j,1,n}],
           {k,1,n}];

sof=Dot[Dot[cso,Inverse[css+cnnf]],ft];
sor=Dot[Dot[v*cso,Inverse[v*css+cnnr]],rs];
so=Table[{m[[k]],sof[[k]]+sor[[k]]},{k,1,Length[m]}];

cso=Table[Table[f[xlst[[k]]-data[[j,1]]],
                {j,1,n}],
          {k,1,Length[xlst]}];

ssf=Dot[Dot[cso,Inverse[css+cnnf]],ft];
ssr=Dot[Dot[v*cso,Inverse[v*css+cnnr]],rs];
ss=Table[{xlst[[k]],ssf[[k]]+ssr[[k]]},{k,1,Length[xlst]}];

coo=v*Table[1,{k,1,Length[xlst]}];
err=Dot[Dot[v*cso,Inverse[v*css+cnnr]],Transpose[v*cso]];
err=Table[err[[k,k]],{k,1,Length[xlst]}];
err=Sqrt[coo-err];

{ss,so,2*err,nn,no}
      ];


DCollocation[data_,fit_,cl_,cd_,pts_,nn_]:=
Module[{n=Length[data],m=Length[pts],an,cnnf,cnnr,cso,css,df,do,dof,
        doo,dor,dso,f,ft,lst,no,rs,so,sof,sor,v},

v=Sum[data[[k,2]]^2,{k,1,n}]/n;
ft=Table[fit[[k,2]],{k,1,n}];
rs=Table[data[[k,2]]-fit[[k,2]],{k,1,n}];
an=Norm[rs]^2/n/v;
lst=Table[data[[k,2]],{k,1,n}];

f[x_]:=Exp[-Min[100,x^2/cl^2]];
df[x_]:=-Exp[-Min[100,x^2/cl^2]]*2*x/cl^2;

css=Table[Table[f[data[[k,1]]-data[[j,1]]],
                {j,1,n}],
          {k,1,n}];
cso=Table[Table[f[pts[[k]]-data[[j,1]]],
                {j,1,n}],
          {k,1,m}];
dso=Table[Table[df[pts[[k]]-data[[j,1]]],
                {j,1,n}],
          {k,1,m}];
cnnf=an*Table[Table[If[k==j,1,0],
                    {j,1,n}],
              {k,1,n}];
cnnr=Table[Table[If[k==j,nn[[k]],0],
                 {j,1,n}],
           {k,1,n}];

sof=Dot[Dot[cso,Inverse[css+cnnf]],ft];
sor=Dot[Dot[v*cso,Inverse[v*css+cnnr]],rs];
so=Table[{pts[[k]],sof[[k]]+sor[[k]]},{k,1,m}];

dof=Dot[Dot[dso,Inverse[css+cnnf]],ft];
dor=Dot[Dot[v*dso,Inverse[v*css+cnnr]],rs];
do=Table[{pts[[k]],dof[[k]]+dor[[k]]},{k,1,m}];

f[x_]:=Exp[-Min[100,x^2/cd^2]];
df[x_]:=-Exp[-Min[100,x^2/cd^2]]*2*x/cd^2;
css=Table[Table[f[data[[k,1]]-data[[j,1]]],
                {j,1,n}],
          {k,1,n}];
dso=Table[Table[df[pts[[k]]-data[[j,1]]],
                {j,1,n}],
          {k,1,m}];

doo=2*v*Table[1,{k,1,m}]/cd^2;
no=Dot[Dot[v*dso,Inverse[v*css+cnnr]],Transpose[v*dso]];
no=Table[no[[k,k]],{k,1,m}];
no=Sqrt[doo-no];

{so,do,2*no}
      ];


AddDigits[number_,digits_]:=
Module[{f,num},
num=ToString[number];
If[IntegerQ[number],
   num=StringJoin[num,"."]];
f=FractionalPart[number];
If[f<digits,
   num=StringJoin[num,
                  StringJoin[Table["0",{k,1,f-digits}]]]];
num
      ];


LinearTicks[range_,scale_]:=
Module[{delta,min,ndelta,order,rng,sgn,ticks,zero},

delta=0.2*(range[[2]]-range[[1]]);
order=10^Floor[Log[10,delta]];
ndelta=delta/order;
Which[ndelta<Sqrt[2],delta=order,
      ndelta<Sqrt[10],delta=2*order,
      ndelta<Sqrt[50],delta=5*order,
      True,order=10*order;delta=order];
Which[range[[1]]<=0<=range[[2]],sgn=0,
      range[[1]]>0,sgn=1,
      True,sgn=-1];
min=Min[Abs[range[[1]]],Abs[range[[2]]]];
min=delta*Floor[min/ndelta];
If[sgn==0,        
   zero=0,
   zero=sgn*min];
zero=zero*order;
rng={Floor[(range[[1]]-zero)/delta],
     Floor[(range[[2]]-zero)/delta]};
ticks=Table[zero+k*delta,{k,rng[[1]],rng[[2]]}];
min=Table[Abs[ticks[[k]]],{k,1,Length[ticks]}];
Which[scale==="MinimizedLabels",
      min=Max[delta,Min[min]];
      order=10^Floor[Log[10,min]],
      scale==="Engeenering",
      min=Max[min];
      order=10^(3*Floor[Log[10,min]/3]),
      scale==="Original",
      min=Max[min];
      order=1,
      True,
      min=Max[min];
      order=10^Floor[Log[10,min]]/10];
ticks=Table[{ticks[[k]],ticks[[k]]/order},
            {k,1,Length[ticks]}];
min=Table[StringLength[ToString[FractionalPart[N[ticks[[k,2]]]]]],
          {k,1,Length[ticks]}];
min=Max[min]-2;
If[min==0,
   ticks=Table[{ticks[[k,1]],ticks[[k,2]]},
               {k,1,Length[ticks]}],
   ticks=Table[{ticks[[k,1]],
                AddDigits[N[ticks[[k,2]]],min]},
               {k,1,Length[ticks]}]];
ticks=Table[{ticks[[k,1]],ticks[[k,2]],
             {0,0.016},{Thickness[0.002]}},
            {k,1,Length[ticks]}];
ticks=Join[ticks,
           Table[{zero+0.2*k*delta,"",
                  {0,0.01},{Thickness[0.002]}},
                 {k,Min[10*rng[[1]],0],
                    Max[10*rng[[2]],0]}]];
{ticks,order}
      ];


LogTicks[range_]:=
Module[{delta,ndelta,order,rng,ticks},

If[range[[2]]-range[[1]]<1,
   delta=(10^range[[2]]-10^range[[1]])*0.2;
   order=10^Floor[Log[10,delta]];
   ndelta=delta/order;
   Which[ndelta<Sqrt[2],delta=order,
         ndelta<Sqrt[10],delta=2*order,
         ndelta<Sqrt[50],delta=5*order,
         True,order=10*order;delta=order];
   rng={Ceiling[10^range[[1]]/delta],
        Floor[10^range[[2]]/delta]};
   ticks=Join[Table[{Log[10,k*delta],N[k*delta],
                     {0,0.016},{Thickness[0.002]}},
                    {k,rng[[1]],rng[[2]]}],
              Table[{Log[10,0.2*k*delta],"",
                     {0,0.01},{Thickness[0.002]}},
                    {k,rng[[1]],5*rng[[2]]+5}]],
   
   rng={Floor[range[[1]]],Ceiling[range[[2]]]};
   ticks={};
   For[i=rng[[1]],i<=rng[[2]],i++,
       ticks=Join[ticks,
                  Join[{{i,10^ToString[i],
                        {0,0.016},{Thickness[0.002]}}},
                       Table[{i+Log[10,k],"",{0,0.01},{Thickness[0.002]}},
                             {k,2,9}]]]]];
ticks];


PowerTicks[range_,expn_,lin_]:=
Module[{rng,ticks,ord},
If[lin,
   rng={range[[1]]^(1/expn),range[[2]]^(1/expn)};
   ticks=LinearTicks[rng,"Original"];
   ord=ticks[[2]];
   ticks=ticks[[1]];
   ticks=Table[{ticks[[k,1]]^expn,ticks[[k,2]],
                ticks[[k,3]],ticks[[k,4]]},
               {k,1,Length[ticks]}];
   ticks={ticks,ord},
   rng={Floor[Log[10,range[[1]]^(1/expn)]],
        Floor[Log[10,range[[2]]^(1/expn)]]};
   ticks={};
   For[i=rng[[1]],i<=rng[[2]],i++,
       ticks=Join[ticks,
                  Join[{{10^(i*expn),10^ToString[i],
                        {0,0.016},{Thickness[0.002]}}},
                       Table[{(k*10^i)^expn,"",{0,0.01},{Thickness[0.002]}},
                             {k,2,9}]]]]];
ticks];


Codica:=
Module[

{asym1,asym2,bool,bthf,cd,cent,choose,cl,cll,coll,count,
dbthf,ddbthf,ddthf,deel,delta,deltadig,deltamin,derlg,derlgerr,derlgh,derlgl,
derlglogerr,derlin,derlinerr,derln,derlnerr,derlnh,derlnl,derlnlogerr,
derlog,derlogerr,derpow,derpowerr,derpw,derpwerr,derpwh,derpwl,derpwlogerr,
derr,derrl,digleft,digright,dim,dir,dithf,double, 
dpoly,dres,dresl,dthf,duns,dx,ee3,ee5,eel,err,err01,err02,err03h,err03l,
err05,err05f,err05h,err05l,err1,err2,err3,errrng,expow,expn,exrng,ferr,
fit,fiterr,fiterrf,fitf,fitlogerr,fitlogerrf,flex,frng,ft,
ithf,l0,l1,l2,l3,labels,linear,linrng,list,logrng,lrng,ls,lst,
maq,med,mono,name,nmatrix,nout,null,numbers,opt,ord,orng,out,poly,power,
powrng,ptexpn,
q,qr,quant1,quant2,ref,removed,rempoints,res,res01,res02,res03,res03blue,
res04,res04i,res05,res05f,res06,res1,res2,res3,res4,resl,resnull,resrng,
rsexpn,scfail,sig,sol,spacing,sug,syexpn,thf,uns,unsp,verr,vrng,weights,
x0,x0r,xrng,xticks,y,yrng,yticks},


(* List of all Codica variables:
asym1        = left asymptote
asym2        = right asymptote
bool         = boolean variable or help variable
bthf         = best thanh-fit for display
cd           = correlation length entered by the user
cent         = index for the position of the median field
choose       = boolean variable for choosing a field scale
cl           = estimated autocorrelation distance of the residuals
cll          = list of parameters that define the autocorrelation function
coll         = boolean variable for filtering the residuals with collocation
count        = index variable or help variable
dbthf        = first derivative of bthf
ddbthf       = second derivative of bthf
ddthf        = second derivative of thf
deel         = first derivative of eel
delta        = help variable for a difference
deltadig     = list of jumps produced by digital truncation effects
deltamin     = ninimum gap in the field values
derlg        = coercivity distribution on a user-defined log scale
derlgerr     = error of derlg
derlgh       = upper confidence limit for derlg
derlgl       = lower confidence limit for derlg
derlglogerr  = logarithm of derlgerr
derlin       = coercivity distribution on a linear scale
derlinerr    = error of derlin
derln        = coercivity distribution on a user-defined linear scale
derlnerr     = error of derln
derlnh       = upper confidence limit of derln
derlnl       = lower confidence limit of derln
derlnlogerr  = logarithm of derlnerr
derlog       = coercivity distribution on a log scale
derlogerr    = error of derlog
derpow       = coercivity distribution on a power scale
derpowerr    = error of derpow
derpw        = coercivity distribution on a user-defined power scale
derpwerr     = error of derpw
derpwh       = higher confidence limit of derpw
derpwl       = lower confidence limit of derpw
derpwlogerr  = logarithm of derpwerr
derr         = error of dres
derrl        = error of dres on a user-defined scale
digleft      = boolean for digital truncation effects on the left-end
digright     = boolean for digital truncation effects on the right-end
dim          = dimension of the input data
dir          = directory of the input data file
dithf        = first derivative of ithf
double       = boolean variable for double points 
dpoly        = first derivative of poly
dres         = first derivative of res
dresl        = first derivative of resl
dthf         = first derivative of thf
duns         = derivative of uns
dx           = step width on the x-scale
ee3          = 3-dimensional empirical error matrix
ee5          = 5-dimensional empirical error matrix
eel          = empirical error matrix at selected points
err          = estimated measurement error on the original scale
err01        = err on the scale of res01
err02        = error obtained from the misfit of err01
err03h       = upper confidence limit of err03
err03l       = lower confidence limit of err03
err05        = error of res05
err05f       = error of res05f
err05h       = upper confidence limit of err05
err05l       = lower confidence limit of err05
err1         = help error variable
err2         = help error variable
err3         = help error variable
errrng       = range of errors
expn         = exponent used to rescale the fields
expow        = exponent used to represent the coercivity distribution
exrng        = list of exponents
ferr         = list of soultions obtained by inversion of ee03
fit          = list of fitted measurements
fiterr       = error of fit
fiterrf      = error of fitf
fitf         = fitted measurement curve sampled on arbitrary points
fitlogerr    = logarithm of fiterr
fitlogerrf   = logarithm of fiterrf
flex         = flexture point
frng         = range for the measurements fit
ft           = list of fitting parameters for bthf and thf
ithf         = inverse function of thf
l0           = length of list
l1           = length of lst
l2           = length of various lists
l3           = length of output data
labels       = axes labels
linear       = boolean variable for the coercivity distribution on linear scale
linrng       = range of coercivity distribution on a lin scale
list         = list of unscaled measurements
logrng       = range of coercivity distribution on a log scale
lrng         = user-defined linrng
ls           = help variable for a list
lst          = scaled list or scaled list without unsaturated component
lstu         = scaled list with unsaturated component
maq          = boolean variable for an acquisition curve
med          = median field
mono         = boolean variable for a monotonic curve
name         = name of the input data file
nmatrix      = error matrix for collocation
nout         = user-defined number of output points
null         = list of outlyers
numbers      = list of number coordinates for graphics
opt          = boolean variable for optimization
ord          = help variable for choosing an optimal exponent
orng         = range of the original data
out          = boolean variable for saving the results
poly         = a polynom
power        = boolean variable for the choice of a power field scale
powrng       = range of the coercivity distribution on a power scale
ptexpn       = exponent for equally spaced field values
q            = damping term for scaling fields
qr           = damping term for scaling residuals
quant1       = left-sided 25/75% quantil
quant2       = right-sided 25/75% quantil
ref          = original measurement data
removed      = boolean variable for removed outlyers
rempoints    = boolean variable for removing outlyers 
res          = help variable for misfit estimation
res01        = unscaled residuals
res02        = (res01 - poly) or help variable
res03        = scaled residuals
res03blue    = res03 every 10th point
res04        = collocation of res03 with empirical autocorrelation function
res04i       = interpolated res04
res05        = filtered res03
res05f       = interpolated res05
res06        = filtered res03 on a user-defined scale
res1         = help variable for misfit estimation
res2         = help variable for misfit estimation
res3         = help variable for misfit estimation
res4         = help variable for misfit estimation
resl         = final residuals
resnull      = list of outlyers
resrng       = range of residuals
rsexpn       = exponent to rescale the residuals
scfail       = boolean variable for failed field rescaling optimization
sig          = help variable to estimate the significance or used as sign
sol          = a solution contained in ferr
spacing      = distance between the measurements on the x-axis
sug          = suggested field scale
syexpn       = scaling exponent for best symmetry
thf          = best thanh-fit for calculating the residuals
un           = parameters for uns
uns          = function for the unsaturated magnetization
unsp         = contribution of uns to the total magnetization
verr         = vector of errors
vrng         = field range chosen by the user
weights      = weights for fitting
x0           = origin of the field x-axis
x0r          = origin of the residuals x-axis
xlist        = list of original field values
xrng         = range of values on the x-axis
xticks       = list of ticks for the x-axis
yrng         = range of values on the y-axis
yticks       = list of ticks for the y-axis

List of all sub-routines used by Codica:
AutoCollocation = collocation with empirical autocorrelation function
AutoCorrelation = estimate an empirical autocorrelation function
Collocation     = collocation with gaussian autocorrelation function
DCollocation    = first derivative of Collocation
FindExtremes    = finds significan local minima and maxima
HypFit          = finds a best-fit hyperbolic function
HypLin          = linearizes a cumulative PDF using hyperbolic functions
InputCheck      = function for interacting with the user
LinearTicks     = produces a list of ticks on a linear scale
LogTicks        = produces a list of ticks on a logarithmic scale
PowerTicks      = produces a list of ticks on a power scale
Prn             = print a number in Fortran format
PutNumbers      = calculates the coordinates of number labels *)


(* Set options *)
$TextStyle={FontSize->12};
Off[InterpolatingFunction::dmval];
Off[Graphics::gptn];
Off[ReadList::nffil];
Off[Read::readn];
Off[Inverse::luc];
Off[Inverse::invc];
Off[Power::infy];
Off[Syntax::com];
Off[FindFit::lstol];
Off[FindFit::fmdig];
Off[FindFit::cvmit];
Off[FindFit::sdprec];
Off[FindFit::sszero];
Off[NMinimize::cvmit];
Off[FindRoot::lstol];
Off[FindRoot::jsing];
Off[NumberForm::sigz];


(* read the data file *)
bool=True;count=0;
While[bool,
      Which[count==0,
            name=InputCheck["Enter the complete path of the data file.

\nInclude file extensions!
\n\nExample. C:/users/data/myfile.dat",
                            {"Path"},{{String,{}}}];
            count++,
            dir==={},
            name=InputCheck[StringJoin["File ",name," not found.\n
\nReenter path and name of the data file:
\n
\nExample. C:/users/data/myfile.dat or C:/users/data/myfile"],
                            {"Path"},{{String,{}}}];
            count++,
            Length[dir]>1,
            name=InputCheck[StringJoin["More files beginning with  ",name,"
have been found.\n
\nSpecify better the file name or include the file extension:
\n
\nExample. C:/users/data/myfile.dat or C:/users/data/myfile"],
                            {"Path"},{{String,{}}}];
            count++];
      
      dir=FileNames[StringJoin[StringTake[name,
                               {Last[StringPosition[name,"/"]][[1]]+1,
                                StringLength[name]}],
                               "*"],
                     StringTake[name,
                                {1,
                                 Last[StringPosition[name,"/"]][[1]]}]];
      If[Length[dir]==1,
         dir=dir[[1]];
         bool=False]

     ]; (* Close While bool *)


(* Check data file *)
Print[name];

count=1;bool=True;
lst=OpenRead[dir];
While[bool,
      list=Read[lst,String,RecordSeparators->{"\n"}];
      If[list===EndOfFile,
         bool=False,
         dim=ReadList[StringToStream[list],Word,
                      WordSeparators->{" ","\t",",",":",";"}];
         If[Length[dim]>1,
            dim=ReadList[StringToStream[StringJoin[dim[[1]]," ",dim[[2]]]],
                         Number];
            If[Last[dim]===$Failed,
               count++,
               bool=False],
            count++]]];
Close[lst];

If[list===EndOfFile,
   StylePrint[StringJoin["File ",name," contains no numerical data!
\nPROGRAM ABORTED."],
              FontColor->RGBColor[1,0,0],FontWeight->"Bold"];
   Abort[]];

lst=OpenRead[dir];
Skip[lst,String,count-1,RecordSeparators->{"\n"}];
list=ReadList[lst,Word,RecordLists->True,
              WordSeparators->{" ","\t",",",":",";"}];
Close[lst];

l1=Length[list];
count=1;
While[count<=l1&&Length[list[[count]]]>1,
      count++];
If[count<9,
   StylePrint[StringJoin["File ",name," does not contain enough data!
\nPROGRAM ABORTED."],
              FontColor->RGBColor[1,0,0],FontWeight->"Bold"];
   Abort[]];
list=Take[list,{1,count-1}];
l1=Length[list];

lst=Flatten[Table[{list[[k,1]]," ",list[[k,2]],"\n"},{k,1,l1}]];
lst=StringJoin[lst];
list=ReadList[StringToStream[lst],Number,RecordLists->True];

If[Last[list]==={$Failed},
   list=Drop[list,-1]];

l1=Length[list];
If[l1<9,
   StylePrint[StringJoin["File ",name," does not contain enough data!
\nPROGRAM ABORTED."],
              FontColor->RGBColor[1,0,0],FontWeight->"Bold"];
   Abort[]];

dim=Dimensions[list];
If[Length[dim]<2,
   StylePrint[StringJoin["File ",name," contains data that is not in a
column format.
\nPROGRAM ABORTED."],
              FontColor->RGBColor[1,0,0],FontWeight->"Bold"];
   Abort[]];
If[dim[[2]]<2,
   StylePrint[StringJoin["File ",name," should contain at least two colums
of numbers.
\nPROGRAM ABORTED."],
              FontColor->RGBColor[1,0,0],FontWeight->"Bold"];
   Abort[]];
If[dim[[2]]>2,
   StylePrint[StringJoin["WARNING! File ",name," contains 3 or more colums
of numbers.
\nCodica will consider only the first two colums."],
              FontColor->RGBColor[1,0,0]];
   list=Table[{list[[k,1]],list[[k,2]]},{k,1,l1}]];

dir=StringTake[dir,
               {1,Last[StringPosition[dir,"\\"]][[1]]}];


Print["Check the measurements..."];
lst=Table[list[[k,1]],{k,1,l1}];
count=(Min[Abs[lst]]+Max[Abs[lst]])/2;
lst=Table[If[Abs[list[[k,1]]]<count,0,list[[k,1]]],
          {k,1,l1}];

If[lst[[1]]==0,
   count=Ordering[lst,1,#1<#2&][[1]];
   If[lst[[count]]==0,
      bool=Ordering[lst,1,#1>#2&][[1]],
      bool=Min[{Ordering[lst,1,#1<#2&][[1]],
                Ordering[lst,1,#1>#2&][[1]]}]];
   list=Take[list,{1,bool}],
   
   StylePrint["Data file begins with a sequence of field values
that does not define a magnetization curve!
\nPROGRAM ABORTED.",
              FontColor->RGBColor[1,0,0],FontWeight->"Bold"]];

l0=Length[list];
If[l0<l1,
   StylePrint["WARNING! Data file contains probably more than one
magnetization curve, or the field does not change monotonically.
\nCODICA will consider only the first subset of data that defines a
magnetization curve.",
              FontColor->RGBColor[1,0,0]]];

l1=l0;
If[Last[list][[1]]<0,
   list=Table[{-list[[k,1]],list[[k,2]]},{k,1,l1}]];

lst=Min[Table[list[[k,1]],{k,1,l1}]];
If[lst<0,
   StylePrint["WARNING! The magnetization curve contains field values with
opposite sign.
\nCODICA will remove the field values with wrong sign.",
              FontColor->RGBColor[1,0,0]];
   count=Last[Position[list,_?(MatchQ[#,{_?(#<0&),_}]&)]][[1]];
   list=Take[list,{count+1,l1}]];
l0=Length[list];

If[l0<9,
   StylePrint[StringJoin["There are less than 9 measurements that are
compatible with a measurement curve! Check the data file.
\nPROGRAM ABORTED."],
              FontColor->RGBColor[1,0,0],FontWeight->"Bold"];
   Abort[]];

list=Sort[list,#1[[1]]<#2[[1]]&];


(* looking after double points *)
l0=Length[list];
double=False;
ref=list;
count={ref[[1]]};
list={};
For[i=2,i<=l0,i++,
    If[ref[[i,1]]==count[[1,1]],
       double=True;
       count=Append[count,ref[[i]]],
       list=Append[list,count];
       count={ref[[i]]}]];
list=Append[list,count];
l0=Length[list];
list=Table[{list[[k,1,1]],
            Sum[list[[k,j,2]],{j,1,Length[list[[k]]]}]/Length[list[[k]]]},
           {k,1,l0}];
If[double,
   StylePrint["WARNING! Multiple measurements have been detected and averaged.
\nCheck the input file to see if they are all valid.",
              FontColor->RGBColor[1,0,0]]]; 


(* reducing lists longer than 300 *)

If[l0>300,
   l1=Ceiling[l0/300];
   lst=Table[{Sum[list[[k+j,1]],{j,0,l1-1}]/l1,
              Sum[list[[k+j,2]],{j,0,l1-1}]/l1},
             {k,1,l0-2*l1+1,l1}];
   l2=Length[lst]*l1;
   lst=Append[lst,
              {Sum[list[[j,1]],{j,l2+1,l0}]/(l0-l2),
               Sum[list[[j,2]],{j,l2+1,l0}]/(l0-l2)}];
   list=lst;
   l0=Length[list];
   ref=lst;
   StylePrint["WARNING! The curve contains more than 300 measurements.
\nA moving-average filter has been applied to reduce the number of
measurements.",
              FontColor->RGBColor[1,0,0]]];
xlist=Table[list[[k,1]],{k,1,l0}];


(* calculating data parameters *)
If[Last[list][[2]]>First[list][[2]],
   maq=True,maq=False];
orng={Table[list[[k,1]],{k,1,l0}],
      Table[list[[k,2]],{k,1,l0}]};
orng={{Min[orng[[1]]],Max[orng[[1]]]},
      {Min[orng[[2]]],Max[orng[[2]]]}};
orng={{orng[[1,1]]-0.01*(orng[[1,2]]-orng[[1,1]]),
       orng[[1,2]]+0.01*(orng[[1,2]]-orng[[1,1]])},
      {orng[[2,1]]-0.01*(orng[[2,2]]-orng[[2,1]]),
       orng[[2,2]]+0.01*(orng[[2,2]]-orng[[2,1]])}};


(* Show original AF curve *)
xticks=LinearTicks[orng[[1]],"Original"][[1]];
yticks=LinearTicks[orng[[2]],"Engeenering"];
count=yticks[[2]];
yticks=yticks[[1]];
Show[ListPlot[ref,
              PlotStyle->PointSize[0.01],
              DisplayFunction->Identity],
     ListPlot[ref,
              PlotJoined->True,
              DisplayFunction->Identity],
     Axes->False,Frame->{True,True,False,False},
     PlotRange->orng,
     ImageSize->500,
     FrameTicks->{xticks,yticks,None,None},
     FrameLabel->{"magnetic field",
                  StringJoin["magnetization, 1e",
                             ToString[Log[10,count]]],
                  None,None},
     RotateLabel->True,
     PlotLabel->StyleForm["ORIGINAL MEASUREMENTS:",
                          FontFamily->"Geneva",FontSize->16],
     DisplayFunction->$DisplayFunction];


(* check digital truncation *)
Print["Evaluate the properties of the measured curve..."];
deltadig=Table[Chop[Abs[list[[k,2]]-list[[k+1,2]]]],
               {k,1,Length[list]-1}];
err1=Flatten[Position[deltadig,0]];
If[Length[err1]>1,
   err2=Table[err1[[k+1]]-err1[[k]],
              {k,1,Length[err1]-1}];
   If[MemberQ[err2,2],
      StylePrint["WARNING! CODICA detected possile digital truncation effects.
\nList of points where jumps arising from digital truncation have been
identified:",
                 FontColor->RGBColor[1,0,0]];
      err3=Flatten[Position[err2,2]];
      err1=Table[err1[[err3[[k]]]]+1,{k,1,Length[err3]}];
      deltadig=Table[{err1[[k]],deltadig[[err1[[k]]]]},{k,1,Length[err1]}],
      deltadig=None],
   deltadig=None];

If[Not[deltadig===None],
   err1=Join[{{"point #","field","detected jump"}},
             Table[{deltadig[[k,1]],list[[deltadig[[k,1]],1]],deltadig[[k,2]]},
                   {k,1,Length[deltadig]}]];
   StylePrint[TableForm[err1],
              FontColor->RGBColor[1,0,0]]];


(* remove points affected by digital truncation *)
digleft=False;
digright=False;
If[Not[deltadig===None],
   count=1;
   lst=list;
   If[maq,
      While[list[[count,2]]<First[list][[2]]+0.5*Last[list][[2]]&&
            count<l0,
            count++],
      While[list[[count,2]]>Last[list][[2]]+0.5*First[list][[2]]&&
            count<l0,
            count++]];
   If[First[deltadig][[1]]<count,
      digleft=True;
      bool=1;
      While[bool<=Length[deltadig]&&deltadig[[bool,1]]<count,bool++];
      res01=Table[{(list[[deltadig[[k,1]],1]]+list[[deltadig[[k,1]]+1,1]])/2,
                   (list[[deltadig[[k,1]],2]]+list[[deltadig[[k,1]]+1,2]])/2},
                  {k,1,bool-1}];
      res01=Prepend[res01,First[list]],
      res01={}];
   If[Last[deltadig][[1]]>count,
      digright=True;
      bool=Length[deltadig];
      While[bool>=1&&deltadig[[bool,1]]>count,bool=bool-1];
      res02=Table[{(list[[deltadig[[k,1]],1]]+list[[deltadig[[k,1]]+1,1]])/2,
                   (list[[deltadig[[k,1]],2]]+list[[deltadig[[k,1]]+1,2]])/2},
                  {k,bool+1,Length[deltadig]}],
      res02={}];
   If[Length[res01]>0,
      While[First[lst][[1]]<=Last[res01][[1]],
            lst=Drop[lst,1]];
      lst=Drop[lst,1];
      lst=Join[res01,lst]];
   If[Length[res02]>0,
      While[Last[lst][[1]]>=First[res02][[1]],
            lst=Drop[lst,-1]];
      lst=Drop[lst,-1];
      lst=Join[lst,res02]];
   If[Length[lst]>8,
      list=lst;
      StylePrint["Some points have been discarded/resampled in order to
minimize digital truncation effects.",
                 FontColor->RGBColor[1,0,0]]]];
If[list[[1,1]]==0,
   q=list[[2,1]]/2,
   q=Min[list[[1,1]],list[[2,1]]-list[[1,1]]]/2];
x0=list[[1,1]];

(* begin section where points can be removed *)
rempoints=True;removed=False;
While[rempoints,
l1=Length[list];
If[removed,
   Print["Analyze the curve with removed points..."]];


(* First estimate of the left asymptote *)
bool=True;
res1=Infinity;
count=Max[4,Min[20,Floor[0.1*l1]]];
While[bool,
      asym1=Fit[Take[list,{1,count}],{1,x,x^2},x];
      res2=Sum[(list[[k,2]]-asym1/.(x->list[[k,1]]))^2,
               {k,1,count}]/(count-1);
      Which[res2>res1,
            bool=False,
            count==l1,
            bool=False,
            True,
            res1=res2;
            count++]];
asym1=asym1/.(x->list[[1,1]]);


(* First estimate of the right asymptote *)
bool=True;
res1=Infinity;
count=Max[5,Min[20,Floor[0.1*l1]]];
While[bool,
      If[maq,
         asym2=FindFit[Take[list,{l1-count,l1}],
                       a1-a2^2*Exp[-a3^2*x],
                       {{a1,Last[list][[2]]},
                        {a2,0.1*Sqrt[Abs[Last[list][[2]]-First[list][[2]]]]},
                        {a3,Sqrt[Abs[2/Last[list][[1]]]]}},x];
         asym2={Utilities`Codica`Private`a1/.asym2[[1]],
                -(Utilities`Codica`Private`a2/.asym2[[2]])^2,
                (Utilities`Codica`Private`a3/.asym2[[3]])^2},
         asym2=FindFit[Take[list,{l1-count,l1}],
                       a1+a2^2*Exp[-a3^2*x],
                       {{a1,Last[list][[2]]},
                        {a2,0.1*Sqrt[Abs[First[list][[2]]-Last[list][[2]]]]},
                        {a3,Sqrt[Abs[2/Last[list][[1]]]]}},x];
         asym2={Utilities`Codica`Private`a1/.asym2[[1]],
                (Utilities`Codica`Private`a2/.asym2[[2]])^2,
                (Utilities`Codica`Private`a3/.asym2[[3]])^2}];
      res2=Sum[(list[[k,2]]-asym2[[1]]-
                asym2[[2]]*Exp[-asym2[[3]]*list[[k,1]]])^2,
               {k,l1-count,l1}]/count;
      Which[res2>res1,
            bool=False,
            count==l1-1,
            bool=False,
            True,
            res1=res2;
            count++]];
asym2=asym2[[1]];
err2=Sqrt[res2];
If[digright,
   err2=Max[err2,Last[deltadig][[2]]]];


(* Estimate the median field *)
med=Table[list[[k,2]],{k,1,l1}];
med=(asym1+asym2)/2;
cent=1;
While[cent<l1&&
      ((Not[maq]&&list[[cent,2]]>med)||
       (maq&&list[[cent,2]]<med)),
      cent++];
bool=True;
res1=Infinity;
count=Max[3,Min[20,Floor[0.1*l1]]];
While[bool,
      poly=Fit[Take[list,{Max[1,cent-count],Min[Length[list],cent+count]}],
               {1,x,x^2,x^3},x];
      res2=Sum[(list[[k,2]]-(poly/.(x->list[[k,1]])))^2,
               {k,Max[1,cent-count],Min[l1,cent+count]}]/
           (Min[Length[list],cent+count]-Max[1,cent-count]);
      Which[res2>res1,
            bool=False,
            cent-count==1||cent+count==l1,
            bool=False,
            True,
            res1=res2;
            count++]];
count=count-1;
med=N[Solve[poly==med,x]];
med={x/.med[[1,1]],x/.med[[2,1]],x/.med[[3,1]]};
med=Select[med,Im[#]==0&];
If[Length[med]==0,
   med=$Failed,
   med=Select[med,list[[cent-count,1]]<=#<=list[[cent+count,1]]&];
   If[Length[med]==0,
      med=$Failed,
      med=med[[1]]]];
If[med===$Failed,
   med=list[[cent,1]]];

(* Estimate the left 25/75% quantil *)
quant1=Table[list[[k,2]],{k,1,l1}];
If[maq,
   quant1=(3*Min[quant1]+Max[quant1])/4,
   quant1=(3*Max[quant1]+Min[quant1])/4];
cent=1;
While[cent<l1&&
      ((Not[maq]&&list[[cent,2]]>quant1)||
       (maq&&list[[cent,2]]<quant1)),
      cent++];
bool=True;
res1=Infinity;
count=Max[3,Min[20,Floor[0.1*l1]]];
While[bool,
      poly=Fit[Take[list,{Max[1,cent-count],Min[l1,cent+count]}],
               {1,x,x^2},x];
      res2=Sum[(list[[k,2]]-(poly/.(x->list[[k,1]])))^2,
               {k,Max[1,cent-count],Min[l1,cent+count]}]/
           (Min[l1,cent+count]-Max[1,cent-count]);
      Which[res2>res1,
            bool=False,
            cent-count==1||cent+count==l1,
            bool=False,
            True,
            res1=res2;
            count++]];
count=count-1;
quant1=N[Solve[poly==quant1,x]];
quant1={x/.quant1[[1,1]],x/.quant1[[2,1]]};
quant1=Select[quant1,Im[#]==0&];
If[Length[quant1]==0,
   quant1=$Failed,
   quant1=Select[quant1,list[[cent-count,1]]<=#<=list[[cent+count,1]]&];
   If[Length[quant1]==0,
      quant1=$Failed,
      quant1=quant1[[1]]]];
If[quant1===$Failed,
   quant1=list[[cent,1]]];


(* Estimate the right 25/75% quantil *)
quant2=Table[list[[k,2]],{k,1,l1}];
If[maq,
   quant2=(Min[quant2]+3*Max[quant2])/4,
   quant2=(Max[quant2]+3*Min[quant2])/4];
cent=1;
While[(Not[maq]&&list[[cent,2]]>quant2)||
      (maq&&list[[cent,2]]<quant2),
      cent++];
bool=True;
res1=Infinity;
count=Max[3,Min[20,Floor[0.1*l1]]];
While[bool,
      poly=Fit[Take[list,{Max[1,cent-count],Min[l1,cent+count]}],
               {1,x,x^2},x];
      res2=Sum[(list[[k,2]]-(poly/.(x->list[[k,1]])))^2,
               {k,Max[1,cent-count],Min[l1,cent+count]}]/
           (Min[l1,cent+count]-Max[1,cent-count]);
      Which[res2>res1,
            bool=False,
            cent-count==1||cent+count==l1,
            bool=False,
            True,
            res1=res2;
            count++]];
count=count-1;
quant2=N[Solve[poly==quant2,x]];
quant2={x/.quant2[[1,1]],x/.quant2[[2,1]]};
quant2=Select[quant2,Im[#]==0&];
If[Length[quant2]==0,
   quant2=$Failed,
   quant2=Select[quant2,list[[cent-count,1]]<=#<=list[[cent+count,1]]&];
   If[Length[quant2]==0,
      quant2=$Failed,
      quant2=quant2[[1]]]];
If[quant2===$Failed,
   quant2=list[[cent,1]]];


(* Second estimate of the left asymptote *)
bool=True;
res1=Infinity;
count=6;
While[list[[count,1]]<med-3.5*Abs[med-Min[quant1,quant2]],
      count++];
count=Min[count,18];
Clear[a1];Clear[a2];Clear[a3];
While[bool,
      asym1=Fit[Take[list,{1,count}],{1,x,x^2},x];
      res2=Sum[(list[[k,2]]-asym1/.(x->list[[k,1]]))^2,
               {k,1,count}]/(count-1);
      Which[res2>res1,
            bool=False,
            count==l1,
            bool=False,
            True,
            res1=res2;
            count++]];
asym1=asym1/.(x->list[[1,1]]);
err1=Sqrt[res2];
If[digleft,
   err1=Max[err1,First[deltadig][[2]]]];

Which[D[asym1,{x,2}]>0&&Not[maq]&&Not[removed],
      StylePrint["Warning! The left-end of the measured curve is concave.
\nCheck if more measurement points are necessary at small fields.",
      FontColor->RGBColor[1,0,0]],
      D[asym1,{x,2}]<0&&maq&&Not[removed],
      StylePrint["Warning! The left-end of the measured curve is convex.
\nCheck if more measurement points are necessary at small fields.",
      FontColor->RGBColor[1,0,0]],
      True,Null];


StylePrint[StringJoin["Estimated initial magnetization: ",
                      Prn[asym1,4],
                      " \[PlusMinus] ",
                      Prn[err1,2],
                      "\nEstimated final magnetization:   ",
                      Prn[asym2,4],
                      " \[PlusMinus] ",
                      Prn[err2,2],
                      " (extrapolated)\nEstimated median field:          ",
                      If[med===$Failed,
                         "failed",
                         Prn[med,4]]],
           FontColor->RGBColor[0,0,1]];


(* Scanning the field scaling exponent *)
Print["Searching for an optimized field scale."];

ord=Table[k,{k,-0.02,-4.6,-0.02}];
exrng={};
spacing=Table[list[[k+1,1]]-list[[k,1]],{k,1,l1-1}];
spacing=Max[spacing]/(Last[list][[1]]-First[list][[1]]);

Print["Scan scaling exponents < 1. Please wait..."];
weights=Join[{list[[2,1]]-list[[1,1]]},
             Table[(list[[k+1,1]]-list[[k-1,1]])/2,{k,2,l1-1}],
             {list[[l1,1]]-list[[l1-1,1]]}];
weights=weights/Sum[weights[[k]],{k,1,l1}];

ft=HypFit[list,{asym1,asym2,med,quant1,quant2},
          Sqrt[weights],maq,False];
res1=Sum[(ft[[3]]*(0.5-0.5*Tanh[ft[[1]]*(list[[k,1]]-ft[[2]])])+ft[[4]]+
          ft[[5]]*(list[[k,1]]-ft[[2]]+
                   Log[Cosh[Abs[ft[[1]]]*(list[[k,1]]-ft[[2]])]]/Abs[ft[[1]]]+
                   Log[2]/Abs[ft[[1]]])-
          list[[k,2]]),
         {k,1,l1}]/l1/Abs[asym1-asym2];
res3=Sqrt[Sum[(ft[[3]]*(0.5-0.5*Tanh[ft[[1]]*(list[[k,1]]-ft[[2]])])+ft[[4]]+
              ft[[5]]*(list[[k,1]]-ft[[2]]+
                       Log[Cosh[Abs[ft[[1]]]*(list[[k,1]]-ft[[2]])]]/
                       Abs[ft[[1]]]+
                       Log[2]/Abs[ft[[1]]])-
              list[[k,2]])^2,
              {k,1,l1}]/l1]/Abs[asym1-asym2];

exrng={{1,res1,spacing,res3}};
count=1;bool=True;scfail=False;
While[bool,
expn=Exp[ord[[count]]];
lst=Table[{(list[[k,1]]+q)^expn-(x0+q)^expn,
            list[[k,2]]},{k,1,l1}];
weights=Join[{lst[[2,1]]-lst[[1,1]]},
             Table[(lst[[k+1,1]]-lst[[k-1,1]])/2,{k,2,l1-1}],
             {lst[[l1,1]]-lst[[l1-1,1]]}];
weights=weights/Sum[weights[[k]],{k,1,l1}];
ft=HypFit[lst,{asym1,asym2,
               (med+q)^expn-(x0+q)^expn,
               (quant1+q)^expn-(x0+q)^expn,
               (quant2+q)^expn-(x0+q)^expn},
          Sqrt[weights],
          maq,
          False];
res2=Sum[(ft[[3]]*(0.5-0.5*Tanh[ft[[1]]*(lst[[k,1]]-ft[[2]])])+ft[[4]]+
          ft[[5]]*(lst[[k,1]]-ft[[2]]+
                   Log[Cosh[Abs[ft[[1]]]*(lst[[k,1]]-ft[[2]])]]/Abs[ft[[1]]]+
                   Log[2]/Abs[ft[[1]]])-
          lst[[k,2]]),
         {k,1,l1}]/l1/Abs[asym1-asym2];
res3=Sqrt[Sum[(ft[[3]]*(0.5-0.5*Tanh[ft[[1]]*(lst[[k,1]]-ft[[2]])])+ft[[4]]+
              ft[[5]]*(lst[[k,1]]-ft[[2]]+
                       Log[Cosh[Abs[ft[[1]]]*(lst[[k,1]]-ft[[2]])]]/
                       Abs[ft[[1]]]+
                       Log[2]/Abs[ft[[1]]])-
              lst[[k,2]])^2,
              {k,1,l1}]/l1]/Abs[asym1-asym2];
delta=Sort[Table[lst[[k+1,1]]-lst[[k,1]],{k,1,l1-1}]];
delta=Max[delta]/(Last[lst][[1]]-First[lst][[1]]);
exrng=Append[exrng,{expn,res2,delta,res3}];
Which[count==Length[ord]&&Sign[res1]==Sign[res2],
      bool=False;
      scfail=True,
      count==Length[ord],
      bool=False,
      Sign[res1]==Sign[res2],
      count++,
      True,
      bool=False]

   ]; (* Close While bool *)

deltamin=Cases[exrng,_?(#[[3]]<Max[1.3*spacing,0.05]&)];
bool=Ordering[exrng,1,#1[[2]]<#2[[2]]&][[1]];
If[scfail,
   count=Ordering[exrng,1,#1[[4]]<#2[[4]]&][[1]];
   If[count>1,
      scfail=False;
      syexpn=exrng[[count,1]]],
   If[count==1,
      syexpn=exrng[[1,1]],
      syexpn=(exrng[[count,1]]*Abs[exrng[[count-1,2]]]+
              exrng[[count-1,1]]*Abs[exrng[[count,2]]])/
             (Abs[exrng[[count-1,2]]]+Abs[exrng[[count,2]]])]];

If[scfail,

scfail=False;
ord=Table[k,{k,0.01,1,0.01}];
res1=exrng[[1,2]];
exrng={exrng[[1]]};
Print["Scan scaling exponents > 1. Please wait..."];
bool=True;count=1;
While[bool,
expn=Exp[ord[[count]]];
lst=Table[{(list[[k,1]]+q)^expn-(x0+q)^expn,
            list[[k,2]]},{k,1,l1}];
weights=Join[{lst[[2,1]]-lst[[1,1]]},
             Table[(lst[[k+1,1]]-lst[[k-1,1]])/2,{k,2,l1-1}],
             {lst[[l1,1]]-lst[[l1-1,1]]}];
weights=weights/Sum[weights[[k]],{k,1,l1}];
ft=HypFit[lst,{asym1,asym2,
               (med+q)^expn-(x0+q)^expn,
               (quant1+q)^expn-(x0+q)^expn,
               (quant2+q)^expn-(x0+q)^expn},
          Sqrt[weights],
          maq,
          False];
res2=Sum[(ft[[3]]*(0.5-0.5*Tanh[ft[[1]]*(lst[[k,1]]-ft[[2]])])+ft[[4]]+
          ft[[5]]*(lst[[k,1]]-ft[[2]]+
                   Log[Cosh[Abs[ft[[1]]]*(lst[[k,1]]-ft[[2]])]]/Abs[ft[[1]]]+
                   Log[2]/Abs[ft[[1]]])-
          lst[[k,2]]),
         {k,1,l1}]/l1/Abs[asym1-asym2];
res3=Sqrt[Sum[(ft[[3]]*(0.5-0.5*Tanh[ft[[1]]*(lst[[k,1]]-ft[[2]])])+ft[[4]]+
              ft[[5]]*(lst[[k,1]]-ft[[2]]+
                       Log[Cosh[Abs[ft[[1]]]*(lst[[k,1]]-ft[[2]])]]/
                       Abs[ft[[1]]]+
                       Log[2]/Abs[ft[[1]]])-
              lst[[k,2]])^2,
              {k,1,l1}]/l1]/Abs[asym1-asym2];
delta=Sort[Table[lst[[k+1,1]]-lst[[k,1]],{k,1,l1-1}]];
delta=Max[delta]/(Last[lst][[1]]-First[lst][[1]]);
exrng=Append[exrng,{expn,res2,delta,res3}];
Which[count==Length[ord]&&Sign[res1]==Sign[res2],
      bool=False;
      scfail=True,
      count==Length[ord],
      bool=False,
      Sign[res1]==Sign[res2],
      count++,
      True,
      bool=False]

   ]; (* Close While bool *)

deltamin=Join[deltamin,
              Cases[exrng,_?(#[[3]]<Max[1.3*spacing,0.05]&)]];
If[Length[deltamin]<1,
   StylePrint["WARNING! Field values are very irregularily spaced.
This may produce a bad fit of the data.
\nConsider to remeasure the curve with better spaced field values.",
              FontColor->RGBColor[1,0,0]]];


If[scfail,
   count=Ordering[exrng,1,#1[[4]]<#2[[4]]&][[1]];
   If[1<count<Length[exrng],
      scfail=False;
      syexpn=exrng[[count,1]],
      syexpn=1;
      StylePrint["WARNING! The search for an optimized scale failed.",
                 FontColor->RGBColor[1,0,0]]],
   If[count==1,
      syexpn=exrng[[1,1]],
      syexpn=(exrng[[count,1]]*Abs[exrng[[count-1,2]]]+
              exrng[[count-1,1]]*Abs[exrng[[count,2]]])/
             (Abs[exrng[[count-1,2]]]+Abs[exrng[[count,2]]])]];

  ]; (* Close If scfail *)


(* Residuals with expn=1 *)
weights=Join[{list[[2,1]]-list[[1,1]]},
             Table[(list[[k+1,1]]-list[[k-1,1]])/2,{k,2,l1-1}],
             {list[[l1,1]]-list[[l1-1,1]]}];
weights=weights/Sum[weights[[k]],{k,1,l1}];
ft=HypFit[list,{asym1,asym2,med,quant1,quant2},
          Sqrt[weights],maq,False];
res1=Sqrt[Sum[(ft[[3]]*(0.5-0.5*Tanh[ft[[1]]*(list[[k,1]]-ft[[2]])])+ft[[4]]+
               ft[[5]]*
              (lst[[k,1]]-ft[[2]]+
               Log[Cosh[Abs[ft[[1]]]*(lst[[k,1]]-ft[[2]])]]/Abs[ft[[1]]]+
              Log[2]/Abs[ft[[1]]])-
               list[[k,2]])^2*weights[[k]],
              {k,1,l1}]]/
     Abs[asym1-asym2];


(* Evaluate the best scale for the field values *)
ls=Table[{k,list[[k,1]]},{k,1,l1}];
bool={1,1};
For[i=1,i<=10,i++,
    poly=FindFit[ls,
                 a1*x^(a2^2),
                 {{a1,bool[[1]]},{a2,bool[[2]]}},
                 x];
    bool={Utilities`Codica`Private`a1/.poly[[1]],
          Utilities`Codica`Private`a2/.poly[[2]]}];
ptexpn=1/bool[[2]]^2;

lst=Table[{(list[[k,1]]+q)^ptexpn-(x0+q)^ptexpn,
           list[[k,2]]},{k,1,l1}];
weights=Join[{lst[[2,1]]-lst[[1,1]]},
             Table[(lst[[k+1,1]]-lst[[k-1,1]])/2,{k,2,l1-1}],
             {lst[[l1,1]]-lst[[l1-1,1]]}];
weights=weights/Sum[weights[[k]],{k,1,l1}];
ft=HypFit[lst,{asym1,asym2,
               (med+q)^ptexpn-(x0+q)^ptexpn,
               (quant1+q)^ptexpn-(x0+q)^ptexpn,
               (quant2+q)^ptexpn-(x0+q)^ptexpn},
          Sqrt[weights],
          maq,
          False];
res2=Sqrt[Sum[(ft[[3]]*(0.5-0.5*Tanh[ft[[1]]*(lst[[k,1]]-ft[[2]])])+ft[[4]]+
               ft[[5]]*
              (lst[[k,1]]-ft[[2]]+
               Log[Cosh[Abs[ft[[1]]]*(lst[[k,1]]-ft[[2]])]]/Abs[ft[[1]]]+
              Log[2]/Abs[ft[[1]]])-
               lst[[k,2]])^2*weights[[k]],
              {k,1,l1}]]/
     Abs[asym1-asym2];


(* Evaluate the best scale for symmetry *)
opt=True;
lst=Table[{(list[[k,1]]+q)^syexpn-(x0+q)^syexpn,
           list[[k,2]]},{k,1,l1}];
weights=Join[{lst[[2,1]]-lst[[1,1]]},
             Table[(lst[[k+1,1]]-lst[[k-1,1]])/2,{k,2,l1-1}],
             {lst[[l1,1]]-lst[[l1-1,1]]}];
weights=weights/Sum[weights[[k]],{k,1,l1}];
ft=HypFit[lst,{asym1,asym2,
               (med+q)^syexpn-(x0+q)^syexpn,
               (quant1+q)^syexpn-(x0+q)^syexpn,
               (quant2+q)^syexpn-(x0+q)^syexpn},
          Sqrt[weights],
          maq,
          False];
res3=Sqrt[Sum[(ft[[3]]*(0.5-0.5*Tanh[ft[[1]]*(lst[[k,1]]-ft[[2]])])+ft[[4]]+
               ft[[5]]*
              (lst[[k,1]]-ft[[2]]+
               Log[Cosh[Abs[ft[[1]]]*(lst[[k,1]]-ft[[2]])]]/Abs[ft[[1]]]+
              Log[2]/Abs[ft[[1]]])-
               lst[[k,2]])^2*weights[[k]],
              {k,1,l1}]]/
     Abs[asym1-asym2];


(* Choose a field scale *)
If[scfail||Length[deltamin]<1,
   expn=1,
   If[exrng[[count,3]]<Max[1.3*spacing,0.05],
      expn=syexpn,
      deltamin=Table[{deltamin[[k,1]],Abs[deltamin[[k,1]]-syexpn]},
                     {k,1,Length[deltamin]}];
      expn=Sort[deltamin,#1[[2]]<#2[[2]]&][[1,1]]]];

lst=Table[{(list[[k,1]]+q)^expn-(x0+q)^expn,
           list[[k,2]]},{k,1,l1}];
weights=Join[{lst[[2,1]]-lst[[1,1]]},
             Table[(lst[[k+1,1]]-lst[[k-1,1]])/2,{k,2,l1-1}],
             {lst[[l1,1]]-lst[[l1-1,1]]}];
weights=weights/Sum[weights[[k]],{k,1,l1}];
ft=HypFit[lst,{asym1,asym2,
               (med+q)^expn-(x0+q)^expn,
               (quant1+q)^expn-(x0+q)^expn,
               (quant2+q)^expn-(x0+q)^expn},
          Sqrt[weights],
          maq,
          False];
res4=Sqrt[Sum[(ft[[3]]*(0.5-0.5*Tanh[ft[[1]]*(lst[[k,1]]-ft[[2]])])+ft[[4]]+
              ft[[5]]*
              (lst[[k,1]]-ft[[2]]+
               Log[Cosh[Abs[ft[[1]]]*(lst[[k,1]]-ft[[2]])]]/Abs[ft[[1]]]+
              Log[2]/Abs[ft[[1]]])-
               lst[[k,2]])^2*weights[[k]],
              {k,1,l1}]]/
     Abs[asym1-asym2];

count={{"Field scale","Transformation","Mean sd of fit"},
       {"original:","H' = H",
        StringJoin[" ",Prn[100*res1,3],"%"]},
       {"for equally spaced points:",
        StringJoin["H' = H^",Prn[ptexpn,3]],
        StringJoin[" ",Prn[100*res2,3],"%"]},
       {"for the most symmetric curve:",
        Which[scfail,
              "not found",
              syexpn<Exp[-4.58],
              StringJoin["H' = log(H+",Prn[q,3],")"],
              True,
              StringJoin["H' = (H+",Prn[q,3],")^",Prn[syexpn,3]]],
        If[scfail,
           "",
           StringJoin[" ",Prn[100*res3,3],"%"]]},
       {"suggested scale:",
        StringJoin["H' = (H+",Prn[q,3],")^",Prn[expn,3]],
        StringJoin[" ",Prn[100*res4,3],"%"]}};

StylePrint[TableForm[count],FontColor->RGBColor[0,0,1]];


(* Asymmetric best-fit *)
bft=HypFit[lst,
           {asym1,asym2,
            (med+q)^expn-(x0+q)^expn,
            (quant1+q)^expn-(x0+q)^expn,
            (quant2+q)^expn-(x0+q)^expn},
           Sqrt[weights],
           maq,
           True];
un={bft[[2]],Abs[bft[[8]]],bft[[7]]};
bthf[x_]:=bft[[3]]*(0.5000000001-0.5*
          Tanh[bft[[5]]*ArcSinh[bft[[1]]*(x-bft[[2]])/bft[[5]]]])^bft[[6]]+
          bft[[4]];
dbthf[y_]:=D[bthf[x],x]/.(x->y);
ddbthf[y_]:=D[bthf[x],{x,2}]/.(x->y);
uns[x_]:=un[[3]]*(x-un[[1]]+Log[Cosh[un[[2]]*(x-un[[1]])]]/un[[2]]+
                  Log[2]/un[[2]]);
duns[y_]:=D[uns[x],x]/.(x->y);

res=Sum[(bthf[lst[[k,1]]]+uns[lst[[k,1]]]-lst[[k,2]])^2*weights[[k]],
        {k,1,l1}];
res=Sqrt[res]/Abs[asym1-asym2];

unsp=NIntegrate[uns[(x+q)^expn-(x0+q)^expn],
                {x,First[list][[1]],Last[list][[1]]}]/
     NIntegrate[bthf[(x+q)^expn-(x0+q)^expn],
                {x,First[list][[1]],Last[list][[1]]}];
unsp=100*Abs[unsp];
StylePrint[StringJoin["Estimated contribution of a high-coercivity
component: ",
                      Prn[unsp,3]," %"],
           FontColor->RGBColor[0,0,1]];


frng={Table[lst[[k,1]],{k,1,l1}],
      Table[lst[[k,2]],{k,1,l1}]};
frng={{Min[frng[[1]]],Max[frng[[1]]]},
      {Min[frng[[2]]],Max[frng[[2]]]}};
frng={frng[[1]],
      {Min[frng[[2,1]],bthf[frng[[1,1]]],bthf[frng[[1,2]]]],
       Max[frng[[2,2]],bthf[frng[[1,1]]],bthf[frng[[1,2]]]]}};
frng={{frng[[1,1]]-0.01*(frng[[1,2]]-frng[[1,1]]),
       frng[[1,2]]+0.01*(frng[[1,2]]-frng[[1,1]])},
      {frng[[2,1]]-0.01*(frng[[2,2]]-frng[[2,1]]),
       frng[[2,2]]+0.01*(frng[[2,2]]-frng[[2,1]])}};

xticks=LinearTicks[frng[[1]],"Original"][[1]];
yticks=LinearTicks[frng[[2]],"Engeenering"];
count=yticks[[2]];
yticks=yticks[[1]];

Show[Plot[bthf[x]+uns[x],{x,frng[[1,1]],frng[[1,2]]},
          PlotPoints->200,
          PlotStyle->RGBColor[1,0,0],
          DisplayFunction->Identity],
     ListPlot[lst,
              PlotStyle->PointSize[0.01],
              DisplayFunction->Identity],
     Axes->False,Frame->{True,True,False,False},
     PlotRange->frng,
     ImageSize->500,
     FrameTicks->{xticks,yticks},
     FrameLabel->{"scaled magnetic field",
                  StringJoin["magnetization, 1e",
                             ToString[Log[10,count]]],
                  None,None},
     RotateLabel->True,
     PlotLabel->StyleForm[StringJoin["HYPERBOLIC MODEL (red). Mean sd: ",
                                     Prn[100*res,3],
                                     "%"],
                          FontFamily->"Geneva",FontSize->16],
     DisplayFunction->$DisplayFunction];


(* Choose another exponent *)

If[scfail,
   ord=InputCheck["CODICA could not find an optimum field scaling exponent!
\n\nAccept the proposed field scaling exponent (y/n)?",
                  {"Expression"},
                  {{String,{"y","n"}}}],
   ord=InputCheck["Accept the field scaling exponent (y/n)?",
                  {"Expression"},
                  {{String,{"y","n"}}}]];
If[ord==="y",scfail=False,scfail=True];


While[scfail,
expn=InputCheck["Enter a positive value for the field scaling exponent.
\n\nPossible range of values: 0.0001 - 5.",
                {"Expression"},
                {{Real,{0.000099,5.01}}}];
StylePrint[StringJoin["Manually chosen field scaling exponent: ",
                      Prn[expn,3]],
           FontColor->RGBColor[0,0,1]];
lst=Table[{(list[[k,1]]+q)^expn-(x0+q)^expn,
           list[[k,2]]},{k,1,l1}];
weights=Join[{lst[[2,1]]-lst[[1,1]]},
             Table[(lst[[k+1,1]]-lst[[k-1,1]])/2,{k,2,l1-1}],
             {lst[[l1,1]]-lst[[l1-1,1]]}];
weights=weights/(Last[lst][[1]]-First[lst][[1]]);
bft=HypFit[lst,
           {asym1,asym2,
            (med+q)^expn-(x0+q)^expn,
            (quant1+q)^expn-(x0+q)^expn,
            (quant2+q)^expn-(x0+q)^expn},
           Sqrt[weights],
           maq,
           True];
un={bft[[2]],Abs[bft[[8]]],bft[[7]]};
bthf[x_]:=bft[[3]]*(0.5000000001-0.5*
          Tanh[bft[[5]]*ArcSinh[bft[[1]]*(x-bft[[2]])/bft[[5]]]])^bft[[6]]+
          bft[[4]];
dbthf[y_]:=D[bthf[x],x]/.(x->y);
ddbthf[y_]:=D[bthf[x],{x,2}]/.(x->y);
uns[x_]:=un[[3]]*(x-un[[1]]+Log[Cosh[un[[2]]*(x-un[[1]])]]/un[[2]]+
                  Log[2]/un[[2]]);
duns[y_]:=D[uns[x],x]/.(x->y);


res=Sum[(bthf[lst[[k,1]]]-lst[[k,2]])^2*weights[[k]],{k,1,l1}]/
    Sum[weights[[k]],{k,1,l1}];
res=Sqrt[res]/Abs[asym1-asym2];

unsp=NIntegrate[uns[(x+q)^expn-(x0+q)^expn],
                {x,First[list][[1]],Last[list][[1]]}]/
     NIntegrate[bthf[(x+q)^expn-(x0+q)^expn],
                {x,First[list][[1]],Last[list][[1]]}];
unsp=100*Abs[unsp];
StylePrint[StringJoin["Estimated contribution of a high-coercivity
component: ",
                       Prn[unsp,3]," %"],
           FontColor->RGBColor[0,0,1]];

frng={Table[lst[[k,1]],{k,1,l1}],
      Table[lst[[k,2]],{k,1,l1}]};
frng={{Min[frng[[1]]],Max[frng[[1]]]},
      {Min[frng[[2]]],Max[frng[[2]]]}};
frng={frng[[1]],
      {Min[frng[[2,1]],bthf[frng[[1,1]]],bthf[frng[[1,2]]]],
       Max[frng[[2,2]],bthf[frng[[1,1]]],bthf[frng[[1,2]]]]}};
frng={{frng[[1,1]]-0.01*(frng[[1,2]]-frng[[1,1]]),
       frng[[1,2]]+0.01*(frng[[1,2]]-frng[[1,1]])},
      {frng[[2,1]]-0.01*(frng[[2,2]]-frng[[2,1]]),
       frng[[2,2]]+0.01*(frng[[2,2]]-frng[[2,1]])}};

xticks=LinearTicks[frng[[1]],"Original"][[1]];
yticks=LinearTicks[frng[[2]],"Engeenering"];
count=yticks[[2]];
yticks=yticks[[1]];

Show[Plot[bthf[x]+uns[x],{x,frng[[1,1]],frng[[1,2]]},
          PlotPoints->200,
          PlotStyle->RGBColor[1,0,0],
          DisplayFunction->Identity],
     ListPlot[lst,
              PlotStyle->PointSize[0.01],
              DisplayFunction->Identity],
     Axes->False,Frame->{True,True,False,False},
     PlotRange->frng,
     ImageSize->500,
     FrameTicks->{xticks,yticks},
     FrameLabel->{"scaled magnetic field",
                  StringJoin["magnetization, 1e",
                             ToString[Log[10,count]]],
                  None,None},
     RotateLabel->True,
     PlotLabel->StyleForm[StringJoin["HYPERBOLIC MODEL (red). Mean sd: ",
                                     Prn[100*res,3],
                                     " %"],
                          FontFamily->"Geneva",FontSize->16],
     DisplayFunction->$DisplayFunction];

ord=InputCheck["Accept the field scaling exponent (y/n)?",
               {"Expression"},
               {{String,{"y","n"}}}];
If[ord==="y",scfail=False]

     ]; (* Close While scfail *)


(* Subtract the unsaturated part of the curve *)
lstu=lst;
lst=Table[{lst[[k,1]],lst[[k,2]]-uns[lst[[k,1]]]},
          {k,1,l1}];


(* Rescale magnetization *)
Print["Search a scale for the magnetization. Please wait..."];
yrng=Table[lst[[k,2]],{k,1,l1}];
yrng={Max[yrng],Min[yrng]};
xrng=Sort[Table[lst[[k,1]],{k,1,l1}]];
dx=Table[xrng[[k+1]]-xrng[[k]],{k,1,l1-1}];
dx={Min[dx],Max[dx]};

ft=HypLin[lst,bft,
          {{yrng[[1]]+0.0000001*(yrng[[2]]-yrng[[1]]),
            yrng[[1]]+Abs[bft[[3]]+bft[[4]]-yrng[[1]]]},
           {yrng[[2]]-0.0000001*(yrng[[2]]-yrng[[1]]),
            yrng[[2]]-Abs[bft[[4]]-yrng[[2]]]}},
          weights];

thf[x_]:=ft[[3]]*(0.5000000001-0.5*
         Tanh[ft[[5]]*ArcSinh[ft[[1]]*(x-ft[[2]])/ft[[5]]]])^ft[[6]]+
         ft[[4]];

dthf[x_]:=D[thf[y],y]/.(y->x);
ddthf[x_]:=D[dthf[y],{y,2}]/.(y->x);
ithf[y_]:=ft[[5]]*
          Sinh[ArcTanh[1.0000000002-2*((y-ft[[4]])/ft[[3]])^(1/ft[[6]])]/
          ft[[5]]]/ft[[1]]+ft[[2]];
dithf[y_]:=D[ithf[x],x]/.(x->y);
dibthf[y_]:=D[ithf[bthf[x]],x]/.(x->y);

res01=Table[{lst[[k,1]],ithf[lst[[k,2]]]-lst[[k,1]]},
            {k,1,l1}];


(* Subtract polynomial, 1st trial *)
Print["Estimate the measurement errors. Please wait..."];
err01=(Sum[res01[[k,2]]^2,{k,1,l1}]-
       Sum[res01[[k,2]]*res01[[k+5,2]],{k,1,l1-5}])/(l1-5);
err01=Sqrt[Abs[err01]];
err01=Table[err01,{k,1,l1}];

ord=FindExtremes[res01,err01];
ord=Min[ord,9];
poly=Fit[res01,Prepend[Table[x^j,{j,1,ord}],1],x];
dpoly=D[poly,x];
res02=Table[{res01[[k,1]],res01[[k,2]]-poly/.(x->res01[[k,1]])},
            {k,1,l1}];


(* Fourier analysis, 1st trial *)
rsexpn=AutoCorrelation[res02,False];
cll=rsexpn[[2]];

count=Max[Table[Abs[res02[[k+1]]-res02[[k]]],{k,1,l1-1}]];
cl=Max[count,Min[cll[[1]],If[cll[[2]]==0,0,0.25/cll[[2]]]]];
rsexpn=rsexpn[[1]];

qr=res02[[2,1]]-res02[[1,1]];
x0r=res02[[1,1]];
res03=Table[{(res02[[k,1]]+qr)^rsexpn-(x0r+qr)^rsexpn,
             res02[[k,2]]},
            {k,1,l1}];
xrng={First[res03][[1]],Last[res03][[1]]};
dx=Table[res03[[k+1,1]]-res03[[k,1]],{k,1,l1-1}];
dx={Min[dx],Max[dx]};


(* Fitting residuals with empirical correlation function, 1st trial *)
res04=AutoCollocation[res03,cll,cl,xrng,
                      Max[Min[dx[[1]]*0.25,cl/4],(xrng[[2]]-xrng[[1]])/100]];
res04i=res04[[2]];
res04=res04[[1]];


(* Estimate measurement errors, 1st trial *)
err01=Table[Sum[Abs[res03[[j,2]]-res04[[j,2]]]*
                Exp[-0.25*(res03[[j,1]]-res03[[k,1]])^2/cl^2],
                {j,1,l1}]/
            Sum[Exp[-0.25*(res03[[j,1]]-res03[[k,1]])^2/cl^2],
                {j,1,l1}],
            {k,1,l1}];


(* Subtract polynomial, definitive *)
Print["Calculate the residuals."];
ord=FindExtremes[res01,3*err01];
ord=Min[ord,9];
Print[StringJoin["Subtract residuals trend with a ",
                 ToString[ord],
                 Which[ord==1,
                       "st",
                       ord==2,
                       "nd",
                       ord==3,
                       "rd",
                       True,
                       "th"],
                 " order polynomial."]];
poly=Fit[res01,Prepend[Table[x^j,{j,1,ord}],1],x];
dpoly=D[poly,x];
res02=Table[{res01[[k,1]],res01[[k,2]]-poly/.(x->res01[[k,1]])},
            {k,1,l1}];


(* Fourier analysis, definitive *)
rsexpn=AutoCorrelation[res02,True];
cll=rsexpn[[2]];

count=Max[Table[Abs[res02[[k+1]]-res02[[k]]],{k,1,l1-1}]];
cl=Max[count,Min[cll[[1]],If[cll[[2]]==0,0,0.25/cll[[2]]]]];
rsexpn=rsexpn[[1]];

qr=res02[[2,1]]-res02[[1,1]];
x0r=res02[[1,1]];
res03=Table[{(res02[[k,1]]+qr)^rsexpn-(x0r+qr)^rsexpn,
             res02[[k,2]]},
            {k,1,l1}];
xrng={First[res03][[1]],Last[res03][[1]]};
dx=Table[res03[[k+1,1]]-res03[[k,1]],{k,1,l1-1}];
dx={Min[dx],Max[dx]};
StylePrint[StringJoin["Estimated lower limit of the autocorrelation
distance: ",
                      Prn[cl,3]],
           FontColor->RGBColor[0,0,1]];


(* Fitting residuals with empirical correlation function, 1st trial *)
Print["Fit the residuals with an autocorrelation model."];
res04=AutoCollocation[res03,cll,cl,xrng,
                      Max[Min[dx[[1]]*0.25,cl/4],(xrng[[2]]-xrng[[1]])/100]];
res04i=res04[[2]];
res04=res04[[1]];


(* Estimate measurement errors, definitive *)
err02=Table[{res03[[k,1]],
             Sum[Abs[res03[[j,2]]-res04[[j,2]]]*
                 Exp[-0.25*(res03[[j,1]]-res03[[k,1]])^2/cl^2],
                 {j,1,l1}]/
             Sum[Exp[-0.25*(res03[[j,1]]-res03[[k,1]])^2/cl^2],
                 {j,1,l1}]},
             {k,1,l1}];

null=Join[{0.5*Abs[res03[[1,2]]-res04[[1,2]]]/err02[[1,2]]},
          Table[Abs[res03[[k,2]]-res04[[k,2]]]/err02[[k,2]],{k,2,l1-1}],
          {0.5*Abs[res03[[l1,2]]-res04[[l1,2]]]/err02[[l1,2]]}];
null=Position[null,_?(#>3&)];
If[Length[null]>0,
   resnull=Extract[res03,null];
   StylePrint["WARNING! CODICA detected some outlyers.",
              FontColor->RGBColor[1,0,0]],
   resnull={Null}];

err02=Interpolation[err02];
err03l=Table[{res04i[[k,1]],res04i[[k,2]]-2*err02[res04i[[k,1]]]},
             {k,1,Length[res04i]}];
err03h=Table[{res04i[[k,1]],res04i[[k,2]]+2*err02[res04i[[k,1]]]},
             {k,1,Length[res04i]}];


(* Plot residuals *)
resrng={Table[res03[[k,1]],{k,1,l1}],
        Table[res03[[k,2]],{k,1,l1}],
        Table[res04i[[k,2]],{k,1,Length[res04i]}],
        Table[err03l[[k,2]],{k,1,Length[res04i]}],
        Table[err03h[[k,2]],{k,1,Length[res04i]}]};
resrng={{Min[resrng[[1]]],Max[resrng[[1]]]},
        {Min[resrng[[2]]],Max[resrng[[2]]]},
        {Min[resrng[[3]]],Max[resrng[[3]]]},
        {Min[resrng[[4]]],Max[resrng[[4]]]},
        {Min[resrng[[5]]],Max[resrng[[5]]]}};
resrng={{resrng[[1,1]]-0.01*(resrng[[1,2]]-resrng[[1,1]]),
         resrng[[1,2]]+0.01*(resrng[[1,2]]-resrng[[1,1]])},
        {resrng[[2,1]]-0.02*(resrng[[2,2]]-resrng[[2,1]]),
         resrng[[2,2]]+0.02*(resrng[[2,2]]-resrng[[2,1]])},
        {resrng[[3,1]]-0.01*(resrng[[3,2]]-resrng[[3,1]]),
         resrng[[3,2]]+0.01*(resrng[[3,2]]-resrng[[3,1]])},
        resrng[[4]],
        resrng[[5]]};
resrng={resrng[[1]],
        {Min[Min[resrng[[2,1]],resrng[[3,1]]],
             Min[resrng[[4,1]],resrng[[5,1]]]],
         Max[Max[resrng[[2,2]],resrng[[3,2]]],
             Max[resrng[[4,2]],resrng[[5,2]]]]}};

Which[l1>160,
      numbers=PutNumbers[res03,resrng,20];
      res03blue=Table[res03[[k]],{k,20,l1,20}],
      l1>20,
      numbers=PutNumbers[res03,resrng,10];
      res03blue=Table[res03[[k]],{k,10,l1,10}],
      True,
      numbers={};
      res03blue={}];

Show[Graphics[{GrayLevel[0.7],Polygon[Join[err03l,Reverse[err03h]]]}],
     ListPlot[res04i,PlotJoined->True,DisplayFunction->Identity],
     ListPlot[res03,PlotStyle->PointSize[0.01],
              DisplayFunction->Identity],
     ListPlot[res03blue,PlotStyle->{PointSize[0.01],RGBColor[0.4,0.4,1]},
              DisplayFunction->Identity],
     ListPlot[resnull,
              PlotStyle->{PointSize[0.014],RGBColor[1,0,0]},
              DisplayFunction->Identity],
     Graphics[numbers], 
     DisplayFunction->$DisplayFunction,
     PlotRange->resrng,
     Frame->{True,True,False,False},
     FrameLabel->{"scaled magnetic field","residuals",None,None},
     RotateLabel->True,
     PlotLabel->StyleForm[
     If[Length[null]>0,
        "RESIDUAL CURVE (estimated error in grey, outlyers in red):",
        "RESIDUAL CURVE (estimated error in grey):"],
                          FontFamily->"Geneva",FontSize->20],
     ImageSize->800];


(* Remove outlyers *)
If[Length[null]>0,
   If[l1-Length[null]>8,
      bool=InputCheck["Remove the detected outlyers? (y/n)",
                      {"Expression"},
                      {{String,{"y","n"}}}];
      If[bool==="y",
         StylePrint["Outlyers will be removed and the cleaned measurements
will be analyzed again.",
                    FontColor->RGBColor[1,0,0]];
         list=Extract[list,
                      Complement[Table[{k},{k,1,l1}],null]];
         rempoints=True,
         rempoints=False],
      StylePrint["Outlyers cannot be removed (not enough measurements)!
\nCODICA will continue using all measurements.",
                 FontColor->RGBColor[1,0,0]];
      rempoints=False],
   rempoints=False]

     ]; (* Close While rempoints *)


(* Filter residuals with collocation *)
xrng={First[res03][[1]],Last[res03][[1]]};
dx=Table[res03[[k+1,1]]-res03[[k,1]],{k,1,l1-1}];
dx={Min[dx],Max[dx]};

coll=True;
While[coll,

If[cll[[4]]/(cll[[3]]+cll[[4]])<0.5,
cd=InputCheck[StringJoin["Enter the correlation length of the residual curve.
\n\nSuggested values are around ",
                         Prn[cl,3],
                         "\nRange of possible values: ",
                         Prn[0.5*dx[[2]],2],
                         " - ",
                         Prn[3*(xrng[[2]]-xrng[[1]]),2],
                         "\n\nUse the largest value that gives a fit within
the error margins. ",
                         Prn[3*(xrng[[2]]-xrng[[1]]),2],
                         " gives a linear fit."],
              {"Expression"},
              {{Real,{0.99*SetPrecision[0.5*dx[[2]],2],
                      1.01*SetPrecision[3(xrng[[2]]-xrng[[1]]),2]}}}],
cd=InputCheck[StringJoin["Enter the correlation length of the residual curve.
\n\nThe residuals are very noisy. Suggested MINIMUN value for a significant 
fit: ",
                         Prn[cl,3],
                         "\nRange of possible values: ",
                         Prn[0.5*dx[[2]],2],
                         " - ",
                         Prn[3*(xrng[[2]]-xrng[[1]]),2],
                         "\n\nUse the largest value that gives a fit within
the error margins. ",
                         Prn[3*(xrng[[2]]-xrng[[1]]),2],
                         " gives a linear fit."],
              {"Expression"},
              {{Real,{0.99*SetPrecision[0.5*dx[[2]],2],
                      1.01*SetPrecision[3(xrng[[2]]-xrng[[1]]),2]}}}]
  ]; (*Close If cll *)

Print[StringJoin["Filter the residual curve using least-squares collocation
with correlation length: ",
                 Prn[cd,3]]];

res05=Collocation[res03,res04,cd,xrng,
                  Max[Min[dx[[1]]*0.25,cl/4],(xrng[[2]]-xrng[[1]])/100],
                  Table[((xlist[[k]]+q)^expn-(x0+q)^expn+qr)^rsexpn-
                        (x0r+qr)^rsexpn,{k,1,Length[xlist]}]];
err05=res05[[3]];
res05f=res05[[2]];
nmatrix=res05[[4]];
err05f=res05[[5]];
res05=res05[[1]];

err05l=Table[{res05f[[k,1]],res05f[[k,2]]-err05f[[k]]},{k,1,Length[res05f]}];
err05h=Table[{res05f[[k,1]],res05f[[k,2]]+err05f[[k]]},{k,1,Length[res05f]}];

resrng={Table[res03[[k,1]],{k,1,l1}],
        Table[res03[[k,2]],{k,1,l1}],
        Table[res05f[[k,2]],{k,1,Length[res05f]}],
        Table[err05l[[k,2]],{k,1,Length[res05f]}],
        Table[err05h[[k,2]],{k,1,Length[res05f]}]};
resrng={{Min[resrng[[1]]],Max[resrng[[1]]]},
        {Min[resrng[[2]]],Max[resrng[[2]]]},
        {Min[resrng[[3]]],Max[resrng[[3]]]},
        {Min[resrng[[4]]],Max[resrng[[4]]]},
        {Min[resrng[[5]]],Max[resrng[[5]]]}};
resrng={{resrng[[1,1]]-0.01*(resrng[[1,2]]-resrng[[1,1]]),
         resrng[[1,2]]+0.01*(resrng[[1,2]]-resrng[[1,1]])},
        {resrng[[2,1]]-0.01*(resrng[[2,2]]-resrng[[2,1]]),
         resrng[[2,2]]+0.01*(resrng[[2,2]]-resrng[[2,1]])},
        {resrng[[3,1]]-0.01*(resrng[[3,2]]-resrng[[3,1]]),
         resrng[[3,2]]+0.01*(resrng[[3,2]]-resrng[[3,1]])},
        resrng[[4]],
        resrng[[5]]};
resrng={resrng[[1]],
        {Min[resrng[[2,1]],resrng[[3,1]],
             resrng[[4,1]],resrng[[5,1]]],
         Max[resrng[[2,2]],resrng[[3,2]],
             resrng[[4,2]],resrng[[5,2]]]}};


Show[Graphics[{GrayLevel[0.7],Polygon[Join[err05l,Reverse[err05h]]]}],
     ListPlot[res05f,PlotJoined->True,DisplayFunction->Identity,
              PlotStyle->RGBColor[1,0,0]],
     ListPlot[res03,PlotStyle->PointSize[0.01],
              DisplayFunction->Identity],
     ListPlot[res03blue,PlotStyle->{PointSize[0.01],RGBColor[0.4,0.4,1]},
              DisplayFunction->Identity],
     Graphics[numbers], 
     DisplayFunction->$DisplayFunction,
     PlotRange->resrng,
     Frame->{True,True,False,False},
     FrameLabel->{"scaled magnetic field","residuals",None,None},
     RotateLabel->True,
     PlotLabel->StyleForm[StringJoin["FILTERED RESIDUAL CURVE with cl = ",
                                     Prn[cd,3],
                                     " (estimated error in grey):"],
                          FontFamily->"Geneva",FontSize->20],
     ImageSize->800];


bool=InputCheck["Accept the filtered residuals? (y/n)",
                {"Expression"},
                {{String,{"y","n"}}}];
If[bool==="y",
   coll=False,coll=True]
     ]; (*Close While coll*)


(* Calcuate the fitted measurement curve *)
If[maq,
   Print["Calculate the fitted magnetization curve..."],
   Print["Calculate the fitted demagnetization curve..."]];
l2=Length[res05f];
l3=Length[xlist];
fit=Table[{xlist[[k]],
           uns[(res05[[k,1]]+(x0r+qr)^rsexpn)^(1/rsexpn)-qr]+
           thf[res05[[k,2]]+
               (res05[[k,1]]+(x0r+qr)^rsexpn)^(1/rsexpn)-qr+
               (poly/.(x->((res05[[k,1]]+(x0r+qr)^rsexpn)^(1/rsexpn)-qr)))]},
          {k,1,l3}];
fiterr=
Table[{xlist[[k]],
       Abs[dthf[res05[[k,2]]+
                (res05[[k,1]]+(x0r+qr)^rsexpn)^(1/rsexpn)-qr+
                (poly/.(x->((res05[[k,1]]+(x0r+qr)^rsexpn)^(1/rsexpn)-qr)))]]*
       err05[[k]]},
      {k,1,l3}];
fitlogerr=Table[{fiterr[[k,1]],Log[10,fiterr[[k,2]]]},{k,1,l3}];

fitf=Table[{((res05f[[k,1]]+(x0r+qr)^rsexpn)^(1/rsexpn)-qr+
             (x0+q)^expn)^(1/expn)-q,
            uns[(res05f[[k,1]]+(x0r+qr)^rsexpn)^(1/rsexpn)-qr]+
            thf[res05f[[k,2]]+
                (res05f[[k,1]]+(x0r+qr)^rsexpn)^(1/rsexpn)-qr+
                (poly/.(x->((res05f[[k,1]]+(x0r+qr)^rsexpn)^(1/rsexpn)-qr)))]},
           {k,1,l2}];
fiterrf=
Table[{((res05f[[k,1]]+(x0r+qr)^rsexpn)^(1/rsexpn)-qr+
        (x0+q)^expn)^(1/expn)-q,
       Abs[dthf[res05f[[k,2]]+
                (res05f[[k,1]]+(x0r+qr)^rsexpn)^(1/rsexpn)-qr+
              (poly/.(x->((res05f[[k,1]]+(x0r+qr)^rsexpn)^(1/rsexpn)-qr)))]]*
       err05f[[k]]},
      {k,1,l2}];
fitlogerrf=Table[{fiterrf[[k,1]],Log[10,fiterrf[[k,2]]]},{k,1,l2}];

xticks=LinearTicks[orng[[1]],"Original"][[1]];
yticks=LinearTicks[orng[[2]],"Engeenering"];
count=yticks[[2]];
yticks=yticks[[1]];
labels={StyleBox[RowBox[{"magnetic ","field"}],
                 FontFamily->"Courier",FontSize->16]//DisplayForm,
        StyleBox[RowBox[{"magnetization , ",
                         SuperscriptBox["10",
                                        ToString[Log[10,count]]],
                         " units"}],
                 FontFamily->"Courier",FontSize->16]//DisplayForm,
        None,None};
Show[ListPlot[ref,
              PlotStyle->PointSize[0.01],
              DisplayFunction->Identity],
     ListPlot[fitf,
              PlotJoined->True,
              PlotStyle->RGBColor[1,0,0],
              DisplayFunction->Identity],
     Axes->False,Frame->{True,True,False,False},
     PlotRange->orng,
     ImageSize->800,
     FrameTicks->{xticks,yticks,None,None},
     FrameLabel->labels,
     RotateLabel->True,
     PlotLabel->StyleForm["MEASUREMENTS (points) WITH BEST-FIT (red):",
                          FontFamily->"Geneva",FontSize->20],
     DisplayFunction->$DisplayFunction];

errrng={Log[10,Min[Table[fiterrf[[k,2]],{k,1,l2}]]],
        Log[10,Max[Table[fiterrf[[k,2]],{k,1,l2}]]]};
If[errrng[[2]]-errrng[[1]]>3,
   errrng={errrng[[2]]-3,errrng[[2]]}];
errrng={errrng[[1]]-0.01*(errrng[[2]]-errrng[[1]]),
        errrng[[2]]+0.01*(errrng[[2]]-errrng[[1]])};
yticks=LogTicks[errrng];
errrng={orng[[1]],errrng};
labels={StyleBox[RowBox[{"magnetic ","field"}],
                 FontFamily->"Courier",FontSize->16]//DisplayForm,
        StyleBox["error",
                 FontFamily->"Courier",FontSize->16]//DisplayForm,
        None,None};
ListPlot[fitlogerrf,
         PlotJoined->True,
         PlotStyle->Thickness[0.003],
         Axes->False,Frame->{True,True,False,False},
         PlotRange->errrng,
         FrameTicks->{xticks,yticks,None,None},
         FrameLabel->labels,
         RotateLabel->True,
         PlotLabel->StyleForm["ESTIMATED BEST-FIT ERROR:",
                              FontFamily->"Geneva",FontSize->20],
         AspectRatio->0.3,
         ImageSize->922,PlotRegion->{{0,0.86},{0,1}}];


(* Calculate derivatives *)
Print["Calculate the coercivity distribution. Please wait..."];
count=Max[Min[dx[[1]]*0.25,cl/4],(xrng[[2]]-xrng[[1]])/100];
dres=DCollocation[res03,res04,cd,cl,
                  Table[k,{k,xrng[[1]],xrng[[2]]+0.1*count,count}],
                  nmatrix];
res06=dres[[1]];
derr=dres[[3]];
dres=dres[[2]];
derr=Table[{dres[[k,1]],derr[[k]]},{k,1,Length[dres]}];
l2=Length[dres];

derlog=
Table[{Log[10,((res06[[k,1]]+(x0r+qr)^rsexpn)^(1/rsexpn)-qr+
               (x0+q)^expn)^(1/expn)-q],
       
       (duns[(res06[[k,1]]+(x0r+qr)^rsexpn)^(1/rsexpn)-qr]+
        
        dthf[res06[[k,2]]+
             (res06[[k,1]]+(x0r+qr)^rsexpn)^(1/rsexpn)-qr+
             (poly/.(x->((res06[[k,1]]+(x0r+qr)^rsexpn)^(1/rsexpn)-qr)))]*
        (dres[[k,2]]*rsexpn*(res06[[k,1]]+(x0r+qr)^rsexpn)^(1-1/rsexpn)+
         1+(dpoly/.(x->((res06[[k,1]]+(x0r+qr)^rsexpn)^(1/rsexpn)-qr)))))*
       
       expn*((res06[[k,1]]+(x0r+qr)^rsexpn)^(1/rsexpn)-qr+
             (x0+q)^expn)^(1-1/expn)*
        
       Log[10]*
       (((res06[[k,1]]+(x0r+qr)^rsexpn)^(1/rsexpn)-qr+(x0+q)^expn)^(1/expn)-q)
      },
      {k,1,l2}];

If[Not[maq],
   derlog=Table[{derlog[[k,1]],-derlog[[k,2]]},{k,1,l2}]];

mono=False;
If[Sort[derlog,#1[[2]]<#2[[2]]&][[1,2]]<0,
   StylePrint["WARNING! The estimated magnetization curve is not strictly
monotonic.",
              FontColor->RGBColor[1,0,0]];
   mono=InputCheck["WARNING! The coercivity distribution is not positive over
the entire range of fields.
\n\nSet negative values to zero (y/n)",
                   {"Expression"},
                   {{String,{"y","n"}}}];
   If[mono==="y",mono=True]];

If[mono,
   derlog=Table[{derlog[[k,1]],Max[0,derlog[[k,2]]]},{k,1,l2}]];

derlogerr=
Table[{Log[10,((res06[[k,1]]+(x0r+qr)^rsexpn)^(1/rsexpn)-qr+
               (x0+q)^expn)^(1/expn)-q],
       
       Abs[dthf[res06[[k,2]]+
                (res06[[k,1]]+(x0r+qr)^rsexpn)^(1/rsexpn)-qr+
                (poly/.(x->((res06[[k,1]]+(x0r+qr)^rsexpn)^(1/rsexpn)-qr)))]*
           derr[[k,2]]*rsexpn*(res06[[k,1]]+(x0r+qr)^rsexpn)^(1-1/rsexpn)*
           
           expn*((res06[[k,1]]+(x0r+qr)^rsexpn)^(1/rsexpn)-qr+
                 (x0+q)^expn)^(1-1/expn)]*
       
       expn*Log[10]*
       (((res06[[k,1]]+(x0r+qr)^rsexpn)^(1/rsexpn)-qr+(x0+q)^expn)^(1/expn)-q)
      },
      {k,1,l2}];

derlin=
Table[{((res06[[k,1]]+(x0r+qr)^rsexpn)^(1/rsexpn)-qr+(x0+q)^expn)^(1/expn)-q,
       
       (duns[(res06[[k,1]]+(x0r+qr)^rsexpn)^(1/rsexpn)-qr]+
        
        dthf[res06[[k,2]]+
             (res06[[k,1]]+(x0r+qr)^rsexpn)^(1/rsexpn)-qr+
             (poly/.(x->((res06[[k,1]]+(x0r+qr)^rsexpn)^(1/rsexpn)-qr)))]*
        (dres[[k,2]]*rsexpn*(res06[[k,1]]+(x0r+qr)^rsexpn)^(1-1/rsexpn)+
         1+(dpoly/.(x->((res06[[k,1]]+(x0r+qr)^rsexpn)^(1/rsexpn)-qr)))))*
       
       expn*((res06[[k,1]]+(x0r+qr)^rsexpn)^(1/rsexpn)-qr+
             (x0+q)^expn)^(1-1/expn)
      },
      {k,1,l2}];

If[Not[maq],
   derlin=Table[{derlin[[k,1]],-derlin[[k,2]]},{k,1,l2}]];
If[mono,
   derlin=Table[{derlin[[k,1]],Max[0,derlin[[k,2]]]},{k,1,l2}]];

derlinerr=
Table[{((res06[[k,1]]+(x0r+qr)^rsexpn)^(1/rsexpn)-qr+(x0+q)^expn)^(1/expn)-q,
       
       Abs[dthf[res06[[k,2]]+
                (res06[[k,1]]+(x0r+qr)^rsexpn)^(1/rsexpn)-qr+
                (poly/.(x->((res06[[k,1]]+(x0r+qr)^rsexpn)^(1/rsexpn)-qr)))]*
           derr[[k,2]]*rsexpn*(res06[[k,1]]+(x0r+qr)^rsexpn)^(1-1/rsexpn)*
           
           expn*((res06[[k,1]]+(x0r+qr)^rsexpn)^(1/rsexpn)-qr+
                 (x0+q)^expn)^(1-1/expn)]
      },
      {k,1,l2}];

If[Chop[((res06[[1,1]]+(x0r+qr)^rsexpn)^(1/rsexpn)-qr+
         (x0+q)^expn)^(1/expn)-q]==0,
   count=2,
   count=1];
bool=Max[Table[derlog[[k,2]],{k,1,l2}]];
While[count<l2&&
      (derlog[[count,2]]/derlogerr[[count,2]]<1&&
       derlog[[count,2]]<0.001*bool),
      count++];
count=count-1;
derlog=Drop[derlog,count];
derlogerr=Drop[derlogerr,count];

logrng={Table[derlog[[k,1]],{k,1,Length[derlog]}],
        Table[derlog[[k,2]],{k,1,Length[derlog]}]};
logrng={{Min[logrng[[1]]],Max[logrng[[1]]]},
        {Min[0,Min[logrng[[2]]]],1.01*Max[logrng[[2]]]}};
xticks=LogTicks[logrng[[1]]];
yticks=LinearTicks[logrng[[2]],"Engeenering"];
count=yticks[[2]];
yticks=yticks[[1]];

labels={StyleBox[RowBox[{"magnetic ","field"}],
                 FontFamily->"Courier",FontSize->16]//DisplayForm,
        StyleBox[RowBox[{"magnetization , ",
                         SuperscriptBox["10",
                                        ToString[Log[10,count]]],
                         " units"}],
                 FontFamily->"Courier",FontSize->16]//DisplayForm,
        None,None};

l2=Length[derlogerr];
count=1;
While[count<l2&&
      derlogerr[[count,2]]/derlog[[count,2]]>1,
      count++];
Which[count==l2,
      sig=None,
      count==1,
      sig={1},
      True,
      sig={count}];
If[Not[sig===None],
   count=l2;
   While[count>1&&
         derlogerr[[count,2]]/derlog[[count,2]]>1,
         count--];
   If[count==l2,
      sig=Append[sig,l2],
      sig=Append[sig,count]]];

count=1;
While[count<l2&&
      derlog[[count,2]]<0.0025*logrng[[2,2]],
      count++];
sug=count;
count=l2;
While[count>1&&
      derlog[[count,2]]<0.0025*logrng[[2,2]],
      count--];
sug={sug,count};
sug={10^derlog[[sug[[1]],1]],
     10^derlog[[sug[[2]],1]]};

Show[ListPlot[derlog,
              PlotJoined->True,
              PlotStyle->Thickness[0.005],
              DisplayFunction->Identity],
     Axes->False,Frame->{True,True,False,False},
     PlotRange->logrng,
     ImageSize->500,
     FrameTicks->{xticks,yticks,None,None},
     FrameLabel->labels,
     RotateLabel->True,
     PlotLabel->StyleForm["COERCIVITY DISTRIBUTION ON A LOG SCALE:",
                          FontFamily->"Geneva",FontSize->16],
     DisplayFunction->$DisplayFunction];


StylePrint[StringJoin["Range of fields spanned by the
coercivity distribution: ",
                      Prn[sug[[1]],3],
                      " - ",
                      Prn[sug[[2]],3]],
           FontColor->RGBColor[0,0,1]];

Which[sig===None,
      StylePrint["WARNING! The coercivity distribution is not significant
over the entire range of fields.",
                 FontColor->RGBColor[1,0,0]],
      sig=={1,l2},
      StylePrint["The coercivity distribution is significant
over the entire displayed range of fields.",
                 FontColor->RGBColor[0,0,1]];
      sig={10^derlog[[sig[[1]],1]],
           10^derlog[[sig[[2]],1]]},
      True,
      sig={10^derlog[[sig[[1]],1]],
           10^derlog[[sig[[2]],1]]};
      StylePrint[StringJoin["The coercivity distribution is significant over
the field range ",
                            Prn[sig[[1]],3],
                            " - ",
                            Prn[sig[[2]],3]],
                 FontColor->RGBColor[0,0,1]]];


(* Calculate the logarithmic coercivity distribution *)
choose=True;
While[choose,
vrng=InputCheck[StringJoin["Choose an adequate field range for the
coercivity distribution.
\n\nType \"a\" to choose the entire range shown in the previous plot
or \"s\" to choose the range of fields spanned by the distribution.
\nTo enter a different range, type the range limits. Example: 1,300.
Maximum range: ",
                           Prn[10^First[derlog][[1]],3],
                           " - ",
                           Prn[10^Last[derlog][[1]],3]],
                   {"Expression","LongList"},
                   {{String,{"a","s"}},
                    {{2,2},Real,
                     {SetPrecision[10^First[derlog][[1]],3],
                      SetPrecision[10^Last[derlog][[1]],3]}}}];

Which[vrng==="a",
      vrng={10^First[derlog][[1]],10^Last[derlog][[1]]},
      vrng==="s",
      vrng=sug,
      True,
      vrng=Sort[vrng]];

nout=InputCheck[StringJoin["Enter the number of sampling points in the
chosen field range.
\n\nAt least ",
                           ToString[l1],
                           " points are recommanded."],
                {"Expression"},
                {{Integer,{1,Infinity}}}];


Print[StringJoin["Resample the coercivity distribution in the field range ",
                 Prn[vrng[[1]],3],
                 " - ",
                 Prn[vrng[[2]],3],
                 " ..."]];
lrng=vrng;
vrng=N[Log[10,vrng]];
vrng=Table[((10^k+q)^expn-(x0+q)^expn+qr)^rsexpn-(x0r+qr)^rsexpn,
           {k,vrng[[1]],vrng[[2]]+0.001*(vrng[[2]]-vrng[[1]]),
              (vrng[[2]]-vrng[[1]])/nout}];

dresl=DCollocation[res03,res04,cd,cl,
                   vrng,nmatrix];

resl=dresl[[1]];
derrl=dresl[[3]];
dresl=dresl[[2]];
derrl=Table[{dresl[[k,1]],derrl[[k]]},{k,1,Length[dresl]}];
l3=Length[dresl];

derlg=
Table[{Log[10,((resl[[k,1]]+(x0r+qr)^rsexpn)^(1/rsexpn)-qr+
               (x0+q)^expn)^(1/expn)-q],
       
       (duns[(resl[[k,1]]+(x0r+qr)^rsexpn)^(1/rsexpn)-qr]+
        
        dthf[resl[[k,2]]+
             (resl[[k,1]]+(x0r+qr)^rsexpn)^(1/rsexpn)-qr+
             (poly/.(x->((resl[[k,1]]+(x0r+qr)^rsexpn)^(1/rsexpn)-qr)))]*
        (dresl[[k,2]]*rsexpn*(resl[[k,1]]+(x0r+qr)^rsexpn)^(1-1/rsexpn)+
         1+(dpoly/.(x->((resl[[k,1]]+(x0r+qr)^rsexpn)^(1/rsexpn)-qr)))))*
       
        expn*((resl[[k,1]]+(x0r+qr)^rsexpn)^(1/rsexpn)-qr+
              (x0+q)^expn)^(1-1/expn)*
        
       Log[10]*
       (((resl[[k,1]]+(x0r+qr)^rsexpn)^(1/rsexpn)-qr+(x0+q)^expn)^(1/expn)-q)
      },
      {k,1,l3}];
If[Not[maq],
   derlg=Table[{derlg[[k,1]],-derlg[[k,2]]},{k,1,l3}]];
If[mono,
   derlg=Table[{derlg[[k,1]],Max[0,derlg[[k,2]]]},{k,1,l3}]];

derlgerr=
Table[{Log[10,((resl[[k,1]]+(x0r+qr)^rsexpn)^(1/rsexpn)-qr+
               (x0+q)^expn)^(1/expn)-q],
       
       Abs[dthf[resl[[k,2]]+
                (resl[[k,1]]+(x0r+qr)^rsexpn)^(1/rsexpn)-qr+
                (poly/.(x->((resl[[k,1]]+(x0r+qr)^rsexpn)^(1/rsexpn)-qr)))]*
           derrl[[k,2]]*rsexpn*(resl[[k,1]]+(x0r+qr)^rsexpn)^(1-1/rsexpn)*
           
           expn*((resl[[k,1]]+(x0r+qr)^rsexpn)^(1/rsexpn)-qr+
                 (x0+q)^expn)^(1-1/expn)]*
       
       expn*Log[10]*
       (((resl[[k,1]]+(x0r+qr)^rsexpn)^(1/rsexpn)-qr+(x0+q)^expn)^(1/expn)-q)
      },
      {k,1,l3}];

derlgl=Table[{derlg[[k,1]],derlg[[k,2]]-derlgerr[[k,2]]},
             {k,1,l3}];
derlgh=Table[{derlg[[k,1]],derlg[[k,2]]+derlgerr[[k,2]]},
             {k,1,l3}];

logrng={Table[derlg[[k,1]],{k,1,Length[derlg]}],
        Table[derlgh[[k,2]],{k,1,Length[derlg]}]};
logrng={{Min[logrng[[1]]],Max[logrng[[1]]]},
        {Min[0,Min[logrng[[2]]]],Max[logrng[[2]]]}};
xticks=LogTicks[logrng[[1]]];

sig=2*Max[Table[derlgerr[[k,2]],{k,1,l3}]]/logrng[[2,2]];
If[sig<0.14,
   logrng={logrng[[1]],
           {0,1.01*Max[Table[derlgh[[k,2]],{k,1,l3}]]}};
   yticks=LinearTicks[logrng[[2]],"Engeenering"];
   count=yticks[[2]];
   yticks=yticks[[1]];
   labels={StyleBox[RowBox[{"magnetic ","field"}],
                    FontFamily->"Courier",FontSize->16]//DisplayForm,
           StyleBox[RowBox[{"magnetization , ",
                            SuperscriptBox["10",
                                           ToString[Log[10,count]]],
                            " units"}],
                    FontFamily->"Courier",FontSize->16]//DisplayForm,
           None,None};
   Print["The thickness of the curve in the following plot corresponds
to the estimated error (2 sd):"];
   Show[Graphics[Polygon[Join[derlgl,Reverse[derlgh]]]],
        ListPlot[derlg,
                 PlotJoined->True,
                 PlotStyle->{Thickness[0.001],GrayLevel[0]},
                 DisplayFunction->Identity],
        Axes->False,Frame->{True,True,False,False},
        PlotRange->logrng,
        ImageSize->800,
        FrameTicks->{xticks,yticks,None,None},
        FrameLabel->labels,
        RotateLabel->True,
        PlotLabel->StyleForm["COERCIVITY DISTRIBUTION ON A LOG SCALE:",
                             FontFamily->"Geneva",FontSize->20],
        DisplayFunction->$DisplayFunction],
   
   logrng={logrng[[1]],
           {0,1.01*Min[4*Max[Table[derlg[[k,2]],{k,1,l3}]],
                       Max[Table[derlgh[[k,2]],{k,1,l3}]]]}};
   yticks=LinearTicks[logrng[[2]],"Engeenering"];
   count=yticks[[2]];
   yticks=yticks[[1]];
   labels={StyleBox[RowBox[{"magnetic ","field"}],
                    FontFamily->"Courier",FontSize->16]//DisplayForm,
           StyleBox[RowBox[{"magnetization , ",
                            SuperscriptBox["10",
                                           ToString[Log[10,count]]],
                            " units"}],
                    FontFamily->"Courier",FontSize->16]//DisplayForm,
           None,None};
   bool=Flatten[Table[{GrayLevel[j],
                       Polygon[Join[
                               Table[{derlg[[k,1]],
                                      derlg[[k,2]]-j*derlgerr[[k,2]]},
                                     {k,1,l3}],
                               Reverse[
                               Table[{derlg[[k,1]],
                                      derlg[[k,2]]+j*derlgerr[[k,2]]},
                                     {k,1,l3}]]]]},
                      {j,1,0,-0.02}],1];
   Print["The gray band around the curve in the following plot corresponds
to the estimated error (2 sd)"];
   Show[Graphics[bool],
        ListPlot[derlg,
                 PlotJoined->True,
                 PlotStyle->{Thickness[0.006],GrayLevel[0]},
                 DisplayFunction->Identity],
        ListPlot[derlg,
                 PlotJoined->True,
                 PlotStyle->{Thickness[0.001],GrayLevel[1]},
                 DisplayFunction->Identity],
        Axes->False,Frame->{True,True,False,False},
        PlotRange->logrng,
        ImageSize->800,
        FrameTicks->{xticks,yticks,None,None},
        FrameLabel->labels,
        RotateLabel->True,
        PlotLabel->StyleForm["COERCIVITY DISTRIBUTION ON A LOG SCALE:",
                             FontFamily->"Geneva",FontSize->20],
        DisplayFunction->$DisplayFunction]];

derlglogerr=Table[{derlgerr[[k,1]],Log[10,derlgerr[[k,2]]]},
                  {k,1,Length[derlgerr]}];

errrng={Log[10,Min[Table[derlgerr[[k,2]],{k,1,Length[derlgerr]}]]],
        Log[10,Max[Table[derlgerr[[k,2]],{k,1,Length[derlgerr]}]]]};
If[errrng[[2]]-errrng[[1]]>3,
   errrng={errrng[[2]]-3,errrng[[2]]}];
errrng={errrng[[1]]-0.01*(errrng[[2]]-errrng[[1]]),
        errrng[[2]]+0.01*(errrng[[2]]-errrng[[1]])};
yticks=LogTicks[errrng];
errrng={logrng[[1]],errrng};
labels={StyleBox[RowBox[{"magnetic ","field"}],
                 FontFamily->"Courier",FontSize->16]//DisplayForm,
        StyleBox["error",
                 FontFamily->"Courier",FontSize->16]//DisplayForm,
        None,None};
ListPlot[derlglogerr,
         PlotJoined->True,
         PlotStyle->Thickness[0.003],
         Axes->False,Frame->{True,True,False,False},
         PlotRange->errrng,
         FrameTicks->{xticks,yticks,None,None},
         FrameLabel->labels,
         RotateLabel->True,
         PlotLabel->StyleForm["ESTIMATED ERROR:",
                              FontFamily->"Geneva",FontSize->20],
         AspectRatio->0.3,
         ImageSize->922,PlotRegion->{{0,0.86},{0,1}}];

choose=InputCheck["Choose a different range of fields? (y/n)",
                   {"Expression"},
                   {{String,{"y","n"}}}];
If[choose==="y",
   choose=True,
   choose=False]

     ]; (* Close While choose *)


(* Calculate the linear coercivity distribution *)
linear=InputCheck["Calculate the coercivity distribution on a
linear scale (y/n)",
                   {"Expression"},
                   {{String,{"y","n"}}}];
If[linear==="y",
   linear=True,
   linear=False];

If[linear,

linrng={Table[derlin[[k,1]],{k,1,Length[derlin]}],
        Table[derlin[[k,2]],{k,1,Length[derlin]}]};
linrng={{Min[linrng[[1]]],Max[linrng[[1]]]},
        {Min[0,Min[linrng[[2]]]],1.01*Max[linrng[[2]]]}};
xticks=LinearTicks[linrng[[1]],"Original"][[1]];
yticks=LinearTicks[linrng[[2]],"Engeenering"];
count=yticks[[2]];
yticks=yticks[[1]];
labels={StyleBox[RowBox[{"magnetic ","field"}],
                 FontFamily->"Courier",FontSize->16]//DisplayForm,
        StyleBox[RowBox[{"magnetization , ",
                         SuperscriptBox["10",
                                        ToString[Log[10,count]]],
                         " units"}],
                 FontFamily->"Courier",FontSize->16]//DisplayForm,
        None,None};

count=1;
While[count<l2&&
      derlin[[count,2]]<0.0025*linrng[[2,2]],
      count++];
If[count==2&&ref[[1,1]]==0,
   sug=1,
   sug=count];
count=l2;
While[count>1&&
      derlin[[count,2]]<0.0025*linrng[[2,2]],
      count--];
sug={sug,count};
sug={Chop[derlin[[sug[[1]],1]]],
     derlin[[sug[[2]],1]]};

Show[ListPlot[derlin,
              PlotJoined->True,
              PlotStyle->Thickness[0.005],
              DisplayFunction->Identity],
     Axes->False,Frame->{True,True,False,False},
     PlotRange->linrng,
     ImageSize->500,
     FrameTicks->{xticks,yticks,None,None},
     FrameLabel->labels,
     RotateLabel->True,
     PlotLabel->StyleForm["COERCIVITY DISTRIBUTION ON A LINEAR SCALE:",
                          FontFamily->"Geneva",FontSize->16],
     DisplayFunction->$DisplayFunction];

StylePrint[StringJoin["Range of fields spanned by the
coercivity distribution: ",
                      Prn[sug[[1]],3],
                      " - ",
                      Prn[sug[[2]],3]],
           FontColor->RGBColor[0,0,1]];

choose=True;
While[choose,
vrng=InputCheck[StringJoin["Choose an adequate field range for the
coercivity distribution.
\n\nType \"a\" to choose the entire range covered by the measurements
or \"s\" to choose the range of fields spanned by the distribution.
\nTo enter a different range, type the range limits. Example: 1,300.
Maximum range: ",
                           Prn[Chop[First[derlin][[1]]],3],
                           " - ",
                           Prn[Last[derlin][[1]],3]],
                   {"Expression","LongList"},
                   {{String,{"a","s"}},
                    {{2,2},Real,
                     {SetPrecision[Chop[First[derlin][[1]]],3],
                      SetPrecision[Last[derlin][[1]],3]}}}];

Which[vrng==="a",
      vrng={Chop[First[derlin][[1]]],Last[derlin][[1]]},
      vrng==="s",
      vrng=sug,
      True,
      vrng=Sort[vrng]];

nout=InputCheck[StringJoin["Enter the number of sampling points in the
chosen field range.
\n\nAt least ",
                           ToString[l1],
                           " points are recommanded."],
                {"Expression"},
                {{Integer,{1,Infinity}}}];

Print[StringJoin["Resample the coercivity distribution in the field range ",
                 Prn[vrng[[1]],3],
                 " - ",
                 Prn[vrng[[2]],3],
                 " ..."]];

vrng=Table[((k+q)^expn-(x0+q)^expn+qr)^rsexpn-(x0r+qr)^rsexpn,
           {k,vrng[[1]],vrng[[2]]+0.001*(vrng[[2]]-vrng[[1]]),
              (vrng[[2]]-vrng[[1]])/nout}];

dres=DCollocation[res03,res04,cd,cl,
                  vrng,nmatrix];

res=dres[[1]];
derr=dres[[3]];
dres=dres[[2]];
derr=Table[{dres[[k,1]],derr[[k]]},{k,1,Length[dres]}];
l3=Length[dres];

derln=
Table[{((res[[k,1]]+(x0r+qr)^rsexpn)^(1/rsexpn)-qr+(x0+q)^expn)^(1/expn)-q,
       
       (duns[(res[[k,1]]+(x0r+qr)^rsexpn)^(1/rsexpn)-qr]+
        
        dthf[res[[k,2]]+
             (res[[k,1]]+(x0r+qr)^rsexpn)^(1/rsexpn)-qr+
             (poly/.(x->((res[[k,1]]+(x0r+qr)^rsexpn)^(1/rsexpn)-qr)))]*
        (dres[[k,2]]*rsexpn*(res[[k,1]]+(x0r+qr)^rsexpn)^(1-1/rsexpn)+
         1+(dpoly/.(x->((res[[k,1]]+(x0r+qr)^rsexpn)^(1/rsexpn)-qr)))))*
       
       expn*((res[[k,1]]+(x0r+qr)^rsexpn)^(1/rsexpn)-qr+
             (x0+q)^expn)^(1-1/expn)
      },
      {k,1,l3}];

If[Not[maq],
   derln=Table[{derln[[k,1]],-derln[[k,2]]},{k,1,l3}]];
If[mono,
   derln=Table[{derln[[k,1]],Max[0,derln[[k,2]]]},{k,1,l3}]];

derlnerr=
Table[{((res[[k,1]]+(x0r+qr)^rsexpn)^(1/rsexpn)-qr+(x0+q)^expn)^(1/expn)-q,
       
       Abs[dthf[res[[k,2]]+
                (res[[k,1]]+(x0r+qr)^rsexpn)^(1/rsexpn)-qr+
                (poly/.(x->((res[[k,1]]+(x0r+qr)^rsexpn)^(1/rsexpn)-qr)))]*
           derr[[k,2]]*rsexpn*(res[[k,1]]+(x0r+qr)^rsexpn)^(1-1/rsexpn)*
           
           expn*((res[[k,1]]+(x0r+qr)^rsexpn)^(1/rsexpn)-qr+
                 (x0+q)^expn)^(1-1/expn)]
      },
      {k,1,l3}];


derlnl=Table[{derln[[k,1]],derln[[k,2]]-derlnerr[[k,2]]},
             {k,1,l3}];
derlnh=Table[{derln[[k,1]],derln[[k,2]]+derlnerr[[k,2]]},
             {k,1,l3}];

linrng={Table[derln[[k,1]],{k,1,Length[derln]}],
        Table[derlnh[[k,2]],{k,1,Length[derln]}]};
linrng={{Min[linrng[[1]]],Max[linrng[[1]]]},
        {Min[0,Min[linrng[[2]]]],1.01*Max[linrng[[2]]]}};
xticks=LinearTicks[linrng[[1]],"Original"][[1]];

sig=2*Max[Table[derlnerr[[k,2]],{k,1,l3}]]/linrng[[2,2]];
If[sig<0.14,
   linrng={linrng[[1]],
           {0,1.01*Max[Table[derlnh[[k,2]],{k,1,l3}]]}};
   yticks=LinearTicks[linrng[[2]],"Engeenering"];
   count=yticks[[2]];
   yticks=yticks[[1]];
   labels={StyleBox[RowBox[{"magnetic ","field"}],
                    FontFamily->"Courier",FontSize->16]//DisplayForm,
           StyleBox[RowBox[{"magnetization , ",
                            SuperscriptBox["10",
                                           ToString[Log[10,count]]],
                            " units"}],
                    FontFamily->"Courier",FontSize->16]//DisplayForm,
           None,None};
   Print["The thickness of the curve in the following plot corresponds
to the estimated error (2 sd):"];
   Show[Graphics[Polygon[Join[derlnl,Reverse[derlnh]]]],
        ListPlot[derln,
                 PlotJoined->True,
                 PlotStyle->{Thickness[0.001],GrayLevel[0]},
                 DisplayFunction->Identity],
        Axes->False,Frame->{True,True,False,False},
        PlotRange->linrng,
        ImageSize->800,
        FrameTicks->{xticks,yticks,None,None},
        FrameLabel->labels,
        RotateLabel->True,
        PlotLabel->StyleForm["COERCIVITY DISTRIBUTION ON A LINEAR SCALE:",
                             FontFamily->"Geneva",FontSize->20],
        DisplayFunction->$DisplayFunction],
   
   linrng={linrng[[1]],
           {0,1.01*Min[4*Max[Table[derln[[k,2]],{k,1,l3}]],
                       Max[Table[derlnh[[k,2]],{k,1,l3}]]]}};
   yticks=LinearTicks[linrng[[2]],"Engeenering"];
   count=yticks[[2]];
   yticks=yticks[[1]];
   labels={StyleBox[RowBox[{"magnetic ","field"}],
                    FontFamily->"Courier",FontSize->16]//DisplayForm,
           StyleBox[RowBox[{"magnetization , ",
                            SuperscriptBox["10",
                                           ToString[Log[10,count]]],
                            " units"}],
                    FontFamily->"Courier",FontSize->16]//DisplayForm,
           None,None};
   bool=Flatten[Table[{GrayLevel[j],
                       Polygon[Join[
                               Table[{derln[[k,1]],
                                      derln[[k,2]]-j*derlnerr[[k,2]]},
                                     {k,1,l3}],
                               Reverse[
                               Table[{derln[[k,1]],
                                      derln[[k,2]]+j*derlnerr[[k,2]]},
                                     {k,1,l3}]]]]},
                      {j,1,0,-0.02}],1];
   Print["The gray band around the curve in the following plot corresponds
to the estimated error (2 sd)"];
   Show[Graphics[bool],
        ListPlot[derln,
                 PlotJoined->True,
                 PlotStyle->{Thickness[0.006],GrayLevel[0]},
                 DisplayFunction->Identity],
        ListPlot[derln,
                 PlotJoined->True,
                 PlotStyle->{Thickness[0.001],GrayLevel[1]},
                 DisplayFunction->Identity],
        Axes->False,Frame->{True,True,False,False},
        PlotRange->linrng,
        ImageSize->800,
        FrameTicks->{xticks,yticks,None,None},
        FrameLabel->labels,
        RotateLabel->True,
        PlotLabel->StyleForm["COERCIVITY DISTRIBUTION ON A LINEAR SCALE:",
                             FontFamily->"Geneva",FontSize->20],
        DisplayFunction->$DisplayFunction]];

derlnlogerr=Table[{derlnerr[[k,1]],Log[10,derlnerr[[k,2]]]},
                  {k,1,Length[derlnerr]}];

errrng={Log[10,Min[Table[derlnerr[[k,2]],{k,1,l3}]]],
        Log[10,Max[Table[derlnerr[[k,2]],{k,1,l3}]]]};

errrng={errrng[[1]]-0.01*(errrng[[2]]-errrng[[1]]),
        errrng[[2]]+0.01*(errrng[[2]]-errrng[[1]])};
yticks=LogTicks[errrng];
errrng={linrng[[1]],errrng};
labels={StyleBox[RowBox[{"magnetic ","field"}],
                 FontFamily->"Courier",FontSize->16]//DisplayForm,
        StyleBox["error",
                 FontFamily->"Courier",FontSize->16]//DisplayForm,
        None,None};

ListPlot[derlnlogerr,
         PlotJoined->True,
         PlotStyle->Thickness[0.003],
         Axes->False,Frame->{True,True,False,False},
         PlotRange->errrng,
         FrameTicks->{xticks,yticks,None,None},
         FrameLabel->labels,
         RotateLabel->True,
         PlotLabel->StyleForm["ESTIMATED ERROR:",
                              FontFamily->"Geneva",FontSize->20],
         AspectRatio->0.3,
         ImageSize->922,PlotRegion->{{0,0.86},{0,1}}];

choose=InputCheck["Choose a different range of fields? (y/n)
\n\nIf the coercivity distrbution does not fill the entire plot range,
numerical instability probabliy occurred at fields near 0. Choose a
range which does not include H = 0.",
                   {"Expression"},
                   {{String,{"y","n"}}}];
If[choose==="y",
   choose=True,
   choose=False]

     ] (* Close While choose *)

   ]; (*Close If linear*)


(* Calculate the coercivity distribution on a power scale *)
power=True;expow=0;
While[power,
power=InputCheck["Calculate the coercivity distribution on a
power field scale H' = H^p ?  (y/n)",
                   {"Expression"},
                   {{String,{"y","n"}}}];
If[power==="y",
   power=True,
   power=False];

If[power,

expow=InputCheck[StringJoin["Enter the exponent p of the field scale
transformation
\nH' = H^p .
\n\nExponent for the most symmetric curve: ",
                            Prn[syexpn,3],
                            "\nExponent chosen by CODICA: ",
                            Prn[expn,3],
                            "\n\nThe exponent should be a positive number."],
                 {"Expression"},
                 {{"Real0",{0,Infinity}}}];

If[Chop[derlin[[1,1]]]==0,
   derpow=Table[{Chop[derlin[[k,1]]]^expow,
                 derlin[[k,2]]*Chop[derlin[[k,1]]]^(1-expow)/expow},
                {k,2,l2}],
   derpow=Table[{Chop[derlin[[k,1]]]^expow,
                 derlin[[k,2]]*Chop[derlin[[k,1]]]^(1-expow)/expow},
                {k,1,l2}]];

powrng={Table[derpow[[k,1]],{k,1,Length[derpow]}],
        Table[derpow[[k,2]],{k,1,Length[derpow]}]};
powrng={{Min[powrng[[1]]],Max[powrng[[1]]]},
        {Min[0,Min[powrng[[2]]]],1.01*Max[powrng[[2]]]}};

If[expow<0.6,
   If[powrng[[1,1]]==0,
      xticks=PowerTicks[{derpow[[2,1]]/5,powrng[[1,2]]},expow,False],
      xticks=PowerTicks[powrng[[1]],expow,False]],
   xticks=PowerTicks[powrng[[1]],expow,True][[1]]];

yticks=LinearTicks[powrng[[2]],"Engeenering"];
count=yticks[[2]];
yticks=yticks[[1]];
labels={StyleBox[RowBox[{"scaled ","magnetic ","field"}],
                 FontFamily->"Courier",FontSize->16]//DisplayForm,
        StyleBox[RowBox[{"magnetization , ",
                         SuperscriptBox["10",
                                        ToString[Log[10,count]]],
                         " units"}],
                 FontFamily->"Courier",FontSize->16]//DisplayForm,
        None,None};

count=1;
While[count<Length[derpow]&&
      derpow[[count,2]]<0.0025*powrng[[2,2]],
      count++];
If[count==2&&ref[[1,1]]==0,
   sug=1,
   sug=count];
count=Length[derpow];
While[count>1&&
      derpow[[count,2]]<0.0025*powrng[[2,2]],
      count--];
sug={sug,count};
sug={Chop[derpow[[sug[[1]],1]]],
     derpow[[sug[[2]],1]]};

Show[ListPlot[derpow,
              PlotJoined->True,
              PlotStyle->Thickness[0.005],
              DisplayFunction->Identity],
     Axes->False,Frame->{True,True,False,False},
     PlotRange->powrng,
     ImageSize->500,
     FrameTicks->{xticks,yticks,None,None},
     FrameLabel->labels,
     RotateLabel->True,
     PlotLabel->StyleForm[StringJoin["COERCIVITY DISTRIBUTION ON A 
H' = H^",Prn[expow,3],"  SCALE:"],
                          FontFamily->"Geneva",FontSize->16],
     DisplayFunction->$DisplayFunction];

StylePrint[StringJoin["Range of fields spanned by the
coercivity distribution: ",
                      Prn[sug[[1]]^(1/expow),3],
                      " - ",
                      Prn[sug[[2]]^(1/expow),3]],
           FontColor->RGBColor[0,0,1]];

choose=True;
While[choose,
vrng=InputCheck[StringJoin["Choose an adequate field range for the
coercivity distribution.
\n\nType \"a\" to choose the entire range shown in the previous plot
or \"s\" to choose the range of fields spanned by the distribution.
\nTo enter a different range, type the range limits. Example: 1,300.
Maximum range: ",
                           Prn[Chop[First[derpow][[1]]]^(1/expow),3],
                           " - ",
                           Prn[Last[derpow][[1]]^(1/expow),3]],
                   {"Expression","LongList"},
                   {{String,{"a","s"}},
                    {{2,2},Real,
                     {SetPrecision[Chop[First[derpow][[1]]]^(1/expow),3],
                      SetPrecision[Last[derpow][[1]]^(1/expow),3]}}}];

Which[vrng==="a",
      vrng={Chop[First[derpow][[1]]],Last[derpow][[1]]},
      vrng==="s",
      vrng=sug,
      True,
      vrng=Sort[vrng]^expow];

nout=InputCheck[StringJoin["Enter the number of sampling points in the
chosen field range.
\n\nAt least ",
                           ToString[l1],
                           " points are recommanded."],
                {"Expression"},
                {{Integer,{1,Infinity}}}];

Print[StringJoin["Resample the coercivity distribution in the field range ",
                 Prn[vrng[[1]]^(1/expow),3],
                 " - ",
                 Prn[vrng[[2]]^(1/expow),3],
                 " ..."]];

vrng=Table[((k^(1/expow)+q)^expn-(x0+q)^expn+qr)^rsexpn-(x0r+qr)^rsexpn,
           {k,vrng[[1]],vrng[[2]]+0.001*(vrng[[2]]-vrng[[1]]),
              (vrng[[2]]-vrng[[1]])/nout}];

dres=DCollocation[res03,res04,cd,cl,
                  vrng,nmatrix];

res=dres[[1]];
derr=dres[[3]];
dres=dres[[2]];
derr=Table[{dres[[k,1]],derr[[k]]},{k,1,Length[dres]}];
l3=Length[dres];

derpw=
Table[{(((res[[k,1]]+(x0r+qr)^rsexpn)^(1/rsexpn)-qr+
        (x0+q)^expn)^(1/expn)-q)^expow,
       
       (duns[(res[[k,1]]+(x0r+qr)^rsexpn)^(1/rsexpn)-qr]+
        
        dthf[res[[k,2]]+
             (res[[k,1]]+(x0r+qr)^rsexpn)^(1/rsexpn)-qr+
             (poly/.(x->((res[[k,1]]+(x0r+qr)^rsexpn)^(1/rsexpn)-qr)))]*
        (dres[[k,2]]*rsexpn*(res[[k,1]]+(x0r+qr)^rsexpn)^(1-1/rsexpn)+
         1+(dpoly/.(x->((res[[k,1]]+(x0r+qr)^rsexpn)^(1/rsexpn)-qr)))))*
       
       expn*((res[[k,1]]+(x0r+qr)^rsexpn)^(1/rsexpn)-qr+
             (x0+q)^expn)^(1-1/expn)*
        
        (((res[[k,1]]+(x0r+qr)^rsexpn)^(1/rsexpn)-qr+
         (x0+q)^expn)^(1/expn)-q)^(1-expow)/expow
      },
      {k,1,l3}];

If[Not[maq],
   derpw=Table[{derpw[[k,1]],-derpw[[k,2]]},{k,1,l3}]];
If[mono,
   derpw=Table[{derpw[[k,1]],Max[0,derpw[[k,2]]]},{k,1,l3}]];

derpwerr=
Table[{(((res[[k,1]]+(x0r+qr)^rsexpn)^(1/rsexpn)-qr+
        (x0+q)^expn)^(1/expn)-q)^expow,
       
       Abs[dthf[res[[k,2]]+
                (res[[k,1]]+(x0r+qr)^rsexpn)^(1/rsexpn)-qr+
                (poly/.(x->((res[[k,1]]+(x0r+qr)^rsexpn)^(1/rsexpn)-qr)))]*
           derr[[k,2]]*rsexpn*(res[[k,1]]+(x0r+qr)^rsexpn)^(1-1/rsexpn)*
           
           expn*((res[[k,1]]+(x0r+qr)^rsexpn)^(1/rsexpn)-qr+
                 (x0+q)^expn)^(1-1/expn)*
           
           (((res[[k,1]]+(x0r+qr)^rsexpn)^(1/rsexpn)-qr+
            (x0+q)^expn)^(1/expn)-q)^(1-expow)/expow]
      },
      {k,1,l3}];

derpwl=Table[{derpw[[k,1]],derpw[[k,2]]-derpwerr[[k,2]]},
             {k,1,l3}];
derpwh=Table[{derpw[[k,1]],derpw[[k,2]]+derpwerr[[k,2]]},
             {k,1,l3}];

powrng={Table[derpw[[k,1]],{k,1,Length[derpw]}],
        Table[derpwh[[k,2]],{k,1,Length[derpw]}]};
powrng={{Min[powrng[[1]]],Max[powrng[[1]]]},
        {Min[0,Min[powrng[[2]]]],1.01*Max[powrng[[2]]]}};

If[expow<0.6,
   If[powrng[[1,1]]==0,
      xticks=PowerTicks[{derpow[[2,1]]/5,powrng[[1,2]]},expow,False],
      xticks=PowerTicks[powrng[[1]],expow,False]],
   xticks=PowerTicks[powrng[[1]],expow,True][[1]]];

sig=2*Max[Table[derpwerr[[k,2]],{k,1,l3}]]/powrng[[2,2]];
If[sig<0.14,
   powrng={powrng[[1]],
           {0,1.01*Max[Table[derpwh[[k,2]],{k,1,l3}]]}};
   yticks=LinearTicks[powrng[[2]],"Engeenering"];
   count=yticks[[2]];
   yticks=yticks[[1]];
   labels={StyleBox[RowBox[{"magnetic ","field"}],
                    FontFamily->"Courier",FontSize->16]//DisplayForm,
           StyleBox[RowBox[{"magnetization , ",
                            SuperscriptBox["10",
                                           ToString[Log[10,count]]],
                            " units"}],
                    FontFamily->"Courier",FontSize->16]//DisplayForm,
           None,None};
   Print["The thickness of the curve in the following plot corresponds
to the estimated error (2 sd):"];
   Show[Graphics[Polygon[Join[derpwl,Reverse[derpwh]]]],
        ListPlot[derpw,
                 PlotJoined->True,
                 PlotStyle->{Thickness[0.001],GrayLevel[0]},
                 DisplayFunction->Identity],
        Axes->False,Frame->{True,True,False,False},
        PlotRange->powrng,
        ImageSize->800,
        FrameTicks->{xticks,yticks,None,None},
        FrameLabel->labels,
        RotateLabel->True,
        PlotLabel->StyleForm[StringJoin["COERCIVITY DISTRIBUTION ON A 
H' = H^",Prn[expow,3],"  SCALE:"],
                             FontFamily->"Geneva",FontSize->20],
        DisplayFunction->$DisplayFunction],
   
   powrng={powrng[[1]],
           {0,1.01*Min[4*Max[Table[derpw[[k,2]],{k,1,l3}]],
                       Max[Table[derpwh[[k,2]],{k,1,l3}]]]}};
   yticks=LinearTicks[powrng[[2]],"Engeenering"];
   count=yticks[[2]];
   yticks=yticks[[1]];
   labels={StyleBox[RowBox[{"magnetic ","field"}],
                    FontFamily->"Courier",FontSize->16]//DisplayForm,
           StyleBox[RowBox[{"magnetization , ",
                            SuperscriptBox["10",
                                           ToString[Log[10,count]]],
                            " units"}],
                    FontFamily->"Courier",FontSize->16]//DisplayForm,
           None,None};
   bool=Flatten[Table[{GrayLevel[j],
                       Polygon[Join[
                               Table[{derpw[[k,1]],
                                      derpw[[k,2]]-j*derpwerr[[k,2]]},
                                     {k,1,l3}],
                               Reverse[
                               Table[{derpw[[k,1]],
                                      derpw[[k,2]]+j*derpwerr[[k,2]]},
                                     {k,1,l3}]]]]},
                      {j,1,0,-0.02}],1];
   Print["The gray band around the curve in the following plot corresponds
to the estimated error (2 sd)"];
   Show[Graphics[bool],
        ListPlot[derpw,
                 PlotJoined->True,
                 PlotStyle->{Thickness[0.006],GrayLevel[0]},
                 DisplayFunction->Identity],
        ListPlot[derpw,
                 PlotJoined->True,
                 PlotStyle->{Thickness[0.001],GrayLevel[1]},
                 DisplayFunction->Identity],
        Axes->False,Frame->{True,True,False,False},
        PlotRange->powrng,
        ImageSize->800,
        FrameTicks->{xticks,yticks,None,None},
        FrameLabel->labels,
        RotateLabel->True,
        PlotLabel->StyleForm[StringJoin["COERCIVITY DISTRIBUTION ON A 
H' = H^",Prn[expow,3],"  SCALE:"],
                             FontFamily->"Geneva",FontSize->20],
        DisplayFunction->$DisplayFunction]];

derpwlogerr=Table[{derpwerr[[k,1]],Log[10,derpwerr[[k,2]]]},
                  {k,1,Length[derpwerr]}];

errrng={Log[10,Min[Table[derpwerr[[k,2]],{k,1,l3}]]],
        Log[10,Max[Table[derpwerr[[k,2]],{k,1,l3}]]]};

errrng={errrng[[1]]-0.01*(errrng[[2]]-errrng[[1]]),
        errrng[[2]]+0.01*(errrng[[2]]-errrng[[1]])};
yticks=LogTicks[errrng];
errrng={powrng[[1]],errrng};
labels={StyleBox[RowBox[{"magnetic ","field"}],
                 FontFamily->"Courier",FontSize->16]//DisplayForm,
        StyleBox["error",
                 FontFamily->"Courier",FontSize->16]//DisplayForm,
        None,None};

ListPlot[derpwlogerr,
         PlotJoined->True,
         PlotStyle->Thickness[0.003],
         Axes->False,Frame->{True,True,False,False},
         PlotRange->errrng,
         FrameTicks->{xticks,yticks,None,None},
         FrameLabel->labels,
         RotateLabel->True,
         PlotLabel->StyleForm["ESTIMATED ERROR:",
                              FontFamily->"Geneva",FontSize->20],
         AspectRatio->0.3,
         ImageSize->922,PlotRegion->{{0,0.86},{0,1}}];

choose=InputCheck["Choose a different range of fields? (y/n)
\n\nIf the coercivity distrbution does not fill the entire plot range,
numerical instability probabliy occurred at fields near 0. Choose a
range which does not include H = 0.",
                   {"Expression"},
                   {{String,{"y","n"}}}];
If[choose==="y",
   choose=True,
   choose=False]

     ] (* Close While choose *)

   ]; (* Close If power *)

     ]; (* Close While power *)

If[expow>0,
   power=True,
   power=False];


(* Save the results *)
out=InputCheck["Enter the common name of the files where you
want to save the results, or type \"exit\" to exit the program.
\n\nAppropriate extensions will be added to each saved file.",
               {"Expression"},
               {{String,{}}}];

If[Not[out==="exit"],
   
   Print[StringJoin["Save the fitted measurements in file: ",
                    out,".cum
\n 1. column: magnetic field,
\n 2. column: magnetization,
\n 3. culumn: absolute fit error."]];
   For[k=1,k<=Length[fit],k++,
       PutAppend[OutputForm[StringJoin[Prn[fit[[k,1]],6],"   ",
                                       Prn[fit[[k,2]],6],"   ",
                                       Prn[fiterr[[k,2]],3]]],
                 StringJoin[dir,out,".cum"]]];
   
   Print[StringJoin["Save the log-scaled coercivity distribution in file: ",
                    out,".slog
\n 1. column: log(magnetic field),
\n 2. column: coercivity distribution (unit: magnetization),
\n 3. culumn: relative error."]];
   For[k=1,k<=Length[derlg],k++,
       PutAppend[OutputForm[StringJoin[Prn[derlg[[k,1]],6],"   ",
                                       Prn[derlg[[k,2]],6],"   ",
                 Prn[derlgerr[[k,2]]/Max[10^-50,Abs[derlg[[k,2]]]],3]]],
                 StringJoin[dir,out,".slog"]]];
   
   If[linear,
      Print[StringJoin["Save the linear scaled coercivity distribution in
file: ",
                       out,".slin
\n 1. column: magnetic field,
\n 2. column: coercivity distribution (unit: magnetization/field),
\n 3. culumn: relative error."]];
      For[k=1,k<=Length[derln],k++,
          PutAppend[OutputForm[StringJoin[Prn[derln[[k,1]],6],"   ",
                                          Prn[derln[[k,2]],6],"   ",
                    Prn[derlnerr[[k,2]]/Max[10^-50,Abs[derln[[k,2]]]],3]]],
                    StringJoin[dir,out,".slin"]]]];
   
   If[power,
      Print[StringJoin["Save the power scaled coercivity distribution in
file: ",
                       out,".spow
\n 1. column: (magnetic field)^",
                       Prn[expow,3],
"\n 2. column: coercivity distribution (unit: magnetization/field^",
                       Prn[expow,3],
                       "),
\n 3. culumn: relative error."]];
      For[k=1,k<=Length[derpw],k++,
          PutAppend[OutputForm[StringJoin[Prn[derpw[[k,1]],6],"   ",
                                          Prn[derpw[[k,2]],6],"   ",
                    Prn[derpwerr[[k,2]]/Max[10^-50,Abs[derpw[[k,2]]]],3]]],
                    StringJoin[dir,out,".spow"]]]]
   ];


StylePrint["Thank you for using CODICA. Are you satisfied?
\nPlease report eventual problems or suggestions to the author!",
           FontColor->RGBColor[0,0.75,0],
           FontWeight->"Bold"];

];


End[]

EndPackage[]