(***************************************************************************)
(*                                                                         *)
(* --------------_ SOURCE CODE OF THE PROGRAM PACKAGE GECA --------------- *)
(*                                                                         *)
(*                          ALL RIGHTS RESERVED.                           *)
(*   THIS PACKAGE IS SUBJECTED TO COPYRIGHT. NO PART OF IT MAY BE USED,    *)
(*        COPIED OR REPRODUCED, IN ANY FORM, PHYSICAL OR DIGITAL,          *)
(*                           FOR COMMERCIAL USE.                           *)
(*                                                                         *)
(* ----------------------------------------------------------------------- *)
(*                                                                         *)
(* :$Id: Geca.m                                                            *)
(*       1st release: december 2002, Mathematica 3.0                       *)
(*       2nd release: february 2004, Mathematica 5.0                       *)
(*       3rd release: april 2005, Mathematica 5.0                          *)
(*                                                                         *)
(* :Geca  : v 1.0 01.11.2001                                               *)
(*          v 1.1 20.11.2002                                               *)
(*          v 2.0 24.02.2004                                               *)
(*          v 2.1 04.04.2005                                               *)
(*                                                                         *)
(* :Copyright: Copyright 2000-2005 by Ramon Egli. All rights reserved.     *)
(*             Actual adress: Dr. Ramon Egli                               *)
(*                            Institute for Rock Magnetism                 *)
(*                            291 Shepherd Laboratories                    *)
(*                            100 Union Street SE                          *)
(*                            Minneapolis, MN 55455-0128                   *)
(*                            U.S.A                                        *)
(*             E-mail: eglix007@umn.edu                                    *)
(*                                                                         *)
(* :Mathematica Version: 5.0 or later for Microsoft Windows                *)
(*                                                                         *)
(* :Title: Geca                                                            *)
(*                                                                         *)
(* :Programs: Geca   (GEneralized Coercivity Analyzer)                     *)
(*                                                                         *)
(* :Summary: This package performs a component analysis with special       *)
(*           distribution functions.                                       *)
(*                                                                         *)
(* Notice: To see the correct structure of the source code, wiew it always *)
(*         with the character format "Courier New". No tabulators are      *)
(*         used. Maximum width: 80 characters.                             *)
(*         Use 10 pt characters to print on A4 paper.                      *)
(*                                                                         *)
(***************************************************************************)


StylePrint[
"GECA v.2.1 for Mathematica 5.0 and later versions.
\nDistributed with the package MAG-MIX release 1, 04/04/2005.
\nCopyright 2005 by Ramon Egli. All rights reserved.",
           FontWeight->"Bold"];

BeginPackage["Utilities`Codica`"]


Geca::usage=
"GECA v. 2.0 is a program to perform a component analysis of coercivity
distributions by using special functions, called SGG. These functions can
reproduce almost every kind of individual natural and artificial components.
GECA also performs a Chi-square goodness of fit test to evaluate the
significance of the model chosen for to perform the component analysis.
Finally, GECA performs an error estimation and gives significance intervals
for each SGG functions.
\nGECA reads ASCII files with three columns, where the first column is the
(scaled) field, the second column is the value of the coercivity distribution
at the corresponding field and the third column gives the relative error of the
coercivity distribution. Output files of CODICA with extensions \".slin\",
\".slog\", \".spow\" can be readed directly by GECA.
\nGeca writes the results of component analysis on the log file
\"components.txt\". Numerical tables for individual SGG functions can be stored
as ASCII files as well.
\n\nRead the reference manual for more details."


Begin["Utilities`Codica`Private`"]


InputCheck[string_,type_,limitations_]:=
Module[{bool,count,delta,DigitalQ,elements,grammar,in,
        in0,l,out,path,PathQ,pos},

Off[Syntax::com];
Off[Read::readn];
DigitalQ[x_]:=MatchQ[ReadList[StringToStream[x],Number],
                     {_?(NumberQ[#]&)}];
PathQ[path_]:=
If[StringLength[path]>3,
   If[Not[Intersection[Join[CharacterRange["a","z"],
                            CharacterRange["A","Z"]],
                       {First[Characters[path]]}]==={}],
      If[StringTake[path,{2,3}]===":/",
         If[Not[Last[Characters[path]]==="/"||
                Last[Characters[path]]==="\\"],
            True,
            False],
         False],
      False],
   False];
grammar=type[[1]];
elements=limitations[[1]];
bool=False;count=0;
While[bool===False,
count++;
If[count==1,
   in0=InputString[string]];
If[count<=Length[type],
   grammar=type[[count]];
   elements=limitations[[count]],
   count=1;
   in0=InputString[StringJoin["NONVALID INPUT.\n",string]];
   grammar=type[[1]];
   elements=limitations[[1]]];
If[in0==="abort",Abort[]];
in=in0;
Which[grammar==="Expression",
      Which[elements[[1]]===String,
            If[elements[[2]]==={},
               bool=True;out=in,
               If[MemberQ[elements[[2]],in],
                  bool=True;
                  out=in]],
            elements[[1]]===Integer,
            If[DigitalQ[in],
               in=ToExpression[in];
               If[Head[in]===Integer,
                  If[elements[[2,1]]<=in<=elements[[2,2]],
                     bool=True;
                     out=in]]],
            elements[[1]]===Real,
            If[DigitalQ[in],
               in=N[ToExpression[in]];
               If[Head[in]===Real,
                  If[elements[[2,1]]<=in<=elements[[2,2]],
                     bool=True;
                     out=in]]],
            elements[[1]]==="Real0",
            If[DigitalQ[in],
               in=N[ToExpression[in]];
               If[Head[in]===Real&&Not[in==0],
                  If[elements[[2,1]]<=in<=elements[[2,2]],
                     bool=True;
                     out=in]]],
            True,
            bool=False],
      grammar==="Path",
      If[PathQ[in]===True,
         bool=True;
         out=in,
         bool=False],
      grammar==="ShortList",
      in=StringJoin["{",in,"}"];
      l=StringLength[in];
      pos=StringPosition[in,","];
      If[StringPosition[in,"{"]=={{1,1}}&&
         StringPosition[in,"}"]=={{l,l}}&&
         StringPosition[in," "]=={}&&
         Length[pos]==Length[elements]-1,
         pos=Join[{1},
                  Table[pos[[k,1]],{k,1,Length[pos]}],
                  {l}];
         delta=Min[Table[pos[[k]]-pos[[k-1]],{k,2,Length[pos]}]];
         If[delta>1,
            in=Table[StringTake[in,{pos[[k-1]]+1,pos[[k]]-1}],
                     {k,2,Length[pos]}];
            bool={};
            For[i=1,i<=Length[in],i++,
                Which[elements[[i,1]]===String,
                      bool=Append[bool,
                                  If[elements[[i,2]]==={},
                                     True,
                                     MemberQ[elements[[i,2]],in[[i]]]]],
                      elements[[i,1]]===Integer,
                      If[DigitalQ[in[[i]]],
                         If[Head[ToExpression[in[[i]]]]===Integer,
                            bool=Append[bool,
                                        elements[[i,2,1]]<=
                                        ToExpression[in[[i]]]<=
                                        elements[[i,2,2]]],
                            bool=Append[bool,False]],
                         bool=Append[bool,False]],
                      elements[[i,1]]===Real,
                      If[DigitalQ[in[[i]]],
                         If[Head[N[ToExpression[in[[i]]]]]===Real,
                            bool=Append[bool,
                                        elements[[i,2,1]]<=
                                        ToExpression[in[[i]]]<=
                                        elements[[i,2,2]]],
                            bool=Append[bool,False]],
                         bool=Append[bool,False]],
                      elements[[i,1]]==="Real0",
                      If[DigitalQ[in[[i]]],
                         If[Head[N[ToExpression[in[[i]]]]]===Real,
                            bool=Append[bool,
                                        (elements[[i,2,1]]<=
                                         ToExpression[in[[i]]]<=
                                         elements[[i,2,2]])&&
                                         Not[ToExpression[in[[i]]]==0]],
                            bool=Append[bool,False]],
                         bool=Append[bool,False]],
                      True,
                      bool=Append[bool,False]]];
            If[FreeQ[bool,False],
               bool=True;out={};
               For[i=1,i<=Length[in],i++,
                   If[elements[[i,1]]===String,
                      out=Append[out,in[[i]]],
                      out=Append[out,ToExpression[in[[i]]]]]],
               bool=False]]],
      grammar==="LongList",
      in=StringJoin["{",in,"}"];
      Which[SyntaxQ[in]&&
            ToExpression[in]==={}&&elements[[1,1]]==0,
            bool=True;
            out={},
            
            elements[[2]]===String,
            l=StringLength[in];
            pos=StringPosition[in,","];
            pos=Join[{1},
                     Table[pos[[k,1]],{k,1,Length[pos]}],
                     {StringLength[in]}];
            If[elements[[1,1]]-1<=Length[pos]<=elements[[1,2]]-1,
               in=Table[StringTake[in,{pos[[k-1]]+1,pos[[k]]-1}],
                        {k,2,Length[pos]}];
               bool=Table[If[elements[[3]]==={},
                             True,
                             MemberQ[elements[[3]],in[[k]]]],
                          {k,1,Length[in]}];
               bool=Not[MemberQ[bool,False]];
               If[bool,out=in],
               bool=False],
            
            True,
            l=StringLength[in];
            pos=StringPosition[in,","];
            pos=Join[{1},
                     Table[pos[[k,1]],{k,1,Length[pos]}],
                     {StringLength[in]}];
            in=Table[StringTake[in,{pos[[k-1]]+1,pos[[k]]-1}],
                     {k,2,Length[pos]}];
            If[elements[[1,1]]<=Length[in]<=elements[[1,2]]&&
               MatchQ[in,{__?(DigitalQ[#]&)}],
               in=Table[ToExpression[in[[k]]],{k,1,Length[in]}];
               Which[elements[[2]]===Integer,
                     bool=MatchQ[in,
                                 {__?(IntegerQ[#]&&
                                      elements[[3,1]]<=#<=elements[[3,2]]&)}],
                     
                     elements[[2]]==="Ordinal",
                     bool=MatchQ[in,
                                 {__?(IntegerQ[#]&&
                                      elements[[3,1]]<=#<=elements[[3,2]]&&
                                      Not[#==0]&)}],
                     
                     elements[[2]]===Real,
                     bool=MatchQ[in,
                                 {__?((Head[#]===Real||Head[#]===Integer)&&
                                      elements[[3,1]]<=#<=elements[[3,2]]&)}],
                     
                     elements[[2]]==="Real0",
                     bool=MatchQ[in,
                                 {__?((Head[#]===Real||Head[#]===Integer)&&
                                      elements[[3,1]]<=#<=elements[[3,2]]&&
                                      Not[#==0]&)}],
                     
                     elements[[2]]==="RealD",
                     bool=MatchQ[in,
                                 {__?((Head[#]===Real||Head[#]===Integer)&&
                                      elements[[3,1]]<=#<=elements[[3,2]]&&
                                      Not[#==0]&)}]&&
                          Length[Intersection[in]]==Length[in],
                     True,
                     bool=True];
               If[bool,out=in],
               
               bool=False]
      ],
      
      
      True,
      bool=False]];
out
      ];


Prn[x_,n_]:=
StringJoin[ToString[SetPrecision[NumberForm[N[x],NumberFormat->(#1&)],n]],
           If[ToString[NumberForm[N[x],NumberFormat->(#3&)]]==="",
              "",
              "e"],
           ToString[NumberForm[N[x],NumberFormat->(#3&)]]];


SampleList[list_,num_,order_,symm_]:=
Module[{l,lst,x1,x2,x3,y1,y2,int,rng,delta,intp},
Off[InterpolatingFunction::dmval];
l=Length[list];
x1=list[[1,1]];x2=list[[l,1]];x3=3*x2-x1;
y1=list[[1,2]];y2=list[[l,2]];y3=y1+y2;
Which[symm==={True,True},
intp=Interpolation[{{2*x2-x1,2*y2-y1},{3*x2-2*x1,2*y1-y2}},
                   InterpolationOrder->1];
lst=Join[Reverse[Table[{2*x1-list[[k,1]],2*y1-list[[k,2]]},
                       {k,2,l}]],
         list,
         Reverse[Table[{2*x2-list[[k,1]],2*y2-list[[k,2]]},
                       {k,1,l-1}]],
         Reverse[Table[{x3-list[[k,1]],intp[x3-list[[k,1]]]},
                       {k,1,l-1}]]],
      symm==={False,True},
intp=Interpolation[{{2*x2-x1,2*y2-y1},{3*x2-2*x1,y2}},
                   InterpolationOrder->1];
lst=Join[Reverse[Table[{2*x1-list[[k,1]],list[[k,2]]},
                       {k,2,l}]],
         list,
         Reverse[Table[{2*x2-list[[k,1]],2*y2-list[[k,2]]},
                       {k,1,l-1}]],
         Reverse[Table[{x3-list[[k,1]],intp[x3-list[[k,1]]]},
                       {k,1,l-1}]]],
      symm==={True,False},
intp=Interpolation[{{2*x2-x1,y1},{3*x2-2*x1,2*y1-y2}},
                   InterpolationOrder->1];
lst=Join[Reverse[Table[{2*x1-list[[k,1]],2*y1-list[[k,2]]},
                       {k,2,l}]],
         list,
         Reverse[Table[{2*x2-list[[k,1]],list[[k,2]]},
                       {k,1,l-1}]],
         Reverse[Table[{x3-list[[k,1]],intp[x3-list[[k,1]]]},
                       {k,1,l-1}]]],
      symm==={False,False},
intp=Interpolation[{{2*x2-x1,y1},{3*x2-2*x1,y2}},
                   InterpolationOrder->1];
lst=Join[Reverse[Table[{2*x1-list[[k,1]],list[[k,2]]},
                       {k,2,l}]],
         list,
         Reverse[Table[{2*x2-list[[k,1]],list[[k,2]]},
                       {k,1,l-1}]],
         Reverse[Table[{x3-list[[k,1]],intp[x3-list[[k,1]]]},
                       {k,1,l-1}]]]];
rng={lst[[1,1]],Last[lst][[1]]};
int=Interpolation[lst,InterpolationOrder->order];
delta=(rng[[2]]-rng[[1]])/(4*num-1);
Table[{rng[[1]]+k*delta,int[rng[[1]]+k*delta]},
      {k,0,4*num-1}]];


Margins[list_]:=
Module[{l,xrng,delta,rng,yrng,filt,diff,max,para,amp,off,x,slp,cen,
        sublist},
l=Length[list];
If[l>50,
   delta=Ceiling[l/50];
   sublist=Table[list[[k]],{k,1,l,delta}];
   max=Length[sublist];
   delta=(sublist[[max,1]]-sublist[[1,1]])/(max-1),
   sublist=list;
   delta=Table[list[[k,1]]-list[[k-1,1]],{k,2,l}];
   delta=Max[delta]];
filt=SampleList[sublist,128,1,{True,True}];
xrng={First[filt][[1]],Last[filt][[1]]};
filt=Butterworth[filt,delta,8];
filt=Take[filt,{129,257}];
delta=(xrng[[2]]-xrng[[1]])/127;
yrng=Table[filt[[k,2]],{k,1,128}];
yrng={Min[yrng],Max[yrng]};
rng=Table[list[[k,2]],{k,1,l}];
rng={Min[rng],Max[rng]};
diff=yrng[[1]]-yrng[[2]];
{(rng[[1]]-yrng[[1]])/diff,(yrng[[2]]-rng[[2]])/diff}];


Minim[function_,parameter_]:=
Module[{para,para1,para2,para3,res,res1,res2,res3,
        bool,rng,func,delta,count},
para1=parameter[[1]];para3=parameter[[2]];
para=(para1+para3)/2;
res=function[para];
bool=True;rng=True;count=0;
While[bool,
      count++;
      res1=function[para1];
      res2=res;para2=para;
      res3=function[para3];
      Print[StringJoin["iteration #",ToString[count],
                       ": exponent ",ToString[para],
                       ", residual ",ToString[FortranForm[res]]]];
      If[res2<res1*(para3-para2)/(para3-para1)+
              res3*(para2-para1)/(para3-para1),
         func[u_]=Fit[{{para1,res1},{para2,res2},{para3,res3}},
                      {1,u,u^2},u];
         para=FindMinimum[func[u],{u,para2}];
         para=u/.para[[2,1]];
         para=Max[0.001,para];
         delta=(para3-para1)/3;
         para1=Max[0.0005,para-delta];para3=para+delta;
         res=function[para];
         If[Abs[para2-para]/para<0.005,
            bool=False],
         bool=False;rng=False]];
If[rng,
   Print[StringJoin["Optimized exponent: ",ToString[para]]];
   para,
   Print["Optimization impossible within the given range."];
   $Failed]
      ];


SecantMinimize[function_,parameter_,precision_,freedom_]:=
Module[{para1,para11,para12,para13,
        para2,para21,para22,para23,
        res11,res13,res21,res23,res2,res,
        bool,count,lim},
lim=Max[0.0000001,0.0001/freedom];
para11=parameter[[1,1]];para13=parameter[[1,2]];
para1=(para11+para13)/2;
para21=parameter[[2,1]];para23=parameter[[2,2]];
para2=(para21+para23)/2;
res=0;
bool=True;count=0;
While[bool,
      count++;
      para12=para1;
      para22=para2;
      res2=res;
      res11=function[para11,para2];res13=function[para13,para2];
      res21=function[para1,para21];res23=function[para1,para23];
      res=function[para1,para2];
      Print[StringJoin["iter. #",ToString[count],
                       ": unsat. ",ToString[InputForm[{para1,para2}]],
                       ", res. ",ToString[Sqrt[res]]]];
      If[res11<res13,
         para13=para12,
         para11=para12];
      para1=(para11+para13)/2;
      If[res21<res23,
         para23=para22,
         para21=para22];
      para2=(para21+para23)/2;
      If[(Abs[para12-para1]/para1<10^(-precision)&&
          Abs[para22-para2]/para2<10^(-precision))||
          Abs[res2-res]/res<lim,
         bool=False]];
{para1,para2}];


MinimumCurvatureFit[list_]:=
Module[{l,x,x0,y,y0,a,b,lst},
l=Length[list];
x0=Table[N[list[[k,1]]],{k,1,l}];
y0=Table[N[list[[k,2]]],{k,1,l}];
x=Table[(x0[[k-1]]+x0[[k]])/2,{k,2,l}];
a={Join[{1/((x[[2]]-x[[1]])^2*(x0[[2]]-x[[1]])^2)+
         1/((x0[[2]]-x[[1]])^2*(x[[1]]-x0[[1]])^2),
         1/((x[[2]]-x[[1]])^2*(x[[2]]-x0[[2]])*(x0[[2]]-x[[1]]))},
        Table[0,{k,3,l-1}]]};
For[i=2,i<=l-2,i++,
    a=Append[a,
             Join[Table[0,{k,1,i-2}],
                  {1/((x[[i]]-x[[i-1]])^2*(x[[i]]-x0[[i]])*
                      (x0[[i]]-x[[i-1]])),
                   1/((x[[i]]-x[[i-1]])^2*(x[[i]]-x0[[i]])^2)+
                   1/((x[[i+1]]-x[[i]])^2*(x0[[i+1]]-x[[i]])^2)+
                   1/((x0[[i+1]]-x[[i]])^2*(x[[i]]-x0[[i]])^2),
                   1/((x[[i+1]]-x[[i]])^2*(x[[i+1]]-x0[[i+1]])*
                      (x0[[i+1]]-x[[i]]))},
                  Table[0,{k,i+2,l-1}]]]];
a=Append[a,
         Join[Table[0,{k,1,l-3}],
              {1/((x[[l-1]]-x[[l-2]])^2*(x[[l-1]]-x0[[l-1]])*
                  (x0[[l-1]]-x[[l-2]])),
               1/((x[[l-1]]-x[[l-2]])^2*(x[[l-1]]-x0[[l-1]])^2)+
               1/((x0[[l]]-x[[l-1]])^2*(x[[l-1]]-x0[[l-1]])^2)}]];
b=Join[{y0[[1]]/((x0[[2]]-x0[[1]])*(x0[[2]]-x[[1]])*
                 (x[[1]]-x0[[1]])^2)+
        y0[[2]]/(x0[[2]]-x[[1]])^2*
        (1/((x0[[2]]-x0[[1]])*(x[[1]]-x0[[1]]))+
         1/((x[[2]]-x[[1]])*(x[[2]]-x0[[2]])))},
       Table[y0[[k]]/(x[[k]]-x0[[k]])^2*
             (1/((x[[k]]-x[[k-1]])*(x0[[k]]-x[[k-1]]))+
              1/((x0[[k+1]]-x0[[k]])*(x0[[k+1]]-x[[k]])))+
             y0[[k+1]]/(x0[[k+1]]-x[[k]])^2*
             (1/((x0[[k+1]]-x0[[k]])*(x[[k]]-x0[[k]]))+
              1/((x[[k+1]]-x[[k]])*(x[[k+1]]-x0[[k+1]]))),
             {k,2,l-2}],
       {y0[[l-1]]/(x[[l-1]]-x0[[l-0]])^2*
        (1/((x[[l-1]]-x[[l-2]])*(x0[[l-1]]-x[[l-2]]))+
         1/((x0[[l]]-x0[[l-1]])*(x0[[l]]-x[[l-1]])))+
        y0[[l]]/((x0[[l]]-x0[[l-1]])*(x[[l-1]]-x0[[l-1]])*
                 (x0[[l]]-x[[l-1]])^2)}];
y=Dot[Inverse[a],b];
lst=Flatten[Table[{{x0[[k]],y0[[k]]},{x[[k]],y[[k]]}},
                  {k,1,l-1}],1];
lst=Append[lst,{x0[[l]],y0[[l]]}]
      ];

AddDigits[number_,digits_]:=
Module[{f,num},
num=ToString[number];
If[IntegerQ[number],
   num=StringJoin[num,"."]];
f=FractionalPart[number];
If[f<digits,
   num=StringJoin[num,
                  StringJoin[Table["0",{k,1,f-digits}]]]];
num
      ];


LinearTicks[range_,scale_]:=
Module[{delta,min,ndelta,order,rng,sgn,ticks,zero},

delta=0.2*(range[[2]]-range[[1]]);
order=10^Floor[Log[10,delta]];
ndelta=delta/order;
Which[ndelta<Sqrt[2],delta=order,
      ndelta<Sqrt[10],delta=2*order,
      ndelta<Sqrt[50],delta=5*order,
      True,order=10*order;delta=order];
Which[range[[1]]<=0<=range[[2]],sgn=0,
      range[[1]]>0,sgn=1,
      True,sgn=-1];
min=Min[Abs[range[[1]]],Abs[range[[2]]]];
min=delta*Floor[min/ndelta];
If[sgn==0,        
   zero=0,
   zero=sgn*min];
zero=zero*order;
rng={Floor[(range[[1]]-zero)/delta],
     Floor[(range[[2]]-zero)/delta]};
ticks=Table[zero+k*delta,{k,rng[[1]],rng[[2]]}];
min=Table[Abs[ticks[[k]]],{k,1,Length[ticks]}];
Which[scale==="MinimizedLabels",
      min=Max[delta,Min[min]];
      order=10^Floor[Log[10,min]],
      scale==="Engeenering",
      min=Max[min];
      order=10^(3*Floor[Log[10,min]/3]),
      scale==="Original",
      min=Max[min];
      order=1,
      True,
      min=Max[min];
      order=10^Floor[Log[10,min]]/10];
ticks=Table[{ticks[[k]],ticks[[k]]/order},
            {k,1,Length[ticks]}];
min=Table[StringLength[ToString[FractionalPart[N[ticks[[k,2]]]]]],
          {k,1,Length[ticks]}];
min=Max[min]-2;
If[min==0,
   ticks=Table[{ticks[[k,1]],ticks[[k,2]]},
               {k,1,Length[ticks]}],
   ticks=Table[{ticks[[k,1]],
                AddDigits[N[ticks[[k,2]]],min]},
               {k,1,Length[ticks]}]];
ticks=Table[{ticks[[k,1]],ticks[[k,2]],
             {0,0.016},{Thickness[0.002]}},
            {k,1,Length[ticks]}];
ticks=Join[ticks,
           Table[{zero+0.2*k*delta,"",
                  {0,0.01},{Thickness[0.002]}},
                 {k,Min[10*rng[[1]],0],
                    Max[10*rng[[2]],0]}]];
{ticks,order}
      ];


LogTicks[range_]:=
Module[{delta,ndelta,order,rng,ticks},

If[range[[2]]-range[[1]]<1,
   delta=(10^range[[2]]-10^range[[1]])*0.2;
   order=10^Floor[Log[10,delta]];
   ndelta=delta/order;
   Which[ndelta<Sqrt[2],delta=order,
         ndelta<Sqrt[10],delta=2*order,
         ndelta<Sqrt[50],delta=5*order,
         True,order=10*order;delta=order];
   rng={Ceiling[10^range[[1]]/delta],
        Floor[10^range[[2]]/delta]};
   ticks=Join[Table[{Log[10,k*delta],N[k*delta],
                     {0,0.016},{Thickness[0.002]}},
                    {k,rng[[1]],rng[[2]]}],
              Table[{Log[10,0.2*k*delta],"",
                     {0,0.01},{Thickness[0.002]}},
                    {k,rng[[1]],5*rng[[2]]+5}]],
   
   rng={Floor[range[[1]]],Ceiling[range[[2]]]};
   ticks={};
   For[i=rng[[1]],i<=rng[[2]],i++,
       ticks=Join[ticks,
                  Join[{{i,10^ToString[i],
                        {0,0.016},{Thickness[0.002]}}},
                       Table[{i+Log[10,k],"",{0,0.01},{Thickness[0.002]}},
                             {k,2,9}]]]]];
ticks];


PowerTicks[range_,expn_,lin_]:=
Module[{rng,ticks,ord},
If[lin,
   rng={range[[1]]^(1/expn),range[[2]]^(1/expn)};
   ticks=LinearTicks[rng,"Original"];
   ord=ticks[[2]];
   ticks=ticks[[1]];
   ticks=Table[{ticks[[k,1]]^expn,ticks[[k,2]],
                ticks[[k,3]],ticks[[k,4]]},
               {k,1,Length[ticks]}];
   ticks={ticks,ord},
   rng={Floor[Log[10,range[[1]]^(1/expn)]],
        Floor[Log[10,range[[2]]^(1/expn)]]};
   ticks={};
   For[i=rng[[1]],i<=rng[[2]],i++,
       ticks=Join[ticks,
                  Join[{{10^(i*expn),10^ToString[i],
                        {0,0.016},{Thickness[0.002]}}},
                       Table[{(k*10^i)^expn,"",{0,0.01},{Thickness[0.002]}},
                             {k,2,9}]]]]];
ticks];

ShortTicks[rng_,f_,sci_,sgn_]:=
Module[{d,ds,out,ex,dd,inf},
d=(rng[[2]]-rng[[1]])/f;
ds=d/10^Floor[Log[10,d]];
Which[ds<1.4,ds=1;dd=0.2,
      ds<3.2,ds=2;dd=0.5,
      ds<7.1,ds=5;dd=1,
      True,ds=10;dd=2];
dd=dd*10^Floor[Log[10,d]];
d=ds*10^Floor[Log[10,d]];
inf=d*Ceiling[rng[[1]]/d];
out=Table[N[k],{k,inf,rng[[2]],d}];
ex=MantissaExponent[rng[[2]]][[2]]-1;
If[sgn,
   out=Table[{out[[k]],
              ToString[SetPrecision[out[[k]]/10^ex,2]]},
             {k,1,Length[out]}];
   out=Table[{out[[k,1]],
              If[out[[k,1]]>=0,
                 Which[StringLength[out[[k,2]]]==1,
                       StringJoin[out[[k,2]],".0"],
                       StringLength[out[[k,2]]]>3,
                       StringTake[out[[k,2]],{1,3}],
                       True,
                       out[[k,2]]],
                 Which[StringLength[out[[k,2]]]==2,
                       StringJoin[out[[k,2]],".0"],
                       StringLength[out[[k,2]]]>4,
                       StringTake[out[[k,2]],{1,4}],
                       True,
                       out[[k,2]]]]},
             {k,1,Length[out]}],
   out=Table[{out[[k]],
              ToString[SetPrecision[Abs[out[[k]]]/10^ex,2]]},
             {k,1,Length[out]}];
   out=Table[{out[[k,1]],
              Which[StringLength[out[[k,2]]]==1,
                    StringJoin[out[[k,2]],".0"],
                    StringLength[out[[k,2]]]>3,
                    StringTake[out[[k,2]],{1,3}],
                    True,
                    out[[k,2]]]},
             {k,1,Length[out]}]];
If[sci,
   ex=StringJoin["E",
                 If[ex>=0,"+","-"],
                 ToString[Abs[ex]]],
   ex=""];
out=Table[{out[[k,1]],
           StringJoin[out[[k,2]],ex],
          {0,0.01}},
          {k,1,Length[out]}];
inf=dd*Ceiling[rng[[1]]/dd];
out=Join[out,
         Table[{N[k],"",{0,0.005}},{k,inf,rng[[2]],dd}]];
{out,ToString[MantissaExponent[rng[[2]]][[2]]-1]}];


Geca:=
Module[{logfile,bool,count,name,dir,list,l0,list1,list2,err,
        inip,comp,nc,tot,rng,mag,rini,txt,txtgr,r,g,b,lr,lg,lb,
        coords,compgr,compgre,fit,inipi,inipo,fitpi,search,compi,toti,
        funcs,pos1,pos2,fitp,ropt,list0,peak,
        listm,sterr,errmax,errcalc,nlist,nerr,merr,fitn,
        paralst,stat,means,vars,morep,morem,morev,
        compl,compm,comp1,comp2,rngn,d,n0,ne,
        compn,rerror,k0,swin,max,fixp,params,prparams,grads,rfit,
        nfile,comps,outdir,
        res,errrng,mqd,
        syss,gradv,fitv,amp,totm,parav,contrs,resvar,wt,redvar,
        loop,selpar,sel,cc,zeros,wts,
        a1,a2,a3,a4,a5,a6,a7,a8,a9,a10,a11,a12,a13,a14,a15,
        a16,a17,a18,a19,a20},

(* Set options *)
logfile=FileNames["components.txt",$TopDirectory,Infinity][[1]];
$TextStyle={FontSize -> 12};
Off[InterpolatingFunction::dmval];
Off[Graphics::gptn];
Off[ReadList::nffil];
Off[Read::readn];
Off[Inverse::luc];
Off[Inverse::invc];
Off[Power::infy];
Off[Syntax::com];
Off[NIntegrate::ncvb];
Off[NIntegrate::slwcon];
Off[NIntegrate::ploss];
Off[General::unfl];
Off[FindRoot::jsing];
Off[FindFit::lstol];
Unprotect[$MessageList];
nb=SelectedNotebook[];
zeroQ[x_]:=If[x==0,False,True];
PageWidth->Infinity;
TotalWidth->Infinity;

<<Statistics`ContinuousDistributions`;

r= {0  ,1  ,0  ,0.6,0  ,0.5};
g= {0  ,0  ,0.5,0  ,0.6,0.5};
b= {1  ,0  ,0  ,0.8,0.6,0  };
lr={    1  ,0.5,0.8,0.5,1  };
lg={    0.5,0.7,0.4,0.8,1  };
lb={    0.5,0.5,1  ,0.8,0.5};
coords={{0.02,1},{0.02,0.89},{0.02,0.73},{0.02,0.57},{0.02,0.41},{0.02,0.25}};

norm[x_,m_,d_]:=Exp[-(x-m)^2/(2*d^2)]/Sqrt[2*Pi*d];

ggd[x_,m_,s_,k_,p_]:=
Abs[k*Exp[k*(x-m)/s]+Exp[(x-m)/(k*s)]/k]*
Exp[-Abs[Log[(Exp[k*(x-m)/s]+Exp[(x-m)/(k*s)])/2]]^p/2]/
(2^(1+1/p)*s*Gamma[1+1/p]*(Exp[k*(x-m)/s]+Exp[(x-m)/(k*s)]));

ggdm[x_,m_,s_,k_,p_]:=
Abs[k*Exp[k*(x-m)/s]+Exp[(x-m)/(k*s)]/k]*
Exp[-Abs[Log[(Exp[k*(x-m)/s]+Exp[(x-m)/(k*s)])/2]]^p/2]/
(2^(1+1/p)*s^2*Gamma[1+1/p]*(Exp[k*(x-m)/s]+Exp[(x-m)/(k*s)]))*
(0.5*p*Sign[(x-m)/s]*Abs[Log[(Exp[k*(x-m)/s]+
Exp[(x-m)/(k*s)])/2]]^(p-1)*
Abs[k*Exp[k*(x-m)/s]+Exp[(x-m)/(k*s)]/k]/(Exp[k*(x-m)/s]+Exp[(x-m)/(k*s)])+
(k*Exp[k*(x-m)/s]+Exp[(x-m)/(k*s)]/k)/(Exp[k*(x-m)/s]+Exp[(x-m)/(k*s)])-
(k^2*Exp[k*(x-m)/s]+Exp[(x-m)/(k*s)]/k^2)/(k*Exp[k*(x-m)/s]+
Exp[(x-m)/(k*s)]/k));

ggds[x_,m_,s_,k_,p_]:=
Abs[k*Exp[k*(x-m)/s]+Exp[(x-m)/(k*s)]/k]*
Exp[-Abs[Log[(Exp[k*(x-m)/s]+Exp[(x-m)/(k*s)])/2]]^p/2]/
(2^(1+1/p)*s^2*Gamma[1+1/p]*(Exp[k*(x-m)/s]+Exp[(x-m)/(k*s)]))*
(0.5*p*Abs[(x-m)/s]*
Abs[Log[(Exp[k*(x-m)/s]+Exp[(x-m)/(k*s)])/2]]^(p-1)*
Abs[k*Exp[k*(x-m)/s]+Exp[(x-m)/(k*s)]/k]/(Exp[k*(x-m)/s]+
Exp[(x-m)/(k*s)])+Abs[(x-m)/s]*
(k*Exp[k*(x-m)/s]+Exp[(x-m)/(k*s)]/k)/(Exp[k*(x-m)/s]+
Exp[(x-m)/(k*s)])-Abs[(x-m)/s]*(k^2*Exp[k*(x-m)/s]+
Exp[(x-m)/(k*s)]/k^2)/(k*Exp[k*(x - m)/s]+Exp[(x-m)/(k*s)]/k)-1);

ggdq[x_,m_,s_,k_,p_]:=
Abs[k*Exp[k*(x-m)/s]+Exp[(x-m)/(k*s)]/k]*
Exp[-Abs[Log[(Exp[k*(x-m)/s]+Exp[(x-m)/(k*s)])/2]]^p/2]/
(2^(1+1/p)*s*Gamma[1+1/p]*(Exp[k*(x-m)/s]+Exp[(x-m)/(k*s)]))*
(-0.5*p*Sign[k]*Abs[(x-m)/s]*
Abs[Log[(Exp[k*(x-m)/s]+Exp[(x-m)/(k*s)])/2]]^(p-1)*
(Exp[k*(x-m)/s]-Exp[(x-m)/(k*s)]/k^2)/(Exp[k*(x-m)/s]+
Exp[(x-m)/(k*s)])+1/k*
(k*Exp[k*(x-m)/s]-Exp[(x-m)/(k*s)]/k)/(k*Exp[k*(x-m)/s]+
Exp[(x-m)/(k*s)]/k)+(x-m)/s*
(Exp[k*(x-m)/s]-Exp[(x-m)/(k*s)]/k^4)/(Exp[k*(x-m)/s]+
Exp[(x-m)/(k*s)]/k^2)-(x-m)/s*
(Exp[k*(x-m)/s]-Exp[(x-m)/(k*s)]/k^2)/(Exp[k*(x-m)/s]+
Exp[(x-m)/(k*s)]));

ggdp[x_,m_,s_,k_,p_]:=
If[(Exp[k*(x-m)/s]+Exp[(x-m)/(k*s)])/2==1,
   Abs[k*Exp[k*(x-m)/s]+Exp[(x-m)/(k*s)]/k]*
   Exp[-Abs[Log[(Exp[k*(x-m)/s]+Exp[(x-m)/(k*s)])/2]]^p/2]/
   (2^(1+1/p)*s*
   Gamma[1+1/p]*(Exp[k*(x-m)/s]+Exp[(x-m)/(k*s)]))*
   (PolyGamma[0,1+1/p]/p^2),
   Abs[k*Exp[k*(x-m)/s]+Exp[(x-m)/(k*s)]/k]*
   Exp[-Abs[Log[(Exp[k*(x-m)/s]+Exp[(x-m)/(k*s)])/2]]^p/2]/
   (2^(1+1/p)*s*
   Gamma[1+1/p]*(Exp[k*(x-m)/s]+Exp[(x-m)/(k*s)]))*
   (PolyGamma[0,1+1/p]/p^2-
   0.5*Log[Abs[Log[(Exp[k*(x-m)/s]+Exp[(x-m)/(k*s)])/2]]]*
   Abs[Log[(Exp[k*(x-m)/s]+Exp[(x-m)/(k*s)])/2]]^p)];

(* read the data file *)
bool=True;count=0;
While[bool,
Which[count==0,
      name=InputCheck["Enter path and name of the data file.
\n(intermediate directories are not necessary)\n
\nExample. To read C:/users/me/data/myfile.dat you can enter:
\nC:/users/myfile.dat",
                      {"Path"},{{String,{}}}];
      count++,
      dir==={},
      name=InputCheck[StringJoin["File ",name," not found.\n
\nReenter path and name of the data file:
\n
\nExample. To read C:/users/me/data/myfile.dat you can enter:
\nC:/users/myfile.dat"],
                      {"Path"},{{String,{}}}];
      count++,
      Length[dir]>1,
      name=InputCheck[StringJoin["More files ",name," were found.\n
\nSpecify better the path of the file.\n
\nExample. Instead of C:/users/myfile.dat enter:
\nC:/users/me/myfile.dat"],
                      {"Path"},{{String,{}}}];
      count++];
dir=FileNames[StringTake[name,
                         {Last[StringPosition[name,"/"]][[1]]+1,
                          StringLength[name]}],
              StringTake[name,
                         {1,
                          Last[StringPosition[name,"/"]][[1]]-1}],
              Infinity];
If[Length[dir]==1,
   dir=dir[[1]];
   list=ReadList[dir,Number,RecordLists->True];
   If[list==={{$Failed}},
      Print["Nonnumerical data encountred during reading."];
      Abort[],
      dim=Dimensions[list];
      If[Length[dim]==2&&dim[[2]]==3,
         If[dim[[1]]>3,
            bool=False;
            Print[StringJoin["Data from file\n",name]],
            Print[StringJoin["File ",name,"
does not contain enough measuring points."]];
            Abort[]],
         Print[StringJoin["File ",name,"
contains data in a wrong format."]];
         Abort[]]]]
     ];

pr={Last[StringPosition[name,"/"]][[1]],StringLength[name]};
outdir=StringTake[name,{1,pr[[1]]-1}];
name=StringTake[name,{pr[[1]]+1,pr[[2]]}];
outdir=StringReplace[outdir,"/"->"\\"];
pr=StringPosition[name,"."];
If[pr==={},
   nfile=name,
   nfile=StringTake[name,{1,pr[[1,1]]-1}]];

(* Calculate distribution properties *)
Print["Checking the coercivity distribution..."];
l0=Length[list];
dir=StringTake[dir,
               {1,Last[StringPosition[dir,"\\"]][[1]]}];
list1=Table[{list[[k,1]],list[[k,2]]*(1-list[[k,3]])},{k,1,l0}];
list2=Table[{list[[k,1]],list[[k,2]]*(1+list[[k,3]])},{k,1,l0}];
err=Table[{list[[k,1]],list[[k,2]]*list[[k,3]]},{k,1,l0}];
list=Table[{list[[k,1]],list[[k,2]]},{k,1,l0}];
peak=Max[Table[list[[k,2]],{k,1,l0}]];
errmax=Max[Table[err[[k,2]],{k,1,l0}]]/peak;
list0=list;
sterr=Table[{err[[k,1]],err[[k,2]]*0.01/errmax},{k,1,l0}];

rng={{First[list][[1]],Last[list][[1]]},
     {0,Max[Table[list2[[k,2]],{k,1,l0}]]}};
mag=Interpolation[list];
mag=NIntegrate[mag[x],{x,rng[[1,1]],rng[[1,2]]}];
txt=StringJoin["Field in 10E",
               ShortTicks[rng[[1]],5,False,True][[2]],
               " units.\nTotal magnetization: ",
               ToString[FortranForm[SetPrecision[mag,2]]]];

(* Plot the coercivity distribution *)
If[Max[Table[err[[k,2]],{k,1,l0}]]/peak>0.05,
   listm=list0;
   Print["Coercivity distribution with confidence limits:"],
   listm={Null};
   Print["Confidence limits of the coercivity distribution:"]];

txtgr=Graphics[{RGBColor[0,0,1],Text[txt,Scaled[{0.02,1}],{-1,1}]}];
     
compgr={ListPlot[list1,PlotJoined->True,
                 PlotStyle->{GrayLevel[0.5],Thickness[0.005]},
                 DisplayFunction->Identity],
        ListPlot[list2,PlotJoined->True,
                 PlotStyle->{GrayLevel[0.5],Thickness[0.005]},
                 DisplayFunction->Identity],
        ListPlot[listm,PlotJoined->True,DisplayFunction->Identity,
                 PlotStyle->{Thickness[0.002]}]};

Show[GraphicsArray[
     {{Show[compgr,
            PlotRange->rng,
            AspectRatio->0.6,
            Axes->True,AxesOrigin->{rng[[1,1]],0},
            Ticks->{ShortTicks[rng[[1]],5,False,True][[1]],
                    ShortTicks[rng[[2]],5,True,False][[1]]},
            DisplayFunction->Identity],
       Show[txtgr,
            AspectRatio->0.6,
            Axes->False,
            DisplayFunction->Identity]}}],
     ImageSize->1200,GraphicsSpacing->0,
     PlotRegion->{{-0.025,1.025},{-0.025,1.025}},
     DisplayFunction->$DisplayFunction];


(* Select the fitting window *)
d=0.75*(rng[[1,2]]-rng[[1,1]])/Length[list0];
int=Interpolation[list0];
list0=Table[{rng[[1,1]]+k*d,int[rng[[1,1]]+k*d]},
            {k,1,Floor[Length[list0]/0.75]}];
l0=Length[list0];
int=Interpolation[err];
err=Table[{rng[[1,1]]+k*d,int[rng[[1,1]]+k*d]},
          {k,1,Floor[Length[list0]/0.75]}];
int=Interpolation[sterr];
sterr=Table[{rng[[1,1]]+k*d,int[rng[[1,1]]+k*d]},
            {k,1,Floor[Length[list0]/0.75]}];

list={};
merr={};
wt={};
wts={};
For[i=1,i<=l0,i++,
    If[err[[i,2]]/list0[[i,2]]<0.5,
       list=Append[list,list0[[i]]];
       merr=Append[merr,sterr[[i]]];
       wt=Append[wt,err[[i]]];
       wts=Append[wt,list0[[i,2]]/err[[i,2]]]]];
Print[StringJoin["Coercivity distribution is significant between ",
                 ToString[FortranForm[SetPrecision[First[list][[1]],4]]],
                 " and ",
                 ToString[FortranForm[SetPrecision[Last[list][[1]],4]]]]];
swin=InputCheck["Enter the range to consider for fitting,
\nmin,max
\nor type \"a\" to select the entire significan part of the
coercivity distribution.
\nExample: -1,3",
                {"Expression","LongList"},
               {{String,{"a"}},
                {{2,2},Real,{-Infinity,Infinity}}}];
If[swin==="a",
   swin={list[[1,1]],Last[list][[1]]},
   swin=Sort[swin];
   While[list[[1,1]]<swin[[1]],
         list=Drop[list,1];
         merr=Drop[merr,1];
         wt=Drop[wt,1];
         wts=Drop[wts,1]];
   While[Last[list][[1]]>swin[[2]],
         list=Drop[list,-1];
         merr=Drop[merr,-1];
         wt=Drop[wt,-1];
         wts=Drop[wts,-1]]];
Print[StringJoin["Fitting is performed in the range between ",
                 ToString[FortranForm[SetPrecision[First[list][[1]],4]]],
                 " and ",
                 ToString[FortranForm[SetPrecision[Last[list][[1]],4]]]]];
l0=Length[list];
k0=Length[list0];
wt=Sqrt[Sum[wt[[k,2]]^2,{k,1,Length[wt]}]/Length[wt]];

loop=True;count=0;search=False;
While[loop,

rfit=True;
While[rfit,

(* Enter initial values *)
rini=True;
While[rini,
Which[search,
      inipi=InputCheck["Choose the distribution parameters corresponding to
a particular point in the optimization plot by entering the number
of the desired point.",
                {"Expression"},
                {{Integer,{-Length[contrs],Length[contrs]}}}];
      inipi=contrs[[inipi]],
      count==0,
      inipi=InputCheck["Enter the initial distribution parameters:\n
\na1,m1,s1,q1,p1,
\na2,m2,s2,q2,p2,
\n . . .",
                    {"ShortList","ShortList","ShortList","ShortList",
                     "ShortList"},
                    {{{Real,{0,Infinity}},
                      {Real,{-Infinity,Infinity}},
                      {"Real0",{0,Infinity}},
                      {Real,{-Infinity,Infinity}},
                      {"Real0",{0,Infinity}}},
                     {{Real,{0,Infinity}},
                      {Real,{-Infinity,Infinity}},
                      {"Real0",{0,Infinity}},
                      {Real,{-Infinity,Infinity}},
                      {"Real0",{0,Infinity}},
                      {Real,{0,Infinity}},
                      {Real,{-Infinity,Infinity}},
                      {"Real0",{0,Infinity}},
                      {Real,{-Infinity,Infinity}},
                      {"Real0",{0,Infinity}}},
                     {{Real,{0,Infinity}},
                      {Real,{-Infinity,Infinity}},
                      {"Real0",{0,Infinity}},
                      {Real,{-Infinity,Infinity}},
                      {"Real0",{0,Infinity}},
                      {Real,{0,Infinity}},
                      {Real,{-Infinity,Infinity}},
                      {"Real0",{0,Infinity}},
                      {Real,{-Infinity,Infinity}},
                      {"Real0",{0,Infinity}},
                      {Real,{0,Infinity}},
                      {Real,{-Infinity,Infinity}},
                      {"Real0",{0,Infinity}},
                      {Real,{-Infinity,Infinity}},
                      {"Real0",{0,Infinity}}},
                     {{Real,{0,Infinity}},
                      {Real,{-Infinity,Infinity}},
                      {"Real0",{0,Infinity}},
                      {Real,{-Infinity,Infinity}},
                      {"Real0",{0,Infinity}},
                      {Real,{0,Infinity}},
                      {Real,{-Infinity,Infinity}},
                      {"Real0",{0,Infinity}},
                      {Real,{-Infinity,Infinity}},
                      {"Real0",{0,Infinity}},
                      {Real,{0,Infinity}},
                      {Real,{-Infinity,Infinity}},
                      {"Real0",{0,Infinity}},
                      {Real,{-Infinity,Infinity}},
                      {"Real0",{0,Infinity}},
                      {Real,{0,Infinity}},
                      {Real,{-Infinity,Infinity}},
                      {"Real0",{0,Infinity}},
                      {Real,{-Infinity,Infinity}},
                      {"Real0",{0,Infinity}}},
                     {{Real,{0,Infinity}},
                      {Real,{-Infinity,Infinity}},
                      {"Real0",{0,Infinity}},
                      {Real,{-Infinity,Infinity}},
                      {"Real0",{0,Infinity}},
                      {Real,{0,Infinity}},
                      {Real,{-Infinity,Infinity}},
                      {"Real0",{0,Infinity}},
                      {Real,{-Infinity,Infinity}},
                      {"Real0",{0,Infinity}},
                      {Real,{0,Infinity}},
                      {Real,{-Infinity,Infinity}},
                      {"Real0",{0,Infinity}},
                      {Real,{-Infinity,Infinity}},
                      {"Real0",{0,Infinity}},
                      {Real,{0,Infinity}},
                      {Real,{-Infinity,Infinity}},
                      {"Real0",{0,Infinity}},
                      {Real,{-Infinity,Infinity}},
                      {"Real0",{0,Infinity}},
                      {Real,{0,Infinity}},
                      {Real,{-Infinity,Infinity}},
                      {"Real0",{0,Infinity}},
                      {Real,{-Infinity,Infinity}},
                      {"Real0",{0,Infinity}}}}],
      count>0,
      inipi=InputCheck["Enter the initial distribution parameters:\n
\na1,m1,s1,q1,p1,
\na2,m2,s2,q2,p2,
\n. . .
\n\nType \"i\" to re-enter previous initial parameters,
\nType \"r\" to enter the parameters of the last model.",
                    {"ShortList","ShortList","ShortList","ShortList",
                     "ShortList",
                     "Expression"},
                    {{{Real,{0,Infinity}},
                      {Real,{-Infinity,Infinity}},
                      {"Real0",{0,Infinity}},
                      {Real,{-Infinity,Infinity}},
                      {"Real0",{0,Infinity}}},
                     {{Real,{0,Infinity}},
                      {Real,{-Infinity,Infinity}},
                      {"Real0",{0,Infinity}},
                      {Real,{-Infinity,Infinity}},
                      {"Real0",{0,Infinity}},
                      {Real,{0,Infinity}},
                      {Real,{-Infinity,Infinity}},
                      {"Real0",{0,Infinity}},
                      {Real,{-Infinity,Infinity}},
                      {"Real0",{0,Infinity}}},
                     {{Real,{0,Infinity}},
                      {Real,{-Infinity,Infinity}},
                      {"Real0",{0,Infinity}},
                      {Real,{-Infinity,Infinity}},
                      {"Real0",{0,Infinity}},
                      {Real,{0,Infinity}},
                      {Real,{-Infinity,Infinity}},
                      {"Real0",{0,Infinity}},
                      {Real,{-Infinity,Infinity}},
                      {"Real0",{0,Infinity}},
                      {Real,{0,Infinity}},
                      {Real,{-Infinity,Infinity}},
                      {"Real0",{0,Infinity}},
                      {Real,{-Infinity,Infinity}},
                      {"Real0",{0,Infinity}}},
                     {{Real,{0,Infinity}},
                      {Real,{-Infinity,Infinity}},
                      {"Real0",{0,Infinity}},
                      {Real,{-Infinity,Infinity}},
                      {"Real0",{0,Infinity}},
                      {Real,{0,Infinity}},
                      {Real,{-Infinity,Infinity}},
                      {"Real0",{0,Infinity}},
                      {Real,{-Infinity,Infinity}},
                      {"Real0",{0,Infinity}},
                      {Real,{0,Infinity}},
                      {Real,{-Infinity,Infinity}},
                      {"Real0",{0,Infinity}},
                      {Real,{-Infinity,Infinity}},
                      {"Real0",{0,Infinity}},
                      {Real,{0,Infinity}},
                      {Real,{-Infinity,Infinity}},
                      {"Real0",{0,Infinity}},
                      {Real,{-Infinity,Infinity}},
                      {"Real0",{0,Infinity}}},
                     {{Real,{0,Infinity}},
                      {Real,{-Infinity,Infinity}},
                      {"Real0",{0,Infinity}},
                      {Real,{-Infinity,Infinity}},
                      {"Real0",{0,Infinity}},
                      {Real,{0,Infinity}},
                      {Real,{-Infinity,Infinity}},
                      {"Real0",{0,Infinity}},
                      {Real,{-Infinity,Infinity}},
                      {"Real0",{0,Infinity}},
                      {Real,{0,Infinity}},
                      {Real,{-Infinity,Infinity}},
                      {"Real0",{0,Infinity}},
                      {Real,{-Infinity,Infinity}},
                      {"Real0",{0,Infinity}},
                      {Real,{0,Infinity}},
                      {Real,{-Infinity,Infinity}},
                      {"Real0",{0,Infinity}},
                      {Real,{-Infinity,Infinity}},
                      {"Real0",{0,Infinity}},
                      {Real,{0,Infinity}},
                      {Real,{-Infinity,Infinity}},
                      {"Real0",{0,Infinity}},
                      {Real,{-Infinity,Infinity}},
                      {"Real0",{0,Infinity}}},
                      {String,{"i","r"}}}];
      Which[inipi==="i",inipi=inipo,
            inipi==="r",inipi=fitpi,
            True,Null]];
inipo=inipi;
nc=Length[inipi]/5;
comp={};
For[i=1,i<=5*nc,i=i+5,
    comp=Append[comp,
    Table[{list0[[k,1]],
           inipi[[i]]*ggd[list0[[k,1]],
                         inipi[[i+1]],inipi[[i+2]],inipi[[i+3]],inipi[[i+4]]]},
          {k,1,k0}]]];
tot=Table[{list0[[k,1]],Sum[comp[[i,k,2]],{i,1,nc}]},{k,1,k0}];
txt={StringJoin["Field in 10E",
                ShortTicks[rng[[1]],10,False,True][[2]],
                " units.\nInitial distribution parameters:"]};
For[i=1,i<=nc,i++,
    txt=Append[txt,StringJoin[
"a",ToString[i]," = ",ToString[FortranForm[SetPrecision[inipi[[5*i-4]],4]]],
"\nm",ToString[i]," = ",ToString[FortranForm[SetPrecision[inipi[[5*i-3]],4]]],
"\ts",ToString[i]," = ",ToString[FortranForm[SetPrecision[inipi[[5*i-2]],4]]],
"\nq",ToString[i]," = ",ToString[FortranForm[SetPrecision[inipi[[5*i-1]],4]]],
"\tp",ToString[i]," = ",ToString[FortranForm[SetPrecision[inipi[[5*i]],4]]]]
              ]];
txtgr=Table[Graphics[{RGBColor[r[[k]],g[[k]],b[[k]]],
                      Text[txt[[k]],Scaled[coords[[k]]],{-1,1}]}],
            {k,1,nc+1}];
compgr=Join[{ListPlot[list1,PlotJoined->True,
                      PlotStyle->{GrayLevel[0.5],Thickness[0.005]},
                      DisplayFunction->Identity],
             ListPlot[list2,PlotJoined->True,
                      PlotStyle->{GrayLevel[0.5],Thickness[0.005]},
                      DisplayFunction->Identity],
             ListPlot[listm,PlotJoined->True,
                      PlotStyle->{Thickness[0.002]},
                      DisplayFunction->Identity],
             ListPlot[tot,PlotJoined->True,
                      PlotStyle->{RGBColor[0,0,1],Thickness[0.002]},
                      DisplayFunction->Identity]},
            Table[ListPlot[comp[[k]],PlotJoined->True,
                           PlotStyle->{RGBColor[r[[k+1]],g[[k+1]],b[[k+1]]],
                                       Thickness[0.002]},
                           DisplayFunction->Identity],
                  {k,1,nc}]];
Show[GraphicsArray[
     {{Show[compgr,
            PlotRange->rng,
            AspectRatio->0.6,
            Axes->True,AxesOrigin->{rng[[1,1]],0},
            Ticks->{ShortTicks[rng[[1]],5,False,True][[1]],
                    ShortTicks[rng[[2]],5,True,False][[1]]},
            DisplayFunction->Identity],
       Show[txtgr,
            AspectRatio->0.6,
            Axes->False,
            DisplayFunction->Identity]}}],
     ImageSize->1200,GraphicsSpacing->0,
     PlotRegion->{{-0.025,1.025},{-0.025,1.025}},
     DisplayFunction->$DisplayFunction];

rini=InputCheck["Accept the initial parameters? (y/n)",
                {"Expression"},
                {{String,{"y","n"}}}];
If[rini==="y",rini=False,rini=True]
     ]; (* Close While rini *)
     
Clear["a1"];Clear["a2"];Clear["a3"];Clear["a4"];Clear["a5"];
Clear["a6"];Clear["a7"];Clear["a8"];Clear["a9"];Clear["a10"];
Clear["a11"];Clear["a12"];Clear["a13"];Clear["a14"];Clear["a15"];
Clear["a16"];Clear["a17"];Clear["a18"];Clear["a19"];Clear["a20"];
Clear["a21"];Clear["a22"];Clear["a23"];Clear["a24"];Clear["a25"];
search=False;

(* Choose fixed parameters *)
fixp=InputCheck["Enter the list parameters you want to keep fixed
during optimization.\n
\nExamples: q2,p2 or leave blank if you want to optimize all
parameters.
\nLast entered values will be taken for the fixed parameters.",
                 {"LongList","Expression"},
                 {{Which[nc==1,{1,9},
                         nc==2,{1,19},
                         nc==3,{1,29},
                         nc==4,{1,39},
                         True,{1,49}],
                   String,
                   Which[nc==1,
                         {"a1","m1","s1","q1","p1"},
                         nc==2,
                         {"a1","m1","s1","q1","p1",
                          "a2","m2","s2","q2","p2"},
                         nc==3,
                         {"a1","m1","s1","q1","p1",
                          "a2","m2","s2","q2","p2",
                          "a3","m3","s3","q3","p3"},
                         nc==4,
                         {"a1","m1","s1","q1","p1",
                          "a2","m2","s2","q2","p2",
                          "a3","m3","s3","q3","p3",
                          "a4","m4","s4","q4","p4"},
                         True,
                         {"a1","m1","s1","q1","p1",
                          "a2","m2","s2","q2","p2",
                          "a3","m3","s3","q3","p3",
                          "a4","m4","s4","q4","p4",
                          "a5","m5","s5","q5","p5"}]},
                  {String,{""}}}];
If[fixp==="",fixp={}];
fixp={If[MemberQ[fixp,"a1"],True,False],
      If[MemberQ[fixp,"m1"],True,False],
      If[MemberQ[fixp,"s1"],True,False],
      If[MemberQ[fixp,"q1"],True,False],
      If[MemberQ[fixp,"p1"],True,False],
      If[MemberQ[fixp,"a2"],True,False],
      If[MemberQ[fixp,"m2"],True,False],
      If[MemberQ[fixp,"s2"],True,False],
      If[MemberQ[fixp,"q2"],True,False],
      If[MemberQ[fixp,"p2"],True,False],
      If[MemberQ[fixp,"a3"],True,False],
      If[MemberQ[fixp,"m3"],True,False],
      If[MemberQ[fixp,"s3"],True,False],
      If[MemberQ[fixp,"q3"],True,False],
      If[MemberQ[fixp,"p3"],True,False],
      If[MemberQ[fixp,"a4"],True,False],
      If[MemberQ[fixp,"m4"],True,False],
      If[MemberQ[fixp,"s4"],True,False],
      If[MemberQ[fixp,"q4"],True,False],
      If[MemberQ[fixp,"p4"],True,False],
      If[MemberQ[fixp,"a5"],True,False],
      If[MemberQ[fixp,"m5"],True,False],
      If[MemberQ[fixp,"s5"],True,False],
      If[MemberQ[fixp,"q5"],True,False],
      If[MemberQ[fixp,"p5"],True,False]};
inip=Flatten[Table[{Sqrt[inipi[[k]]],
                    inipi[[k+1]],
                    inipi[[k+2]],
                    inipi[[k+3]],
                    inipi[[k+4]]},{k,1,5*nc,5}]];
params={};
For[i=1,i<=5*nc,i++,
    If[fixp[[i]],
       ToExpression[StringJoin["a",ToString[i],
                               "=",ToString[InputForm[inip[[i]]]]]],
       params=Append[params,{ToExpression[StringJoin["a",
                                                     ToString[i]]],
                             inip[[i]]}]]];
prparams={};
For[i=1,i<=nc,i++,
    If[Not[fixp[[5*i-4]]],
       prparams=Append[prparams,
          StringJoin["a",ToString[i],"=",
                     ToString[FortranForm[SetPrecision[inip[[5*i-4]]^2,4]]]]]];
    If[Not[fixp[[5*i-3]]],
       prparams=Append[prparams,
          StringJoin["m",ToString[i],"=",
                     ToString[FortranForm[SetPrecision[inip[[5*i-3]],4]]]]]];
    If[Not[fixp[[5*i-2]]],
       prparams=Append[prparams,
          StringJoin["s",ToString[i],"=",
                     ToString[FortranForm[SetPrecision[inip[[5*i-2]],4]]]]]];
    If[Not[fixp[[5*i-1]]],
       prparams=Append[prparams,
          StringJoin["q",ToString[i],"=",
                     ToString[FortranForm[SetPrecision[inip[[5*i-1]],4]]]]]];
    If[Not[fixp[[5*i]]],
       prparams=Append[prparams,
          StringJoin["p",ToString[i],"=",
                     ToString[FortranForm[SetPrecision[inip[[5*i]],4]]]]]]
   ];

funcs=Sum[ToExpression[StringJoin["a",ToString[i]]]^2*
          ggd[x,
              ToExpression[StringJoin["a",ToString[i+1]]],
              ToExpression[StringJoin["a",ToString[i+2]]],
              ToExpression[StringJoin["a",ToString[i+3]]],
              ToExpression[StringJoin["a",ToString[i+4]]]],
            {i,1,5*nc,5}];

(* perform optimization *)
Print["Optimizing the distribution parameters. Please wait..."];
ropt=True;
While[ropt,
Print[prparams];
fit=FindFit[list,
            funcs,
            params,
            x,
            MaxIterations->100];
fit=ToString[InputForm[fit]];
pos1=StringPosition[fit,"->"];
pos2=Join[StringPosition[fit,","],
          StringPosition[fit,"}"]];
fitp=Table[StringTake[fit,{pos1[[k,2]]+2,pos2[[k,1]]-1}],
           {k,1,Length[pos1]}];
fitp=ToExpression[fitp];
pos1={};
For[i=1,i<=5*nc,i++,
    If[fixp[[i]],
       pos1=Append[pos1,inip[[i]]],
       pos1=Append[pos1,fitp[[1]]];
       fitp=Drop[fitp,1]]];
fitp=pos1;
If[ToString[$MessageList]==="{FindFit::cvmit}",
   ropt=InputCheck["Optimization not reached after 100 iterations.
\nContinue optimization? (y/n)",
                   {"Expression"},
                   {{String,{"y","n"}}}];
   If[ropt==="y",
      ropt=True;
      params={};
      For[i=1,i<=5*nc,i++,
      If[Not[fixp[[i]]],
         params=Append[params,{ToExpression[StringJoin["a",
                                                       ToString[i]]],
                               2*fitp[[i]]-inip[[i]]}]]];
         prparams={};
         For[i=1,i<=nc,i++,
             If[Not[fixp[[5*i-4]]],
                prparams=Append[prparams,
             StringJoin["a",ToString[i],"=",
                        ToString[FortranForm[
                        SetPrecision[(2*fitp[[5*i-4]]-inip[[5*i-4]])^2,4]]]]]];
             If[Not[fixp[[5*i-3]]],
                prparams=Append[prparams,
             StringJoin["m",ToString[i],"=",
                        ToString[FortranForm[
                        SetPrecision[2*fitp[[5*i-3]]-inip[[5*i-3]],4]]]]]];
             If[Not[fixp[[5*i-2]]],
                prparams=Append[prparams,
             StringJoin["s",ToString[i],"=",
                        ToString[FortranForm[
                        SetPrecision[2*fitp[[5*i-2]]-inip[[5*i-2]],4]]]]]];
             If[Not[fixp[[5*i-1]]],
                prparams=Append[prparams,
             StringJoin["q",ToString[i],"=",
                        ToString[FortranForm[
                        SetPrecision[2*fitp[[5*i-1]]-inip[[5*i-1]],4]]]]]];
             If[Not[fixp[[5*i]]],
                prparams=Append[prparams,
             StringJoin["p",ToString[i],"=",
                        ToString[FortranForm[
                        SetPrecision[2*fitp[[5*i]]-inip[[5*i]],4]]]]]]
            ];
      inip=fitp,
      ropt=False],
   ropt=False];
$MessageList={};
     ]; (* Close While ropt *)
fitp=Flatten[Table[{fitp[[k]]^2,
                    fitp[[k+1]],
                    fitp[[k+2]],
                    fitp[[k+3]],
                    fitp[[k+4]]},{k,1,5*nc,5}]];
fitpi=fitp;
comp={};
For[i=1,i<=5*nc,i=i+5,
    comp=Append[comp,
    Table[{list0[[k,1]],
           fitp[[i]]*ggd[list0[[k,1]],
                         fitp[[i+1]],fitp[[i+2]],fitp[[i+3]],fitp[[i+4]]]},
          {k,1,k0}]]];
compi=comp;

tot=Table[{list0[[k,1]],Sum[comp[[i,k,2]],{i,1,nc}]},{k,1,k0}];
toti=tot;

res=Table[{list0[[k,1]],100*(tot[[k,2]]-list0[[k,2]])/peak},{k,1,k0}];
errrng=Table[100*err[[k,2]]/peak,{k,1,k0}];
errrng={{First[list0][[1]],Last[list0][[1]]},
        {-1.2*Max[errrng],1.2*Max[errrng]}};
mqd=Table[{list0[[k,1]],(tot[[k,2]]-list0[[k,2]])^2/
                        (err[[k,2]]+0.001*errrng[[2,2]]*peak)^2},{k,1,k0}];
While[First[mqd][[1]]<swin[[1]],
      mqd=Drop[mqd,1]];
While[Last[mqd][[1]]>swin[[2]],
      mqd=Drop[mqd,-1]];
mqd=Sqrt[Sum[mqd[[k,2]],{k,1,Length[mqd]}]/Length[mqd]];
zeros={};
For[i=1,i<=Length[res]-1,i++,
    If[(Not[Sign[res[[i,2]]]==Sign[res[[i+1,2]]]])&&
       (swin[[1]]<=res[[i+1,1]])&&
       (res[[i,1]]<=swin[[2]]),
       zeros=Append[zeros,res[[i,1]]]]];
zeros=Max[Length[zeros]+1,1];
zeros={(ToExpression[StringJoin[
        "x/.FindRoot[CDF[ChiSquareDistribution[",
        ToString[zeros],
        "],x]==0.05,{x,",
        ToString[zeros/(1+1.6*Sqrt[2/zeros])],
        "}]"]])/zeros,
       (ToExpression[StringJoin[
        "x/.FindRoot[CDF[ChiSquareDistribution[",
        ToString[zeros],
        "],x]==0.95,{x,",
        ToString[zeros*(1+1.6*Sqrt[2/zeros])],
        "}]"]])/zeros};
zeros=Re[zeros];

txt={StringJoin["Field in 10E",
                ShortTicks[rng[[1]],10,False,True][[2]],
                " units.\nOptimized distribution parameters:"]};
For[i=1,i<=nc,i++,
    txt=Append[txt,StringJoin[
"a",ToString[i]," = ",ToString[FortranForm[SetPrecision[fitp[[5*i-4]],4]]],
"\nm",ToString[i]," = ",ToString[FortranForm[SetPrecision[fitp[[5*i-3]],4]]],
"\ts",ToString[i]," = ",ToString[FortranForm[SetPrecision[fitp[[5*i-2]],4]]],
"\nq",ToString[i]," = ",ToString[FortranForm[SetPrecision[fitp[[5*i-1]],4]]],
"\tp",ToString[i]," = ",ToString[FortranForm[SetPrecision[fitp[[5*i]],4]]]]
              ]];
txtgr=Table[Graphics[{RGBColor[r[[k]],g[[k]],b[[k]]],
                      Text[txt[[k]],Scaled[coords[[k]]],{-1,1}]}],
            {k,1,nc+1}];
compgr=Join[{ListPlot[list1,PlotJoined->True,
                      PlotStyle->{GrayLevel[0.5],Thickness[0.005]},
                      DisplayFunction->Identity],
             ListPlot[list2,PlotJoined->True,
                      PlotStyle->{GrayLevel[0.5],Thickness[0.005]},
                      DisplayFunction->Identity],
             ListPlot[listm,PlotJoined->True,
                      PlotStyle->{Thickness[0.002]},
                      DisplayFunction->Identity],
             ListPlot[tot,PlotJoined->True,
                      PlotStyle->{RGBColor[0,0,1],Thickness[0.002]},
                      DisplayFunction->Identity]},
            Table[ListPlot[comp[[k]],PlotJoined->True,
                           PlotStyle->{RGBColor[r[[k+1]],g[[k+1]],b[[k+1]]],
                                       Thickness[0.002]},
                           DisplayFunction->Identity],
                  {k,1,nc}]];

txt={Graphics[{RGBColor[0,0,1],
               Text["Residulas (modelled - measured) and",
                    Scaled[{0.02,1}],{-1,1}]}],
     Graphics[{GrayLevel[0.5],
               Text["\nmeasurement errors",
                    Scaled[{0.02,0.97}],{-1,1}]}],
     Graphics[{GrayLevel[0],
               Text[
"\nin % of the maximum value of\nthe coercivity distribution.\n
\nDashed lines delimitate the interval
\nconsidered for the component analysis.",
                    Scaled[{0.02,0.84}],{-1,1}]}],
     Graphics[{GrayLevel[0],
               Text[StringJoin["\nChiSquare/#points: ",
                    ToString[FortranForm[SetPrecision[mqd,2]]],
                               "\nConfidence limits: [",
                    ToString[FortranForm[SetPrecision[zeros[[1]],2]]],
                               ",",
                    ToString[FortranForm[SetPrecision[zeros[[2]],2]]],
                    "]"],
                    Scaled[{0.02,0.3}],{-1,1}]}]};

compgre={ListPlot[Table[{err[[k,1]],100*err[[k,2]]/peak},{k,1,k0}],
                  PlotJoined->True,
                  PlotStyle->{GrayLevel[0.5],Thickness[0.005]},
                  DisplayFunction->Identity],
         ListPlot[Table[{err[[k,1]],-100*err[[k,2]]/peak},{k,1,k0}],
                  PlotJoined->True,
                  PlotStyle->{GrayLevel[0.5],Thickness[0.005]},
                  DisplayFunction->Identity],
         ListPlot[res,PlotJoined->True,
                  PlotStyle->{RGBColor[0,0,1],Thickness[0.002]},
                  DisplayFunction->Identity],
         ListPlot[{{swin[[1]],errrng[[2,1]]},{swin[[1]],errrng[[2,2]]}},
                  PlotJoined->True,
                  PlotStyle->{GrayLevel[0],Thickness[0.002],
                              Dashing[{0.02,0.01}]},
                  DisplayFunction->Identity],
         ListPlot[{{swin[[2]],errrng[[2,1]]},{swin[[2]],errrng[[2,2]]}},
                  PlotJoined->True,
                  PlotStyle->{GrayLevel[0],Thickness[0.002],
                              Dashing[{0.02,0.01}]},
                  DisplayFunction->Identity]};

Show[GraphicsArray[
     {{Show[compgr,
            PlotRange->rng,
            AspectRatio->0.6,
            Axes->True,AxesOrigin->{rng[[1,1]],0},
            Ticks->{ShortTicks[rng[[1]],5,False,True][[1]],
                    ShortTicks[rng[[2]],5,True,False][[1]]},
            DisplayFunction->Identity],
       Show[txtgr,
            AspectRatio->0.6,
            Axes->False,
            DisplayFunction->Identity]},
      {Show[compgre,
            PlotRange->errrng,
            AspectRatio->0.3,
            Axes->True,AxesOrigin->{errrng[[1,1]],0},
            Ticks->{ShortTicks[errrng[[1]],5,False,True][[1]],
                    ShortTicks[errrng[[2]],3,True,False][[1]]},
            PlotRegion->{{0,1},{0.5,1}},
            DisplayFunction->Identity],
       Show[txt,
            AspectRatio->0.3,
            Axes->False,
            PlotRegion->{{0,1},{0.5,1}},
            DisplayFunction->Identity]}}],
     ImageSize->1200,GraphicsSpacing->0,
     PlotRegion->{{-0.025,1.025},{-0.025,1.025}},
     DisplayFunction->$DisplayFunction];

Which[mqd>zeros[[2]],
      Print["Model and data are significantly different. Refine your model."],
      mqd<zeros[[1]],
      Print["Not all model parameters are significant.
You may reduce the number of parameters."],
      True,
      Print["Model and data are compatible.
You may accept this component analysis."]];

rfit=InputCheck["Accept this component analysis? (y/n)",
                {"Expression"},
                {{String,{"y","n"}}}];
If[rfit==="y",rfit=False,rfit=True;count++]
     ]; (* Close rfit *)

(* Perform a systematic search for local minima in the residuals *)
syss=InputCheck["Perform an automatic variation of the contribution of one
component?\n
\nEnter the number of this component or type \"n\".",
                {"Expression","Expression"},
                {{Integer,{1,nc}},{String,{"n"}}}];
If[syss==="n",
loop=False,
Print[StringJoin["Perform an automatic variation of the contribution
of component #",ToString[syss],":
\nThis process takes several minutes. Please wait..."]];
amp=fitp[[5*syss-4]];
totm=Sum[fitp[[5*i-4]],{i,1,nc}];

Print[StringJoin["Decreasing contribution of component #",
                 ToString[syss],"..."]];
search=True;
Off[FindFit::cvmit];
fitv=fitp;
contrs={};
resvar={};
While[amp>0,
fitv=Flatten[Table[{Sqrt[fitv[[k]]],
                    fitv[[k+1]],
                    fitv[[k+2]],
                    fitv[[k+3]],
                    fitv[[k+4]]},{k,1,5*nc,5}]];
parav={};
For[i=1,i<=5*nc,i++,
    Which[i==5*syss-4,
          ToExpression[StringJoin["a",ToString[i],
                                  "=",ToString[InputForm[Sqrt[amp]]]]],
          fixp[[i]],
          ToExpression[StringJoin["a",ToString[i],
                                  "=",ToString[InputForm[fitv[[i]]]]]],
          True,
          parav=Append[parav,{ToExpression[StringJoin["a",
                                                        ToString[i]]],
                                fitv[[i]]}]]];
fit=FindFit[list,
            funcs,
            parav,
            x,
            MaxIterations->200];
fit=ToString[InputForm[fit]];
pos1=StringPosition[fit,"->"];
pos2=Join[StringPosition[fit,","],
          StringPosition[fit,"}"]];
fit=Table[StringTake[fit,{pos1[[k,2]]+2,pos2[[k,1]]-1}],
           {k,1,Length[pos1]}];
fit=ToExpression[fit];
pos1={};
For[i=1,i<=5*nc,i++,
    Which[i==5*syss-4,
          pos1=Append[pos1,Sqrt[amp]],
          fixp[[i]],
          pos1=Append[pos1,fitv[[i]]],
          True,
          pos1=Append[pos1,fit[[1]]];
          fit=Drop[fit,1]]];
fitv=pos1;
fitv=Flatten[Table[{fitv[[k]]^2,
                    fitv[[k+1]],
                    fitv[[k+2]],
                    fitv[[k+3]],
                    fitv[[k+4]]},{k,1,5*nc,5}]];

comp={};
For[i=1,i<=5*nc,i=i+5,
    comp=Append[comp,
    Table[{list0[[k,1]],
           fitp[[i]]*ggd[list0[[k,1]],
                         fitv[[i+1]],fitv[[i+2]],fitv[[i+3]],fitv[[i+4]]]},
          {k,1,k0}]]];
tot=Table[{list0[[k,1]],Sum[comp[[i,k,2]],{i,1,nc}]},{k,1,k0}];
mqd=Table[{list0[[k,1]],(tot[[k,2]]-list0[[k,2]])^2/
                        (err[[k,2]]+0.001*errrng[[2,2]]*peak)^2},{k,1,k0}];
While[First[mqd][[1]]<swin[[1]],
      mqd=Drop[mqd,1]];
While[Last[mqd][[1]]>swin[[2]],
      mqd=Drop[mqd,-1]];
mqd=Sqrt[Sum[mqd[[k,2]],{k,1,Length[mqd]}]/Length[mqd]];

contrs=Prepend[contrs,fitv];
resvar=Prepend[resvar,{amp,mqd}];
amp=amp-totm/100
     ]; (* Close While amp>0 *)
redvar={Last[resvar]};

Print[StringJoin["Increasing contribution of component #",
                 ToString[syss],"..."]];
fitv=fitp;
amp=fitp[[5*syss-4]];
amp=amp+totm/100;
While[amp<totm,
fitv=Flatten[Table[{Sqrt[fitv[[k]]],
                    fitv[[k+1]],
                    fitv[[k+2]],
                    fitv[[k+3]],
                    fitv[[k+4]]},{k,1,5*nc,5}]];
parav={};
For[i=1,i<=5*nc,i++,
    Which[fixp[[i]],
          ToExpression[StringJoin["a",ToString[i],
                                  "=",ToString[InputForm[fitv[[i]]]]]],
          i==5*syss-4,
          ToExpression[StringJoin["a",ToString[i],
                                  "=",ToString[InputForm[Sqrt[amp]]]]],
          True,
          parav=Append[parav,{ToExpression[StringJoin["a",
                                                      ToString[i]]],
                                fitv[[i]]}]]];
fit=FindFit[list,
            funcs,
            parav,
            x,
            MaxIterations->200];
fit=ToString[InputForm[fit]];
pos1=StringPosition[fit,"->"];
pos2=Join[StringPosition[fit,","],
          StringPosition[fit,"}"]];
fit=Table[StringTake[fit,{pos1[[k,2]]+2,pos2[[k,1]]-1}],
           {k,1,Length[pos1]}];
fit=ToExpression[fit];
pos1={};
For[i=1,i<=5*nc,i++,
    Which[fixp[[i]],
          pos1=Append[pos1,fitv[[i]]],
          i==5*syss-4,
          pos1=Append[pos1,Sqrt[amp]],
          True,
          pos1=Append[pos1,fit[[1]]];
          fit=Drop[fit,1]]];
fitv=pos1;
fitv=Flatten[Table[{fitv[[k]]^2,
                    fitv[[k+1]],
                    fitv[[k+2]],
                    fitv[[k+3]],
                    fitv[[k+4]]},{k,1,5*nc,5}]];

comp={};
For[i=1,i<=5*nc,i=i+5,
    comp=Append[comp,
    Table[{list0[[k,1]],
           fitp[[i]]*ggd[list0[[k,1]],
                         fitv[[i+1]],fitv[[i+2]],fitv[[i+3]],fitv[[i+4]]]},
          {k,1,k0}]]];
tot=Table[{list0[[k,1]],Sum[comp[[i,k,2]],{i,1,nc}]},{k,1,k0}];
mqd=Table[{list0[[k,1]],(tot[[k,2]]-list0[[k,2]])^2/
                        (err[[k,2]]+0.001*errrng[[2,2]]*peak)^2},{k,1,k0}];
While[First[mqd][[1]]<swin[[1]],
      mqd=Drop[mqd,1]];
While[Last[mqd][[1]]>swin[[2]],
      mqd=Drop[mqd,-1]];
mqd=Sqrt[Sum[mqd[[k,2]],{k,1,Length[mqd]}]/Length[mqd]];

contrs=Append[contrs,fitv];
resvar=Append[resvar,{amp,mqd}];
amp=amp+totm/100;
     ]; (* Close While amp<totm *)
On[FindFit::cvmit];

redvar={{redvar[[1,1]],Log[10,redvar[[1,2]]]}};
resvar=Table[{resvar[[k,1]],Log[10,resvar[[k,2]]]},{k,1,Length[resvar]}];
resvar10={};
For[i=1,i<=Length[resvar],i++,
    If[IntegerQ[i/10],
       resvar10=Append[resvar10,resvar[[i]]]]];

errrng=Table[resvar[[k,2]],{k,1,Length[resvar]}];
errrng={{resvar[[1,1]],Last[resvar][[1]]},{Min[errrng],Max[errrng]}};

Print[StringJoin["log(chi square) as a function of the contribution
of component #",ToString[syss],
"\n(Every 10th point in gray, red point is the starting solution)"]];
Show[ListPlot[resvar,PlotJoined->True,
              PlotStyle->{GrayLevel[0],Thickness[0.002]},
              DisplayFunction->Identity],
     ListPlot[resvar,PlotStyle->{GrayLevel[0],PointSize[0.008]},
              DisplayFunction->Identity],
     ListPlot[resvar10,PlotStyle->{GrayLevel[0.5],PointSize[0.008]},
              DisplayFunction->Identity],
     ListPlot[redvar,PlotStyle->{RGBColor[1,0,0],PointSize[0.008]},
              DisplayFunction->Identity],
     AspectRatio->0.6,
     Axes->True,AxesOrigin->{errrng[[1,1]],errrng[[2,1]]},
     ImageSize->{600,400},
     PlotRange->All,
     DisplayFunction->$DisplayFunction];

selpar=InputCheck["Accept the original component analysis
(red point)? (y/n).",
                  {"Expression"},
                  {{String,{"y","n"}}}];
If[selpar==="y",
   loop=False,
   loop=True;
   count++]

  ];  (* Close If syss *)

     ]; (* Close While loop *)

(* Calculate statistical parameters of the distribution *)
Print["Calculating statistical parameters of the distributions..."];
stat=Table[
     NIntegrate[ggd[x,fitp[[k+1]],fitp[[k+2]],fitp[[k+3]],fitp[[k+4]]]*x,
                {x,-Infinity,Infinity}],
           {k,1,5*nc,5}];
stat=Table[{stat[[Floor[k/5]+1]],
     NIntegrate[ggd[x,fitp[[k+1]],fitp[[k+2]],fitp[[k+3]],fitp[[k+4]]]*
                (x-stat[[Floor[k/5]+1]])^2,
                {x,-Infinity,Infinity}],
     NIntegrate[ggd[x,fitp[[k+1]],fitp[[k+2]],fitp[[k+3]],fitp[[k+4]]]*
                (x-stat[[Floor[k/5]+1]])^3,
                {x,-Infinity,Infinity}],
     NIntegrate[ggd[x,fitp[[k+1]],fitp[[k+2]],fitp[[k+3]],fitp[[k+4]]]*
                (x-stat[[Floor[k/5]+1]])^4,
                {x,-Infinity,Infinity}]},
           {k,1,5*nc,5}];
stat=Table[{stat[[k,1]],
            Sqrt[stat[[k,2]]],
            stat[[k,3]]/stat[[k,2]]^1.5,
            stat[[k,4]]/stat[[k,2]]^2-3},
           {k,1,nc}];

rerror=InputCheck["Perform an error estimation of the
distribution parameters\n?
\nType \"n\", or enter the number of error simulations to perform
(minimum: 16, recommanded: 64)",
                  {"Expression","Expression"},
                  {{String,{"n"}},{Integer,{16,Infinity}}}];
If[Not[rerror==="n"],
   ne=rerror];
If[rerror==="n",

(* Print fitting results (no errors) *)
Print[TableForm[{Table[
StringJoin[
"Parameters of component #",ToString[k],":
\na = ",ToString[FortranForm[SetPrecision[fitp[[5*k-4]],4]]],
"\n\[Mu] = ",ToString[FortranForm[SetPrecision[fitp[[5*k-3]],4]]],
"\n\[Sigma] = ",ToString[FortranForm[SetPrecision[fitp[[5*k-2]],4]]],
"\nq = ",ToString[FortranForm[SetPrecision[fitp[[5*k-1]],4]]],
"\np = ",ToString[FortranForm[SetPrecision[fitp[[5*k]],4]]],
"\nMDF = ",ToString[FortranForm[SetPrecision[fitp[[5*k-3]],4]]],
"\nmean = ",ToString[FortranForm[SetPrecision[stat[[k,1]],4]]],
"\nDP = ",ToString[FortranForm[SetPrecision[stat[[k,2]],4]]],
"\nskewness = ",ToString[FortranForm[SetPrecision[stat[[k,3]],4]]],
"\nkurtosis = ",ToString[FortranForm[SetPrecision[stat[[k,4]],4]]]],
                       {k,1,nc}]}]];

(* Plot normalized distributions (no errors) *)
rngn={};
For[i=1,i<=5*nc,i=i+5,
    max=ggd[fitp[[i+1]],
            fitp[[i+1]],fitp[[i+2]],fitp[[i+3]],fitp[[i+4]]];
    rngn=Append[rngn,x/.FindRoot[
         ggd[x,fitp[[i+1]],fitp[[i+2]],fitp[[i+3]],fitp[[i+4]]]==0.002*max,
                {x,{fitp[[i+1]]-4*fitp[[i+2]],fitp[[i+1]]-0.5*fitp[[i+2]]}}]]];
For[i=1,i<=5*nc,i=i+5,
    max=ggd[fitp[[i+1]],
            fitp[[i+1]],fitp[[i+2]],fitp[[i+3]],fitp[[i+4]]];
    rngn=Append[rngn,x/.FindRoot[
         ggd[x,fitp[[i+1]],fitp[[i+2]],fitp[[i+3]],fitp[[i+4]]]==0.002*max,
                {x,{fitp[[i+1]]+0.5*fitp[[i+2]],fitp[[i+1]]+4*fitp[[i+2]]}}]]];
rngn={Min[Take[rngn,{1,nc}]],Max[Take[rngn,{nc+1,2*nc}]]};
d=Min[Table[fitp[[k]],{k,3,5*nc,5}]]/100;
n0=(rngn[[2]]-rngn[[1]])/d;
rngn=Table[rngn[[1]]+k*d,{k,1,n0}];
n0=Length[rngn];
compn={};
For[i=1,i<=5*nc,i=i+5,
    compn=Append[compn,
    Table[{rngn[[k]],
           ggd[rngn[[k]],fitp[[i+1]],fitp[[i+2]],fitp[[i+3]],fitp[[i+4]]]},
          {k,1,n0}]]];
rngn={{First[rngn],Last[rngn]},
      {0,Max[Join[Table[Table[compn[[i,k,2]],{k,1,n0}],{i,1,nc}]]]}};
Print["Normalized components:"];
txt={StringJoin["Field in 10E",
                ShortTicks[rngn[[1]],10,False,True][[2]],
                " units.\nDistribution parameters:"]};
For[i=1,i<=nc,i++,
    txt=Append[txt,StringJoin[
"MDF",ToString[i]," = ",ToString[FortranForm[SetPrecision[fitp[[5*i-3]],4]]],
"\tDP",ToString[i]," = ",ToString[FortranForm[SetPrecision[stat[[i,2]],4]]],
"\nsk",ToString[i]," = ",ToString[FortranForm[SetPrecision[stat[[i,3]],4]]],
"\tku",ToString[i]," = ",ToString[FortranForm[SetPrecision[stat[[i,4]],4]]]]
              ]];
txtgr=Table[Graphics[{RGBColor[r[[k]],g[[k]],b[[k]]],
                      Text[txt[[k]],Scaled[coords[[k]]],{-1,1}]}],
            {k,1,nc+1}];
compgr=Table[ListPlot[compn[[k]],PlotJoined->True,
                      PlotStyle->{RGBColor[r[[k+1]],g[[k+1]],b[[k+1]]],
                                  Thickness[0.002]},
                      DisplayFunction->Identity],
             {k,1,nc}];
Show[GraphicsArray[
     {{Show[compgr,
            PlotRange->rngn,
            AspectRatio->0.6,
            Axes->True,AxesOrigin->{rngn[[1,1]],0},
            Ticks->{ShortTicks[rngn[[1]],5,False,True][[1]],
                    ShortTicks[rngn[[2]],5,True,False][[1]]},
            DisplayFunction->Identity],
       Show[txtgr,
            AspectRatio->0.6,
            Axes->False,
            DisplayFunction->Identity]}}],
     ImageSize->1200,GraphicsSpacing->0,
     PlotRegion->{{-0.025,1.025},{-0.025,1.025}},
     DisplayFunction->$DisplayFunction];

(* Print data to a log file (no errors) *)
pr=InputCheck["Print results to the log file \"components.txt\"? (y/n)",
                {"Expression"},
                {{String,{"y","n"}}}];
If[pr==="y",
   Print["Printing results to:  components.txt"];
   pr=Import[logfile,"Text"];
   pr=StringJoin[pr,"\n",name];
   For[k=1,k<=nc,k++,
       pr=StringJoin[pr,
       "\t",
       ToString[FortranForm[SetPrecision[fitp[[5*k-4]],4]]],"\t","\t",
       ToString[FortranForm[SetPrecision[fitp[[5*k-3]],4]]],"\t","\t",
       ToString[FortranForm[SetPrecision[fitp[[5*k-2]],4]]],"\t","\t",
       ToString[FortranForm[SetPrecision[fitp[[5*k-1]],4]]],"\t","\t",
       ToString[FortranForm[SetPrecision[fitp[[5*k]],4]]],"\t","\t",
       ToString[FortranForm[SetPrecision[stat[[k,2]],4]]],"\t","\t",
       ToString[FortranForm[SetPrecision[stat[[k,3]],4]]],"\t","\t",
       ToString[FortranForm[SetPrecision[stat[[k,4]],4]]],"\t"]];
   Export[logfile,pr,"Text"];];
pr=InputCheck["Save the coercivity distributions to a file? (y/n)",
                {"Expression"},
                {{String,{"y","n"}}}];
If[pr==="y",
   pr=StringJoin["Saving the coercivity distributions to ",
                    nfile,".comp :
\n Column #1: magnetic field,"];
   For[i=1,i<=nc,i++,
       pr=StringJoin[pr,
                     "\n Column #",
                     ToString[i+1],
                     ": component #",
                     ToString[i]]];
   Print[pr];
   comps=StringJoin["field",
                    Table[StringJoin["\tcomp. #",ToString[k]],{k,1,nc}],
                    "\n"];
   For[k=1,k<=n0,k++,
       comps=StringJoin[comps,
       Join[{ToString[FortranForm[SetPrecision[compn[[1,k,1]],6]]]},
            Table[StringJoin["\t",ToString[FortranForm[
                  SetPrecision[fitp[[5*i-4]]*compn[[i,k,2]],6]]]],
                  {i,1,nc}]]];
       comps=StringJoin[comps,"\n"]];
   Export[StringJoin[outdir,"\\",nfile,".comp"],comps,"Text"];
   ];
Print["END"],

(* Perform an error estimation of the distribution parameters *)
Clear["a1"];Clear["a2"];Clear["a3"];Clear["a4"];Clear["a5"];
Clear["a6"];Clear["a7"];Clear["a8"];Clear["a9"];Clear["a10"];
Clear["a11"];Clear["a12"];Clear["a13"];Clear["a14"];Clear["a15"];
Clear["a16"];Clear["a17"];Clear["a18"];Clear["a19"];Clear["a20"];
comp=compi;tot=toti;
Off[FindFit::cvmit];
Print[StringJoin["Perform an error estimation of the distribution parameters
with ",ToString[ne]," error simulations. Accuracy of the error estimation: ",
ToString[FortranForm[SetPrecision[100/Sqrt[ne],2]]],
"%\nThis process takes several minutes time. Please wait..."]];
fitp=Flatten[Table[{Sqrt[fitpi[[k]]],
                    fitpi[[k+1]],
                    fitpi[[k+2]],
                    fitpi[[k+3]],
                    fitpi[[k+4]]},{k,1,5*nc,5}]];
params={};
For[i=1,i<=5*nc,i++,
    If[fixp[[i]],
       ToExpression[StringJoin["a",ToString[i],
                               "=",ToString[InputForm[fitp[[i]]]]]],
       params=Append[params,{ToExpression[StringJoin["a",
                                                     ToString[i]]],
                             fitp[[i]]}]]];
paralst={};
For[j=1,j<=ne,j++,
    nerr=ToExpression[StringJoin[
         "RandomArray[NormalDistribution[0,1],",
         ToString[InputForm[Length[list]]],"]"]];
    nerr=Table[nerr[[k]]*0.5*merr[[k,2]],{k,1,Length[list]}];
    nlist=Table[{list[[k,1]],list[[k,2]]+nerr[[k]]},{k,1,Length[list]}];
    fit=FindFit[nlist,
                funcs,
                params,
                x,
                MaxIterations->100];
    fit=ToString[InputForm[fit]];
    pos1=StringPosition[fit,"->"];
    pos2=Join[StringPosition[fit,","],
              StringPosition[fit,"}"]];
    fitn=Table[StringTake[fit,{pos1[[k,2]]+2,pos2[[k,1]]-1}],
               {k,1,Length[pos1]}];
    fitn=ToExpression[fitn];
    pos1={};
    For[i=1,i<=5*nc,i++,
        If[fixp[[i]],
           pos1=Append[pos1,fitp[[i]]],
           pos1=Append[pos1,fitn[[1]]];
           fitn=Drop[fitn,1]]];
    fitn=pos1;
    fitn=Flatten[Table[{fitn[[k]]^2,
                        fitn[[k+1]],
                        fitn[[k+2]],
                        fitn[[k+3]],
                        fitn[[k+4]]},{k,1,5*nc,5}]];
    paralst=Append[paralst,fitn];
   ];
means=Table[Sum[paralst[[k,j]],{k,1,ne}]/ne,{j,1,5*nc}];
vars=Table[errmax/0.01*
           Sqrt[Sum[(paralst[[k,j]]-means[[j]])^2,{k,1,ne}]/ne],
           {j,1,5*nc}];
On[FindFit::cvmit];
fitp=Flatten[Table[{fitp[[k]]^2,
                    fitp[[k+1]],
                    fitp[[k+2]],
                    fitp[[k+3]],
                    fitp[[k+4]]},{k,1,5*nc,5}]];

(* Perform an error estimation of the statistical parameters *)
Print["Error estimation of the statistical parameters. Please wait..."];
morep=Table[Table[
NIntegrate[
ggd[x,paralst[[k,i+1]],paralst[[k,i+2]],paralst[[k,i+3]],paralst[[k,i+4]]]*x,
           {x,-Infinity,Infinity}],
                  {k,1,ne}],
           {i,1,5*nc,5}];

morep=Table[Table[{morep[[Floor[i/5]+1,k]],
NIntegrate[
ggd[x,paralst[[k,i+1]],paralst[[k,i+2]],paralst[[k,i+3]],paralst[[k,i+4]]]*
           (x-morep[[Floor[i/5]+1,k]])^2,
           {x,-Infinity,Infinity}],
NIntegrate[
ggd[x,paralst[[k,i+1]],paralst[[k,i+2]],paralst[[k,i+3]],paralst[[k,i+4]]]*
           (x-morep[[Floor[i/5]+1,k]])^3,
           {x,-Infinity,Infinity}],
NIntegrate[
ggd[x,paralst[[k,i+1]],paralst[[k,i+2]],paralst[[k,i+3]],paralst[[k,i+4]]]*
           (x-morep[[Floor[i/5]+1,k]])^4,
           {x,-Infinity,Infinity}]},
                  {k,1,ne}],
            {i,1,5*nc,5}];

morep=Table[Table[{morep[[i,k,1]],
                   Sqrt[morep[[i,k,2]]],
                   morep[[i,k,3]]/morep[[i,k,2]]^1.5,
                   morep[[i,k,4]]/morep[[i,k,2]]^2-3},
                  {k,1,ne}],
            {i,1,nc}];

morem=Table[Table[Sum[morep[[i,k,l]],{k,1,ne}]/ne,
                  {l,1,4}],
            {i,1,nc}];

morev=Table[Table[errmax/0.01*
                  Sqrt[Sum[(morep[[i,k,l]]-morem[[i,l]])^2,{k,1,ne}]/ne],
                  {l,1,4}],
            {i,1,nc}];

(* Calculate the confidence limits of each component *)
Print["Calculating the confidence limits of the components. Please wait..."];
compl={};
For[i=1,i<=5*nc,i=i+5,
    compl=Append[compl,
    Table[Table[paralst[[j,i]]*
                ggd[list0[[k,1]],
                    paralst[[j,i+1]],paralst[[j,i+2]],
                    paralst[[j,i+3]],paralst[[j,i+4]]],{j,1,ne}],
          {k,1,k0}]]];

compm=Table[Table[Sum[compl[[i,k,j]],{j,1,ne}]/ne,{k,1,k0}],
            {i,1,nc}];
compv=Table[Table[errmax/0.01*
                  Sqrt[Sum[(compl[[i,k,j]]-compm[[i,k]])^2,{j,1,ne}]/ne],
                  {k,1,k0}],
            {i,1,nc}];

comp1=Table[Table[{comp[[i,k,1]],comp[[i,k,2]]-compv[[i,k]]},
                  {k,1,k0}],
            {i,1,nc}];
comp2=Table[Table[{comp[[i,k,1]],comp[[i,k,2]]+compv[[i,k]]},
                  {k,1,k0}],
            {i,1,nc}];

(* Print final results *)
Print[TableForm[{Table[
StringJoin[
"Parameters of component #",ToString[k],":
\na = ",ToString[FortranForm[SetPrecision[fitp[[5*k-4]],4]]],
" \[PlusMinus] ",ToString[FortranForm[SetPrecision[vars[[5*k-4]],2]]],
"\n\[Mu] = ",ToString[FortranForm[SetPrecision[fitp[[5*k-3]],4]]],
" \[PlusMinus] ",ToString[FortranForm[SetPrecision[vars[[5*k-3]],2]]],
"\n\[Sigma] = ",ToString[FortranForm[SetPrecision[fitp[[5*k-2]],4]]],
" \[PlusMinus] ",ToString[FortranForm[SetPrecision[vars[[5*k-2]],2]]],
"\nq = ",ToString[FortranForm[SetPrecision[fitp[[5*k-1]],4]]],
" \[PlusMinus] ",ToString[FortranForm[SetPrecision[vars[[5*k-1]],2]]],
"\np = ",ToString[FortranForm[SetPrecision[fitp[[5*k]],4]]],
" \[PlusMinus] ",ToString[FortranForm[SetPrecision[vars[[5*k]],2]]],
"\nMDF = ",ToString[FortranForm[SetPrecision[fitp[[5*k-3]],4]]],
" \[PlusMinus] ",ToString[FortranForm[SetPrecision[vars[[5*k-3]],2]]],
"\nmean = ",ToString[FortranForm[SetPrecision[stat[[k,1]],4]]],
" \[PlusMinus] ",ToString[FortranForm[SetPrecision[morev[[k,1]],2]]],
"\nDP = ",ToString[FortranForm[SetPrecision[stat[[k,2]],4]]],
" \[PlusMinus] ",ToString[FortranForm[SetPrecision[morev[[k,2]],2]]],
"\nskewness = ",ToString[FortranForm[SetPrecision[stat[[k,3]],4]]],
" \[PlusMinus] ",ToString[FortranForm[SetPrecision[morev[[k,3]],2]]],
"\nkurtosis = ",ToString[FortranForm[SetPrecision[stat[[k,4]],4]]],
" \[PlusMinus] ",ToString[FortranForm[SetPrecision[morev[[k,4]],2]]]],
                       {k,1,nc}]}]];

Print["Result of the component analysis:"];
txt={StringJoin["Field in 10E",
                ShortTicks[rng[[1]],10,False,True][[2]],
                " units.\nTotal magnetization: ",
                ToString[FortranForm[SetPrecision[
                Sum[fitp[[i]],{i,1,5*nc,5}],4]]],
                " \[PlusMinus] ",
                ToString[FortranForm[SetPrecision[
                Sqrt[Sum[vars[[i]]^2,{i,1,5*nc,5}]],2]]]]};
For[i=1,i<=nc,i++,
    txt=Append[txt,StringJoin[
"a",ToString[i]," = ",ToString[FortranForm[SetPrecision[fitp[[5*i-4]],4]]],
"\nMDF",ToString[i]," = ",ToString[FortranForm[SetPrecision[fitp[[5*i-3]],4]]],
"\tDP",ToString[i]," = ",ToString[FortranForm[SetPrecision[stat[[i,2]],4]]],
"\nsk",ToString[i]," = ",ToString[FortranForm[SetPrecision[stat[[i,3]],4]]],
"\tku",ToString[i]," = ",ToString[FortranForm[SetPrecision[stat[[i,4]],4]]]]
              ]];
txtgr=Table[Graphics[{RGBColor[r[[k]],g[[k]],b[[k]]],
                      Text[txt[[k]],Scaled[coords[[k]]],{-1,1}]}],
            {k,1,nc+1}];
compgr=Join[{ListPlot[list1,PlotJoined->True,
                      PlotStyle->{GrayLevel[0.5],Thickness[0.005]},
                      DisplayFunction->Identity],
             ListPlot[list2,PlotJoined->True,
                      PlotStyle->{GrayLevel[0.5],Thickness[0.005]},
                      DisplayFunction->Identity],
             ListPlot[listm,PlotJoined->True,
                      PlotStyle->{Thickness[0.002]},
                      DisplayFunction->Identity],
             ListPlot[tot,PlotJoined->True,
                      PlotStyle->{RGBColor[0,0,1],Thickness[0.002]},
                      DisplayFunction->Identity]},
            Table[ListPlot[comp1[[k]],PlotJoined->True,
                           PlotStyle->{RGBColor[lr[[k]],lg[[k]],lb[[k]]],
                                       Thickness[0.005]},
                           DisplayFunction->Identity],
                  {k,1,nc}],
            Table[ListPlot[comp2[[k]],PlotJoined->True,
                           PlotStyle->{RGBColor[lr[[k]],lg[[k]],lb[[k]]],
                                       Thickness[0.005]},
                           DisplayFunction->Identity],
                  {k,1,nc}],
            Table[ListPlot[comp[[k]],PlotJoined->True,
                           PlotStyle->{RGBColor[r[[k+1]],g[[k+1]],b[[k+1]]],
                                       Thickness[0.002]},
                           DisplayFunction->Identity],
                  {k,1,nc}]];

Show[GraphicsArray[
     {{Show[compgr,
            PlotRange->rng,
            AspectRatio->0.6,
            Axes->True,AxesOrigin->{rng[[1,1]],0},
            Ticks->{ShortTicks[rng[[1]],5,False,True][[1]],
                    ShortTicks[rng[[2]],5,True,False][[1]]},
            DisplayFunction->Identity],
       Show[txtgr,
            AspectRatio->0.6,
            Axes->False,
            DisplayFunction->Identity]}}],
     ImageSize->1200,GraphicsSpacing->0,
     PlotRegion->{{-0.025,1.025},{-0.025,1.025}},
     DisplayFunction->$DisplayFunction];


(* Calculate the confidence limits of each component *)
Print["Calculating the confidence limits of each component. Please wait..."];
rngn={};
For[i=1,i<=5*nc,i=i+5,
    max=ggd[fitp[[i+1]],
            fitp[[i+1]],fitp[[i+2]],fitp[[i+3]],fitp[[i+4]]];
    rngn=Append[rngn,x/.FindRoot[
         ggd[x,fitp[[i+1]],fitp[[i+2]],fitp[[i+3]],fitp[[i+4]]]==0.002*max,
                {x,{fitp[[i+1]]-3*fitp[[i+2]],fitp[[i+1]]-fitp[[i+2]]}}]]];
For[i=1,i<=5*nc,i=i+5,
    max=ggd[fitp[[i+1]],
            fitp[[i+1]],fitp[[i+2]],fitp[[i+3]],fitp[[i+4]]];
    rngn=Append[rngn,x/.FindRoot[
         ggd[x,fitp[[i+1]],fitp[[i+2]],fitp[[i+3]],fitp[[i+4]]]==0.002*max,
                {x,{fitp[[i+1]]+fitp[[i+2]],fitp[[i+1]]+3*fitp[[i+2]]}}]]];
rngn={Min[Take[rngn,{1,nc}]],Max[Take[rngn,{nc+1,2*nc}]]};
d=Min[Table[fitp[[k]],{k,3,5*nc,5}]]/100;
n0=(rngn[[2]]-rngn[[1]])/d;
rngn=Table[rngn[[1]]+k*d,{k,1,n0}];
n0=Length[rngn];
compn={};
For[i=1,i<=5*nc,i=i+5,
    compn=Append[compn,
    Table[{rngn[[k]],
           ggd[rngn[[k]],fitp[[i+1]],fitp[[i+2]],fitp[[i+3]],fitp[[i+4]]]},
          {k,1,n0}]]];
compl={};
For[i=1,i<=5*nc,i=i+5,
    compl=Append[compl,
    Table[Table[ggd[rngn[[k]],
                    paralst[[j,i+1]],paralst[[j,i+2]],
                    paralst[[j,i+3]],paralst[[j,i+4]]],{j,1,ne}],
          {k,1,n0}]]];
compm=Table[Table[Sum[compl[[i,k,j]],{j,1,ne}]/ne,{k,1,n0}],
            {i,1,nc}];
compv=Table[Table[errmax/0.01*
                  Sqrt[Sum[(compl[[i,k,j]]-compm[[i,k]])^2,{j,1,ne}]/ne],
                  {k,1,n0}],
            {i,1,nc}];
comp1=Table[Table[{compn[[i,k,1]],compn[[i,k,2]]-compv[[i,k]]},
                  {k,1,n0}],
            {i,1,nc}];
comp2=Table[Table[{compn[[i,k,1]],compn[[i,k,2]]+compv[[i,k]]},
                  {k,1,n0}],
            {i,1,nc}];
rngn={{First[rngn],Last[rngn]},
      {0,Max[Join[Table[Table[compn[[i,k,2]],{k,1,n0}],{i,1,nc}],
                  Table[Table[comp1[[i,k,2]],{k,1,n0}],{i,1,nc}],
                  Table[Table[comp2[[i,k,2]],{k,1,n0}],{i,1,nc}]]]}};

Print["Normalized components with confidence limits:"];
txt={StringJoin["Field in 10E",
                ShortTicks[rngn[[1]],10,False,True][[2]],
                " units.\nDistribution parameters:"]};
For[i=1,i<=nc,i++,
    txt=Append[txt,StringJoin[
"\nMDF",ToString[i]," = ",ToString[FortranForm[SetPrecision[fitp[[5*i-3]],4]]],
"\tDP",ToString[i]," = ",ToString[FortranForm[SetPrecision[fitp[[5*i-2]],4]]],
"\nsk",ToString[i]," = ",ToString[FortranForm[SetPrecision[stat[[i,3]],4]]],
"\tku",ToString[i]," = ",ToString[FortranForm[SetPrecision[stat[[i,4]],4]]]]
              ]];
txtgr=Table[Graphics[{RGBColor[r[[k]],g[[k]],b[[k]]],
                      Text[txt[[k]],Scaled[coords[[k]]],{-1,1}]}],
            {k,1,nc+1}];
compgr=Join[Table[ListPlot[comp1[[k]],PlotJoined->True,
                           PlotStyle->{RGBColor[lr[[k]],lg[[k]],lb[[k]]],
                                       Thickness[0.005]},
                           DisplayFunction->Identity],
                  {k,1,nc}],
            Table[ListPlot[comp2[[k]],PlotJoined->True,
                           PlotStyle->{RGBColor[lr[[k]],lg[[k]],lb[[k]]],
                                       Thickness[0.005]},
                           DisplayFunction->Identity],
                  {k,1,nc}],
            Table[ListPlot[compn[[k]],PlotJoined->True,
                           PlotStyle->{RGBColor[r[[k+1]],g[[k+1]],b[[k+1]]],
                                       Thickness[0.002]},
                           DisplayFunction->Identity],
                  {k,1,nc}]];
Show[GraphicsArray[
     {{Show[compgr,
            PlotRange->rngn,
            AspectRatio->0.6,
            Axes->True,AxesOrigin->{rngn[[1,1]],0},
            Ticks->{ShortTicks[rngn[[1]],5,False,True][[1]],
                    ShortTicks[rngn[[2]],5,True,False][[1]]},
            DisplayFunction->Identity],
       Show[txtgr,
            AspectRatio->0.6,
            Axes->False,
            DisplayFunction->Identity]}}],
     ImageSize->1200,GraphicsSpacing->0,
     PlotRegion->{{-0.025,1.025},{-0.025,1.025}},
     DisplayFunction->$DisplayFunction];


(* Print data to a log file *)
pr=InputCheck["Print results to the log file \"components.txt\"? (y/n)",
                {"Expression"},
                {{String,{"y","n"}}}];
If[pr==="y",
   Print["Printing results to:  components.txt"];
   pr=Import[logfile,"Text"];
   pr=StringJoin[pr,"\n",name];
   For[k=1,k<=nc,k++,
       pr=StringJoin[pr,
       "\t",
       ToString[FortranForm[SetPrecision[fitp[[5*k-4]],4]]],"\t",
       ToString[FortranForm[SetPrecision[vars[[5*k-4]],4]]],"\t",
       ToString[FortranForm[SetPrecision[fitp[[5*k-3]],4]]],"\t",
       ToString[FortranForm[SetPrecision[vars[[5*k-3]],4]]],"\t",
       ToString[FortranForm[SetPrecision[fitp[[5*k-2]],4]]],"\t",
       ToString[FortranForm[SetPrecision[vars[[5*k-2]],4]]],"\t",
       ToString[FortranForm[SetPrecision[fitp[[5*k-1]],4]]],"\t",
       ToString[FortranForm[SetPrecision[vars[[5*k-1]],4]]],"\t",
       ToString[FortranForm[SetPrecision[fitp[[5*k]],4]]],"\t",
       ToString[FortranForm[SetPrecision[vars[[5*k]],4]]],"\t",
       ToString[FortranForm[SetPrecision[stat[[k,2]],4]]],"\t",
       ToString[FortranForm[SetPrecision[morev[[k,2]],4]]],"\t",
       ToString[FortranForm[SetPrecision[stat[[k,3]],4]]],"\t",
       ToString[FortranForm[SetPrecision[morev[[k,3]],4]]],"\t",
       ToString[FortranForm[SetPrecision[stat[[k,4]],4]]],"\t",
       ToString[FortranForm[SetPrecision[morev[[k,4]],4]]]]];
   Export[logfile,pr,"Text"];];
pr=InputCheck["Save the coercivity distributions to a file? (y/n)",
                {"Expression"},
                {{String,{"y","n"}}}];
If[pr==="y",
   Print["Preparing data to an export format. Please wait..."];
   rngn={};
   For[i=1,i<=5*nc,i=i+5,
       max=ggd[fitp[[i+1]],
               fitp[[i+1]],fitp[[i+2]],fitp[[i+3]],fitp[[i+4]]];
       rngn=Append[rngn,x/.FindRoot[
            ggd[x,fitp[[i+1]],fitp[[i+2]],fitp[[i+3]],fitp[[i+4]]]==0.002*max,
                   {x,{fitp[[i+1]]-3*fitp[[i+2]],fitp[[i+1]]-fitp[[i+2]]}}]]];
   For[i=1,i<=5*nc,i=i+5,
       max=ggd[fitp[[i+1]],
               fitp[[i+1]],fitp[[i+2]],fitp[[i+3]],fitp[[i+4]]];
       rngn=Append[rngn,x/.FindRoot[
            ggd[x,fitp[[i+1]],fitp[[i+2]],fitp[[i+3]],fitp[[i+4]]]==0.002*max,
                   {x,{fitp[[i+1]]+fitp[[i+2]],fitp[[i+1]]+3*fitp[[i+2]]}}]]];
   rngn={Min[Take[rngn,{1,nc}]],Max[Take[rngn,{nc+1,2*nc}]]};
   d=Min[Table[fitp[[k]],{k,3,5*nc,5}]]/100;
   n0=(rngn[[2]]-rngn[[1]])/d;
   rngn=Table[rngn[[1]]+k*d,{k,1,n0}];
   n0=Length[rngn];
   compn={};
   For[i=1,i<=5*nc,i=i+5,
       compn=Append[compn,
       Table[{rngn[[k]],
              fitp[[i]]*
              ggd[rngn[[k]],fitp[[i+1]],fitp[[i+2]],fitp[[i+3]],fitp[[i+4]]]},
             {k,1,n0}]]];
   compl={};
   For[i=1,i<=5*nc,i=i+5,
    compl=Append[compl,
       Table[Table[paralst[[j,i]]*
                   ggd[rngn[[k]],
                       paralst[[j,i+1]],paralst[[j,i+2]],
                       paralst[[j,i+3]],paralst[[j,i+4]]],{j,1,ne}],
             {k,1,n0}]]];
   compm=Table[Table[Sum[compl[[i,k,j]],{j,1,ne}]/ne,{k,1,n0}],
               {i,1,nc}];
   compv=Table[Table[errmax/0.01*
                     Sqrt[Sum[(compl[[i,k,j]]-compm[[i,k]])^2,{j,1,ne}]/ne],
                     {k,1,n0}],
               {i,1,nc}];
   pr=StringJoin["Saving the coercivity distributions to ",
                    nfile,".comp :
\n Column #1: magnetic field,"];
   For[i=1,i<=nc,i++,
       pr=StringJoin[pr,
                     "\n Column #",
                     ToString[2*i],
                     ": component #",
                     ToString[i],
                     "    Column #",
                     ToString[2*i+1],
                     ": error of component #",
                     ToString[i]]];
   Print[pr];
   comps=StringJoin["field",
                    Table[StringJoin["\tcomp. #",
                                     ToString[k],
                                     "\tabs. error"],{k,1,nc}],
                    "\n"];
   For[k=1,k<=n0,k++,
       comps=StringJoin[comps,
       Join[{ToString[FortranForm[SetPrecision[compn[[1,k,1]],6]]]},
            Table[StringJoin["\t",ToString[FortranForm[
                             SetPrecision[compn[[i,k,2]],6]]],
                             "\t",ToString[FortranForm[
                             SetPrecision[compv[[i,k]],3]]]],
                  {i,1,nc}]]];
       comps=StringJoin[comps,"\n"]];
   Export[StringJoin[outdir,"\\",nfile,".comp"],comps,"Text"];
   ];
Print["END"]

  ] (* Close If rerror *)
]

End[]

EndPackage[]